@@ -1,5 +1,5 @@
-#ifndef __AUTH_CACHE_H
-#define __AUTH_CACHE_H
+#ifndef AUTH_CACHE_H
+#define AUTH_CACHE_H
 
 struct auth_cache_node {
 	struct auth_cache_node *prev, *next;
@@ -22,8 +22,10 @@ char *auth_cache_parse_key(pool_t pool, const char *query);
 
 /* Create a new cache. max_size specifies the maximum amount of memory in
    bytes to use for cache (it's not fully exact). ttl_secs specifies time to
-   live for cache record, requests older than that are not used. */
-struct auth_cache *auth_cache_new(size_t max_size, unsigned int ttl_secs);
+   live for cache record, requests older than that are not used.
+   neg_ttl_secs specifies the TTL for negative entries. */
+struct auth_cache *auth_cache_new(size_t max_size, unsigned int ttl_secs,
+				  unsigned int neg_ttl_secs);
 void auth_cache_free(struct auth_cache **cache);
 
 /* Clear the cache. */
@@ -36,8 +38,13 @@ const char *
 auth_cache_lookup(struct auth_cache *cache, const struct auth_request *request,
 		  const char *key, struct auth_cache_node **node_r,
 		  bool *expired_r);
-/* Insert key => value into cache. */
+/* Insert key => value into cache. "" value means negative cache entry. */
 void auth_cache_insert(struct auth_cache *cache, struct auth_request *request,
 		       const char *key, const char *value, bool last_success);
 
+/* Remove key from cache */
+void auth_cache_remove(struct auth_cache *cache,
+		       const struct auth_request *request,
+		       const char *key);
+
 #endif
