@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2005 Timo Sirainen */
+/* Copyright (c) 2002-2008 Dovecot authors, see the included COPYING file */
 
 #include "common.h"
 #include "ioloop.h"
@@ -7,6 +7,7 @@
 #include "network.h"
 #include "array.h"
 #include "str.h"
+#include "str-sanitize.h"
 #include "safe-memset.h"
 #include "auth-stream.h"
 #include "auth-request-handler.h"
@@ -21,7 +22,7 @@
 
 static void auth_client_connection_unref(struct auth_client_connection **_conn);
 
-static void auth_client_input(void *context);
+static void auth_client_input(struct auth_client_connection *conn);
 
 static const char *reply_line_hide_pass(const char *line)
 {
@@ -41,19 +42,13 @@ static const char *reply_line_hide_pass(const char *line)
 static void auth_client_send(struct auth_client_connection *conn,
 			     const char *cmd)
 {
-	string_t *str;
+	struct const_iovec iov[2];
 
-	t_push();
-	str = t_str_new(256);
-	str_append(str, cmd);
-	str_append_c(str, '\n');
-
-	if (conn->auth->verbose_debug) {
-		i_info("client out: %s", conn->auth->verbose_debug_passwords ?
-		       cmd : reply_line_hide_pass(cmd));
-	}
-
-	(void)o_stream_send(conn->output, str_data(str), str_len(str));
+	iov[0].iov_base = cmd;
+	iov[0].iov_len = strlen(cmd);
+	iov[1].iov_base = "\n";
+	iov[1].iov_len = 1;
+	(void)o_stream_sendv(conn->output, iov, 2);
 
 	if (o_stream_get_buffer_used_size(conn->output) >=
 	    OUTBUF_THROTTLE_SIZE) {
@@ -62,13 +57,16 @@ static void auth_client_send(struct auth_client_connection *conn,
 		if (conn->io != NULL)
 			io_remove(&conn->io);
 	}
-	t_pop();
+
+	if (conn->auth->verbose_debug) {
+		i_info("client out: %s", conn->auth->verbose_debug_passwords ?
+		       cmd : reply_line_hide_pass(cmd));
+	}
 }
 
-static void auth_callback(struct auth_stream_reply *reply, void *context)
+static void auth_callback(struct auth_stream_reply *reply,
+			  struct auth_client_connection *conn)
 {
-	struct auth_client_connection *conn = context;
-
 	if (reply == NULL) {
 		/* handler destroyed */
 		auth_client_connection_unref(&conn);
@@ -124,10 +122,8 @@ auth_client_input_cpid(struct auth_client_connection *conn, const char *args)
 	return TRUE;
 }
 
-static int auth_client_output(void *context)
+static int auth_client_output(struct auth_client_connection *conn)
 {
-	struct auth_client_connection *conn = context;
-
 	if (o_stream_flush(conn->output) < 0) {
 		auth_client_connection_destroy(&conn);
 		return 1;
@@ -188,15 +184,13 @@ auth_client_handle_line(struct auth_client_connection *conn, const char *line)
 							  line + 5);
 	}
 
-	/* ignore unknown command */
-	if (conn->auth->verbose_debug)
-		i_info("client in (unknown command): %s", line);
-	return TRUE;
+	i_error("BUG: Authentication client sent unknown command: %s",
+		str_sanitize(line, 80));
+	return FALSE;
 }
 
-static void auth_client_input(void *context)
+static void auth_client_input(struct auth_client_connection *conn)
 {
-	struct auth_client_connection *conn = context;
 	char *line;
 	bool ret;
 
@@ -241,15 +235,21 @@ static void auth_client_input(void *context)
 				auth_client_connection_destroy(&conn);
 				return;
 			}
+		} else {
+			i_error("BUG: Authentication client sent "
+				"unknown handshake command: %s",
+				str_sanitize(line, 80));
+			auth_client_connection_destroy(&conn);
+			return;
 		}
 	}
 
         conn->refcount++;
 	while ((line = i_stream_next_line(conn->input)) != NULL) {
-		t_push();
-		ret = auth_client_handle_line(conn, line);
-		safe_memset(line, 0, strlen(line));
-		t_pop();
+		T_BEGIN {
+			ret = auth_client_handle_line(conn, line);
+			safe_memset(line, 0, strlen(line));
+		} T_END;
 
 		if (!ret) {
 			struct auth_client_connection *tmp_conn = conn;
@@ -275,11 +275,9 @@ auth_client_connection_create(struct auth_master_listener *listener, int fd)
 	conn->connect_uid = ++connect_uid_counter;
 
 	conn->fd = fd;
-	conn->input =
-		i_stream_create_file(fd, default_pool,
-				     AUTH_CLIENT_MAX_LINE_LENGTH, FALSE);
-	conn->output =
-		o_stream_create_file(fd, default_pool, (size_t)-1, FALSE);
+	conn->input = i_stream_create_fd(fd, AUTH_CLIENT_MAX_LINE_LENGTH,
+					 FALSE);
+	conn->output = o_stream_create_fd(fd, (size_t)-1, FALSE);
 	o_stream_set_flush_callback(conn->output, auth_client_output, conn);
 	conn->io = io_add(fd, IO_READ, auth_client_input, conn);
 
@@ -364,9 +362,8 @@ auth_client_connection_lookup(struct auth_master_listener *listener,
 	return NULL;
 }
 
-static void request_timeout(void *context)
+static void request_timeout(struct auth_master_listener *listener)
 {
-        struct auth_master_listener *listener = context;
 	struct auth_client_connection *const *clients;
 	unsigned int i, count;
 
