@@ -468,7 +468,9 @@ void auth_request_verify_plain(struct auth_request *request,
                 /* no masterdbs, master logins not supported */
                 i_assert(request->requested_login_user != NULL);
 		auth_request_log_info(request, "passdb",
-			"Attempted master login with no master passdbs");
+			"Attempted master login with no master passdbs "
+			"(trying to log in as user: %s)",
+			request->requested_login_user);
 		callback(PASSDB_RESULT_USER_UNKNOWN, request);
 		return;
 	}
@@ -702,8 +704,14 @@ void auth_request_userdb_callback(enum userdb_result result,
 		   request->client_pid != 0) {
 		/* this was an actual login attempt, the user should
 		   have been found. */
-		auth_request_log_error(request, "userdb",
-				       "user not found from userdb");
+		if (request->auth->userdbs->next == NULL) {
+			auth_request_log_error(request, "userdb",
+				"user not found from userdb %s",
+				request->auth->userdbs->userdb->iface->name);
+		} else {
+			auth_request_log_error(request, "userdb",
+				"user not found from any userdbs");
+		}
 	}
 
 	if (result != USERDB_RESULT_INTERNAL_FAILURE)
