@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 Timo Sirainen */
+/* Copyright (c) 2005-2008 Dovecot authors, see the included COPYING file */
 
 #include "common.h"
 #include "network.h"
@@ -33,7 +33,6 @@ struct auth *auth_preinit(void)
 		auth->verbose_debug_passwords;
 	auth->verbose = getenv("VERBOSE") != NULL || auth->verbose_debug;
 
-	t_push();
 	passdb_p = &auth->passdbs;
 	masterdb_p = &auth->masterdbs;
 	auth_passdb = NULL;
@@ -61,11 +60,15 @@ struct auth *auth_preinit(void)
 			masterdb_p = &auth_passdb->next;
 		}
 	}
-	if (auth_passdb != NULL && auth_passdb->pass)
-		i_fatal("Last passdb can't have pass=yes");
-	t_pop();
+	if (auth_passdb != NULL && auth_passdb->pass) {
+		if (masterdb_p != &auth_passdb->next)
+			i_fatal("Last passdb can't have pass=yes");
+		else if (auth->passdbs == NULL) {
+			i_fatal("Master passdb can't have pass=yes "
+				"if there are no passdbs");
+		}
+	}
 
-	t_push();
 	for (i = 1; ; i++) {
 		driver = getenv(t_strdup_printf("USERDB_%u_DRIVER", i));
 		if (driver == NULL)
@@ -73,12 +76,12 @@ struct auth *auth_preinit(void)
 
                 args = getenv(t_strdup_printf("USERDB_%u_ARGS", i));
 		userdb_preinit(auth, driver, args);
-
 	}
-	t_pop();
 
-	if (auth->userdbs == NULL)
-		i_fatal("You'll need to add at least one userdb");
+	if (auth->userdbs == NULL) {
+		/* use a dummy userdb static. */
+		userdb_preinit(auth, "static", "");
+	}
 	return auth;
 }
 
@@ -94,7 +97,7 @@ const string_t *auth_mechanisms_get_list(struct auth *auth)
 	return str;
 }
 
-static void auth_mech_register(struct auth *auth, struct mech_module *mech)
+static void auth_mech_register(struct auth *auth, const struct mech_module *mech)
 {
 	struct mech_module_list *list;
 
@@ -144,6 +147,17 @@ static bool auth_passdb_list_have_credentials(struct auth *auth)
 	return FALSE;
 }
 
+static int auth_passdb_list_have_set_credentials(struct auth *auth)
+{
+	struct auth_passdb *passdb;
+
+	for (passdb = auth->passdbs; passdb != NULL; passdb = passdb->next) {
+		if (passdb->passdb->iface.set_credentials != NULL)
+			return TRUE;
+	}
+	return FALSE;
+}
+
 static void auth_mech_list_verify_passdb(struct auth *auth)
 {
 	struct mech_module_list *list;
@@ -155,6 +169,9 @@ static void auth_mech_list_verify_passdb(struct auth *auth)
 		if (list->module.passdb_need_credentials &&
                     !auth_passdb_list_have_credentials(auth))
 			break;
+ 		if (list->module.passdb_need_set_credentials &&
+ 		    !auth_passdb_list_have_set_credentials(auth))
+ 			break;
 	}
 
 	if (list != NULL) {
@@ -172,7 +189,7 @@ void auth_init(struct auth *auth)
 {
 	struct auth_passdb *passdb;
 	struct auth_userdb *userdb;
-	struct mech_module *mech;
+	const struct mech_module *mech;
 	const char *const *mechanisms;
 	const char *env;
 
@@ -218,13 +235,10 @@ void auth_init(struct auth *auth)
 		i_fatal("No authentication mechanisms configured");
 	auth_mech_list_verify_passdb(auth);
 
-	/* get our realm - note that we allocate from data stack so
-	   this function should never be called inside I/O loop or anywhere
-	   else where t_pop() is called */
 	env = getenv("REALMS");
 	if (env == NULL)
 		env = "";
-	auth->auth_realms = t_strsplit_spaces(env, " ");
+	auth->auth_realms = p_strsplit_spaces(auth->pool, env, " ");
 
 	env = getenv("DEFAULT_REALM");
 	if (env != NULL && *env != '\0')
@@ -283,5 +297,5 @@ void auth_deinit(struct auth **_auth)
 	auth_request_handler_deinit();
 	passdb_cache_deinit();
 
-	pool_unref(auth->pool);
+	pool_unref(&auth->pool);
 }
