@@ -30,14 +30,14 @@ struct apop_auth_request {
 };
 
 static bool verify_credentials(struct apop_auth_request *request,
-			       const char *credentials)
+			       const unsigned char *credentials, size_t size)
 {
 	unsigned char digest[16];
 	struct md5_context ctx;
 
 	md5_init(&ctx);
 	md5_update(&ctx, request->challenge, strlen(request->challenge));
-	md5_update(&ctx, credentials, strlen(credentials));
+	md5_update(&ctx, credentials, size);
 	md5_final(&ctx, digest);
 
 	return memcmp(digest, request->digest, 16) == 0;
@@ -45,7 +45,7 @@ static bool verify_credentials(struct apop_auth_request *request,
 
 static void
 apop_credentials_callback(enum passdb_result result,
-			  const char *credentials,
+			  const unsigned char *credentials, size_t size,
 			  struct auth_request *auth_request)
 {
 	struct apop_auth_request *request =
@@ -53,7 +53,7 @@ apop_credentials_callback(enum passdb_result result,
 
 	switch (result) {
 	case PASSDB_RESULT_OK:
-		if (verify_credentials(request, credentials))
+		if (verify_credentials(request, credentials, size))
 			auth_request_success(auth_request, NULL, 0);
 		else
 			auth_request_fail(auth_request);
@@ -133,8 +133,7 @@ mech_apop_auth_initial(struct auth_request *auth_request,
 
 	memcpy(request->digest, tmp, sizeof(request->digest));
 
-	auth_request_lookup_credentials(auth_request,
-					PASSDB_CREDENTIALS_PLAINTEXT,
+	auth_request_lookup_credentials(auth_request, "PLAIN",
 					apop_credentials_callback);
 }
 
@@ -158,6 +157,7 @@ const struct mech_module mech_apop = {
 
 	MEMBER(passdb_need_plain) FALSE,
 	MEMBER(passdb_need_credentials) TRUE,
+	MEMBER(passdb_need_set_credentials) FALSE,
 
 	mech_apop_auth_new,
 	mech_apop_auth_initial,
