@@ -1,4 +1,4 @@
-/* Copyright (C) 2002 Timo Sirainen */
+/* Copyright (c) 2002-2008 Dovecot authors, see the included COPYING file */
 
 #include "common.h"
 #include "ioloop.h"
@@ -10,7 +10,7 @@
 
 static struct mech_module_list *mech_modules;
 
-void mech_register_module(struct mech_module *module)
+void mech_register_module(const struct mech_module *module)
 {
 	struct mech_module_list *list;
 
@@ -21,7 +21,7 @@ void mech_register_module(struct mech_module *module)
 	mech_modules = list;
 }
 
-void mech_unregister_module(struct mech_module *module)
+void mech_unregister_module(const struct mech_module *module)
 {
 	struct mech_module_list **pos, *list;
 
@@ -35,7 +35,7 @@ void mech_unregister_module(struct mech_module *module)
 	}
 }
 
-struct mech_module *mech_module_find(const char *name)
+const struct mech_module *mech_module_find(const char *name)
 {
 	struct mech_module_list *list;
 
@@ -59,20 +59,24 @@ void mech_generic_auth_initial(struct auth_request *request,
 
 void mech_generic_auth_free(struct auth_request *request)
 {
-	pool_unref(request->pool);
+	pool_unref(&request->pool);
 }
 
-extern struct mech_module mech_plain;
-extern struct mech_module mech_login;
-extern struct mech_module mech_apop;
-extern struct mech_module mech_cram_md5;
-extern struct mech_module mech_digest_md5;
-extern struct mech_module mech_ntlm;
-extern struct mech_module mech_rpa;
-extern struct mech_module mech_anonymous;
+extern const struct mech_module mech_plain;
+extern const struct mech_module mech_login;
+extern const struct mech_module mech_apop;
+extern const struct mech_module mech_cram_md5;
+extern const struct mech_module mech_digest_md5;
+extern const struct mech_module mech_ntlm;
+extern const struct mech_module mech_otp;
+extern const struct mech_module mech_skey;
+extern const struct mech_module mech_rpa;
+extern const struct mech_module mech_anonymous;
 #ifdef HAVE_GSSAPI
-extern struct mech_module mech_gssapi;
+extern const struct mech_module mech_gssapi;
 #endif
+extern const struct mech_module mech_winbind_ntlm;
+extern const struct mech_module mech_winbind_spnego;
 
 void mech_init(void)
 {
@@ -81,10 +85,16 @@ void mech_init(void)
 	mech_register_module(&mech_apop);
 	mech_register_module(&mech_cram_md5);
 	mech_register_module(&mech_digest_md5);
-	mech_register_module(&mech_ntlm);
+	if (getenv("NTLM_USE_WINBIND") != NULL)
+		mech_register_module(&mech_winbind_ntlm);
+	else
+		mech_register_module(&mech_ntlm);
+	mech_register_module(&mech_winbind_spnego);
+	mech_register_module(&mech_otp);
+	mech_register_module(&mech_skey);
 	mech_register_module(&mech_rpa);
 	mech_register_module(&mech_anonymous);
-#ifdef HAVE_GSSAPI
+#ifdef BUILTIN_GSSAPI
 	mech_register_module(&mech_gssapi);
 #endif
 }
@@ -96,10 +106,16 @@ void mech_deinit(void)
 	mech_unregister_module(&mech_apop);
 	mech_unregister_module(&mech_cram_md5);
 	mech_unregister_module(&mech_digest_md5);
-	mech_unregister_module(&mech_ntlm);
+	if (getenv("NTLM_USE_WINBIND") != NULL)
+		mech_unregister_module(&mech_winbind_ntlm);
+	else
+		mech_unregister_module(&mech_ntlm);
+	mech_unregister_module(&mech_winbind_spnego);
+	mech_unregister_module(&mech_otp);
+	mech_unregister_module(&mech_skey);
 	mech_unregister_module(&mech_rpa);
 	mech_unregister_module(&mech_anonymous);
-#ifdef HAVE_GSSAPI
+#ifdef BUILTIN_GSSAPI
 	mech_unregister_module(&mech_gssapi);
 #endif
 }
