@@ -16,6 +16,11 @@
 
 #define VPOPMAIL_DEFAULT_PASS_SCHEME "CRYPT"
 
+/* pw_flags was added in vpopmail 5.4, olders use pw_gid field */
+#ifndef VQPASSWD_HAS_PW_FLAGS
+#  define pw_flags pw_gid
+#endif
+
 static char *
 vpopmail_password_lookup(struct auth_request *auth_request, bool cleartext,
 			 enum passdb_result *result_r)
@@ -30,12 +35,13 @@ vpopmail_password_lookup(struct auth_request *auth_request, bool cleartext,
 		return NULL;
 	}
 
-	if (((vpw->pw_gid & NO_IMAP) != 0 &&
+	if (((vpw->pw_flags & NO_IMAP) != 0 &&
 	     strcasecmp(auth_request->service, "IMAP") == 0) ||
-	    ((vpw->pw_gid & NO_POP) != 0 &&
+	    ((vpw->pw_flags & NO_POP) != 0 &&
 	     strcasecmp(auth_request->service, "POP3") == 0)) {
 		auth_request_log_info(auth_request, "vpopmail",
-				      "%s disabled", auth_request->service);
+				      "%s disabled in vpopmail for this user",
+				      auth_request->service);
 		password = NULL;
 		*result_r = PASSDB_RESULT_USER_DISABLED;
 	} else {
