@@ -328,6 +328,7 @@ static void config_file_init(const char *path)
 		i_fatal_status(EX_CONFIG, "open(%s) failed: %m", path);
 
 	input = i_stream_create_fd(fd, 1024, TRUE);
+	i_stream_set_return_partial_line(input, TRUE);
 	while ((line = i_stream_read_next_line(input)) != NULL) {
 		/* @UNSAFE: line is modified */
 
@@ -361,6 +362,13 @@ static void config_file_init(const char *path)
 			len--;
 		line[len] = '\0';
 
+		if (strncmp(line, "!include_try ", 13) == 0)
+			continue;
+		if (strncmp(line, "!include ", 9) == 0) {
+			i_fatal_status(EX_CONFIG, "Error in config file %s: "
+				       "deliver doesn't support !include directive", path);
+		}
+
 		value = p = strchr(line, '=');
 		if (value == NULL) {
 			if (strchr(line, '{') != NULL) {
