@@ -82,6 +82,7 @@ static void client_auth_failed(struct imap_client *client)
 		io_remove(&client->io);
 	client->io = io_add(client->common.fd, IO_READ,
 			    client_input, client);
+	client_input(client);
 }
 
 static bool client_handle_args(struct imap_client *client,
@@ -109,6 +110,12 @@ static bool client_handle_args(struct imap_client *client,
 			destuser = *args + 9;
 		else if (strncmp(*args, "pass=", 5) == 0)
 			pass = *args + 5;
+		else if (strncmp(*args, "user=", 5) == 0) {
+			/* already handled in login-common */
+		} else if (auth_debug) {
+			i_info("Ignoring unknown passdb extra field: %s",
+			       *args);
+		}
 	}
 
 	if (destuser == NULL)
