@@ -15,7 +15,6 @@ static int proxy_input_line(struct imap_client *client,
 			    struct ostream *output, const char *line)
 {
 	string_t *str;
-	const char *msg;
 
 	i_assert(!client->destroyed);
 
@@ -53,10 +52,17 @@ static int proxy_input_line(struct imap_client *client,
 		(void)o_stream_send(client->output,
 				    str_data(str), str_len(str));
 
-		msg = t_strdup_printf("proxy(%s): started proxying to %s:%u",
-				      client->common.virtual_user,
-				      login_proxy_get_host(client->proxy),
-				      login_proxy_get_port(client->proxy));
+		str_truncate(str, 0);
+		str_printfa(str, "proxy(%s): started proxying to %s:%u",
+			    client->common.virtual_user,
+			    login_proxy_get_host(client->proxy),
+			    login_proxy_get_port(client->proxy));
+		if (strcmp(client->common.virtual_user,
+			   client->proxy_user) != 0) {
+			/* remote username is different, log it */
+			str_append_c(str, '/');
+			str_append(str, client->proxy_user);
+		}
 
 		(void)client_skip_line(client);
 		login_proxy_detach(client->proxy, client->input,
@@ -66,7 +72,7 @@ static int proxy_input_line(struct imap_client *client,
 		client->input = NULL;
 		client->output = NULL;
 		client->common.fd = -1;
-		client_destroy_success(client, msg);
+		client_destroy_success(client, str_c(str));
 		return -1;
 	} else if (strncmp(line, "P ", 2) == 0) {
 		/* If the backend server isn't Dovecot, the error message may
@@ -84,6 +90,26 @@ static int proxy_input_line(struct imap_client *client,
 		   be using only Dovecot as their backend :) */
 		client_send_tagline(client, line + 2);
 
+		if (verbose_auth) {
+			str = t_str_new(128);
+			str_printfa(str, "proxy(%s): Login failed to %s:%u",
+				    client->common.virtual_user,
+				    login_proxy_get_host(client->proxy),
+				    login_proxy_get_port(client->proxy));
+			if (strcmp(client->common.virtual_user,
+				   client->proxy_user) != 0) {
+				/* remote username is different, log it */
+				str_append_c(str, '/');
+				str_append(str, client->proxy_user);
+			}
+			str_append(str, ": ");
+			if (strncasecmp(line + 2, "NO ", 3) == 0)
+				str_append(str, line + 2 + 3);
+			else
+				str_append(str, line + 2);
+			i_info("%s", str_c(str));
+		}
+
 		/* allow client input again */
 		i_assert(client->io == NULL);
 		client->io = io_add(client->common.fd, IO_READ,
