@@ -1,4 +1,4 @@
-/* Copyright (C) 2002 Timo Sirainen */
+/* Copyright (c) 2002-2008 Dovecot authors, see the included COPYING file */
 
 #include "common.h"
 #include "commands.h"
@@ -18,9 +18,12 @@ bool cmd_close(struct client_command_context *cmd)
 
 	if (!imap_expunge(mailbox, NULL))
 		client_send_untagged_storage_error(client, storage);
+	else if (mailbox_sync(mailbox, 0, 0, NULL) < 0)
+		client_send_untagged_storage_error(client, storage);
 
 	if (mailbox_close(&mailbox) < 0)
                 client_send_untagged_storage_error(client, storage);
+	client_update_mailbox_flags(client, NULL);
 
 	client_send_tagline(cmd, "OK Close completed.");
 	return TRUE;
