@@ -1,5 +1,5 @@
-#ifndef __COMMANDS_UTIL_H
-#define __COMMANDS_UTIL_H
+#ifndef COMMANDS_UTIL_H
+#define COMMANDS_UTIL_H
 
 struct msgset_generator_context {
 	string_t *str;
@@ -11,7 +11,7 @@ struct mailbox_keywords;
 
 /* Finds namespace for given mailbox from namespaces. If not found,
    sends "Unknown namespace" error message to client. */
-struct namespace *
+struct mail_namespace *
 client_find_namespace(struct client_command_context *cmd, const char **mailbox);
 /* Finds mail storage for given mailbox from namespaces. If not found,
    sends "Unknown namespace" error message to client. */
@@ -32,6 +32,9 @@ bool client_verify_mailbox_name(struct client_command_context *cmd,
    error message to client. */
 bool client_verify_open_mailbox(struct client_command_context *cmd);
 
+/* Send last mailbox list error message to client. */
+void client_send_list_error(struct client_command_context *cmd,
+			    struct mailbox_list *list);
 /* Send last mail storage error message to client. */
 void client_send_storage_error(struct client_command_context *cmd,
 			       struct mail_storage *storage);
@@ -43,17 +46,20 @@ void client_send_untagged_storage_error(struct client *client,
 /* Parse flags. Returns TRUE if successful, if not sends an error message to
    client. */
 bool client_parse_mail_flags(struct client_command_context *cmd,
-			     struct imap_arg *args, enum mail_flags *flags_r,
+			     const struct imap_arg *args,
+			     enum mail_flags *flags_r,
 			     const char *const **keywords_r);
 
-/* Send FLAGS + PERMANENTFLAGS to client. */
-void client_send_mailbox_flags(struct client *client, struct mailbox *box,
-			       const array_t *keywords);
-
-/* Copy keywords into dest. dest must have been initialized. Returns TRUE if
-   keywords changed. */
-bool client_save_keywords(struct mailbox_keywords *dest,
-			  const array_t *keywords);
+/* Send FLAGS + PERMANENTFLAGS to client if they have changed,
+   or if selecting=TRUE. */
+void client_send_mailbox_flags(struct client *client, bool selecting);
+/* Update client->keywords array. Use keywords=NULL when unselecting. */
+void client_update_mailbox_flags(struct client *client,
+				 const ARRAY_TYPE(keywords) *keywords);
+/* Convert keyword indexes to keyword names in selected mailbox. */
+const char *const *
+client_get_keyword_names(struct client *client, ARRAY_TYPE(keywords) *dest,
+			 const ARRAY_TYPE(keyword_indexes) *src);
 
 bool mailbox_equals(struct mailbox *box1, struct mail_storage *storage2,
 		    const char *name2);
