@@ -155,7 +155,8 @@ mail_cache_register_get_list(struct mail_cache *cache, pool_t pool,
 	if (!cache->opened)
 		(void)mail_cache_open_and_verify(cache);
 
-	list = p_new(pool, struct mail_cache_field, cache->fields_count);
+	list = cache->fields_count == 0 ? NULL :
+		p_new(pool, struct mail_cache_field, cache->fields_count);
 	for (i = 0; i < cache->fields_count; i++) {
 		list[i] = cache->fields[i].field;
 		list[i].name = p_strdup(pool, list[i].name);
