@@ -4,7 +4,7 @@
 #include "ioloop.h"
 #include "array.h"
 #include "mail-index-private.h"
-#include "mail-transaction-log.h"
+#include "mail-transaction-log-private.h"
 
 static void mail_index_fsck_error(struct mail_index *index,
 				  const char *fmt, ...) ATTR_FORMAT(2, 3);
@@ -26,17 +26,12 @@ static void mail_index_fsck_error(struct mail_index *index,
 	}
 
 static void
-mail_index_fsck_header(struct mail_index *index, struct mail_index_map *map,
-		       struct mail_index_header *hdr)
+mail_index_fsck_log_pos(struct mail_index *index, struct mail_index_map *map,
+			struct mail_index_header *hdr)
 {
 	uint32_t file_seq;
 	uoff_t file_offset;
 
-	/* mail_index_map_check_header() has already checked that the index
-	   isn't completely broken. */
-	if (hdr->uid_validity == 0 && hdr->next_uid != 1)
-		hdr->uid_validity = ioloop_time;
-
 	mail_transaction_log_get_head(index->log, &file_seq, &file_offset);
 	if (hdr->log_file_seq < file_seq) {
 		hdr->log_file_head_offset = hdr->log_file_tail_offset =
@@ -57,6 +52,19 @@ mail_index_fsck_header(struct mail_index *index, struct mail_index_map *map,
 	}
 }
 
+static void
+mail_index_fsck_header(struct mail_index *index, struct mail_index_map *map,
+		       struct mail_index_header *hdr)
+{
+	/* mail_index_map_check_header() has already checked that the index
+	   isn't completely broken. */
+	if (hdr->uid_validity == 0 && hdr->next_uid != 1)
+		hdr->uid_validity = ioloop_time;
+
+	if (index->log->head != NULL)
+		mail_index_fsck_log_pos(index, map, hdr);
+}
+
 static bool
 array_has_name(const ARRAY_TYPE(const_string) *names, const char *name)
 {
@@ -253,20 +261,24 @@ mail_index_fsck_extensions(struct mail_index *index, struct mail_index_map *map,
 	const struct mail_index_ext_header *ext_hdr;
 	ARRAY_TYPE(const_string) names;
 	const char *name, *error;
-	unsigned int offset, ext_offset, i;
+	unsigned int offset, next_offset, i;
 
 	t_array_init(&names, 64);
 	offset = MAIL_INDEX_HEADER_SIZE_ALIGN(hdr->base_header_size);
 	for (i = 0; offset < hdr->header_size; i++) {
-		ext_offset = offset;
-		if (mail_index_map_ext_get_next(map, &offset,
+		/* mail_index_map_ext_get_next() uses map->hdr, so make sure
+		   it's up-to-date */
+		map->hdr = *hdr;
+
+		next_offset = offset;
+		if (mail_index_map_ext_get_next(map, &next_offset,
 						&ext_hdr, &name) < 0) {
 			/* the extension continued outside header, drop it */
 			mail_index_fsck_error(index,
 					      "Dropped extension #%d (%s) "
 					      "with invalid header size",
 					      i, name);
-			hdr->header_size = ext_offset;
+			hdr->header_size = offset;
 			break;
 		}
 		if (mail_index_map_ext_hdr_check(hdr, ext_hdr, name,
@@ -282,20 +294,20 @@ mail_index_fsck_extensions(struct mail_index *index, struct mail_index_map *map,
 
 			if (strcmp(name, "keywords") == 0) {
 				mail_index_fsck_keywords(index, map, hdr,
-							 ext_hdr, ext_offset,
-							 &offset);
+							 ext_hdr, offset,
+							 &next_offset);
 			}
 			array_append(&names, &name, 1);
+			offset = next_offset;
 			continue;
 		}
 
 		/* drop the field */
-		hdr->header_size -= offset - ext_offset;
-		buffer_copy(map->hdr_copy_buf, ext_offset,
-			    map->hdr_copy_buf, offset, (size_t)-1);
+		hdr->header_size -= next_offset - offset;
+		buffer_copy(map->hdr_copy_buf, offset,
+			    map->hdr_copy_buf, next_offset, (size_t)-1);
 		buffer_set_used_size(map->hdr_copy_buf, hdr->header_size);
 		map->hdr_base = map->hdr_copy_buf->data;
-		offset = ext_offset;
 	}
 }
 
@@ -396,12 +408,13 @@ mail_index_fsck_map(struct mail_index *index, struct mail_index_map *map)
 {
 	struct mail_index_header hdr;
 
-	/* Remember the log head position. If we go back in the index's head
-	   offset, ignore errors in the log up to this offset. */
-	mail_transaction_log_get_head(index->log,
-				      &index->fsck_log_head_file_seq,
-				      &index->fsck_log_head_file_offset);
-
+	if (index->log->head != NULL) {
+		/* Remember the log head position. If we go back in the index's
+		   head offset, ignore errors in the log up to this offset. */
+		mail_transaction_log_get_head(index->log,
+			&index->fsck_log_head_file_seq,
+			&index->fsck_log_head_file_offset);
+	}
 	hdr = map->hdr;
 
 	mail_index_fsck_header(index, map, &hdr);
@@ -420,6 +433,13 @@ int mail_index_fsck(struct mail_index *index)
 
 	i_warning("fscking index file %s", index->filepath);
 
+	if (index->log->head == NULL) {
+		/* we're trying to open the index files, but there wasn't
+		   any .log file. this should be rare, so just fsck it without
+		   locking. */
+		orig_locked = TRUE;
+	}
+
 	if (!orig_locked) {
 		if (mail_transaction_log_sync_lock(index->log, &file_seq,
 						   &file_offset) < 0)
