@@ -1,19 +1,32 @@
-/* Copyright (C) 2003-2004 Timo Sirainen */
+/* Copyright (c) 2003-2008 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
-#include "buffer.h"
 #include "mail-index-view-private.h"
 #include "mail-index-sync-private.h"
 #include "mail-index-transaction-private.h"
 #include "mail-transaction-log-private.h"
-#include "mail-transaction-util.h"
 #include "mail-cache.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 
-struct uid_range {
-	uint32_t uid1, uid2;
+struct mail_index_sync_ctx {
+	struct mail_index *index;
+	struct mail_index_view *view;
+	struct mail_index_transaction *sync_trans, *ext_trans;
+	enum mail_index_sync_flags flags;
+
+	const struct mail_transaction_header *hdr;
+	const void *data;
+
+	ARRAY_DEFINE(sync_list, struct mail_index_sync_list);
+	uint32_t next_uid;
+	uint32_t last_tail_seq, last_tail_offset;
+
+	uint32_t append_uid_first, append_uid_last;
+
+	unsigned int sync_appends:1;
 };
 
 static void mail_index_sync_add_expunge(struct mail_index_sync_ctx *ctx)
@@ -24,7 +37,7 @@ static void mail_index_sync_add_expunge(struct mail_index_sync_ctx *ctx)
 
 	for (i = 0; i < size; i++) {
 		for (uid = e[i].uid1; uid <= e[i].uid2; uid++)
-			mail_index_expunge(ctx->trans, uid);
+			mail_index_expunge(ctx->sync_trans, uid);
 	}
 }
 
@@ -35,13 +48,13 @@ static void mail_index_sync_add_flag_update(struct mail_index_sync_ctx *ctx)
 
 	for (i = 0; i < size; i++) {
 		if (u[i].add_flags != 0) {
-			mail_index_update_flags_range(ctx->trans,
+			mail_index_update_flags_range(ctx->sync_trans,
 						      u[i].uid1, u[i].uid2,
 						      MODIFY_ADD,
 						      u[i].add_flags);
 		}
 		if (u[i].remove_flags != 0) {
-			mail_index_update_flags_range(ctx->trans,
+			mail_index_update_flags_range(ctx->sync_trans,
 						      u[i].uid1, u[i].uid2,
 						      MODIFY_REMOVE,
 						      u[i].remove_flags);
@@ -63,22 +76,20 @@ static void mail_index_sync_add_keyword_update(struct mail_index_sync_ctx *ctx)
 		uidset_offset += 4 - (uidset_offset % 4);
 	uids = CONST_PTR_OFFSET(u, uidset_offset);
 
-	t_push();
 	keyword_names[0] = t_strndup(u + 1, u->name_size);
 	keyword_names[1] = NULL;
-	keywords = mail_index_keywords_create(ctx->trans, keyword_names);
+	keywords = mail_index_keywords_create(ctx->index, keyword_names);
 
 	size = (ctx->hdr->size - uidset_offset) / sizeof(uint32_t);
 	for (i = 0; i < size; i += 2) {
 		/* FIXME: mail_index_update_keywords_range() */
 		for (uid = uids[i]; uid <= uids[i+1]; uid++) {
-			mail_index_update_keywords(ctx->trans, uid,
+			mail_index_update_keywords(ctx->sync_trans, uid,
 						   u->modify_type, keywords);
 		}
 	}
 
 	mail_index_keywords_free(&keywords);
-	t_pop();
 }
 
 static void mail_index_sync_add_keyword_reset(struct mail_index_sync_ctx *ctx)
@@ -88,10 +99,10 @@ static void mail_index_sync_add_keyword_reset(struct mail_index_sync_ctx *ctx)
 	struct mail_keywords *keywords;
 	uint32_t uid;
 
-	keywords = mail_index_keywords_create(ctx->trans, NULL);
+	keywords = mail_index_keywords_create(ctx->index, NULL);
 	for (i = 0; i < size; i++) {
 		for (uid = u[i].uid1; uid <= u[i].uid2; uid++) {
-			mail_index_update_keywords(ctx->trans, uid,
+			mail_index_update_keywords(ctx->sync_trans, uid,
 						   MODIFY_REPLACE, keywords);
 		}
 	}
@@ -112,7 +123,7 @@ static void mail_index_sync_add_append(struct mail_index_sync_ctx *ctx)
 	ctx->sync_appends = TRUE;
 }
 
-static void mail_index_sync_add_transaction(struct mail_index_sync_ctx *ctx)
+static bool mail_index_sync_add_transaction(struct mail_index_sync_ctx *ctx)
 {
 	switch (ctx->hdr->type & MAIL_TRANSACTION_TYPE_MASK) {
 	case MAIL_TRANSACTION_EXPUNGE:
@@ -130,10 +141,13 @@ static void mail_index_sync_add_transaction(struct mail_index_sync_ctx *ctx)
 	case MAIL_TRANSACTION_APPEND:
 		mail_index_sync_add_append(ctx);
 		break;
+	default:
+		return FALSE;
 	}
+	return TRUE;
 }
 
-static int mail_index_sync_add_dirty_updates(struct mail_index_sync_ctx *ctx)
+static void mail_index_sync_add_dirty_updates(struct mail_index_sync_ctx *ctx)
 {
 	struct mail_transaction_flag_update update;
 	const struct mail_index_record *rec;
@@ -143,112 +157,92 @@ static int mail_index_sync_add_dirty_updates(struct mail_index_sync_ctx *ctx)
 
 	messages_count = mail_index_view_get_messages_count(ctx->view);
 	for (seq = 1; seq <= messages_count; seq++) {
-		if (mail_index_lookup(ctx->view, seq, &rec) < 0)
-			return -1;
-
+		rec = mail_index_lookup(ctx->view, seq);
 		if ((rec->flags & MAIL_INDEX_MAIL_FLAG_DIRTY) == 0)
 			continue;
 
-		mail_index_update_flags(ctx->trans, rec->uid,
+		mail_index_update_flags(ctx->sync_trans, rec->uid,
 					MODIFY_REPLACE, rec->flags);
 	}
-	return 0;
 }
 
-static int mail_index_sync_add_recent_updates(struct mail_index_sync_ctx *ctx)
+static void
+mail_index_sync_update_mailbox_pos(struct mail_index_sync_ctx *ctx)
 {
-	const struct mail_index_record *rec;
-	uint32_t seq, messages_count;
-	bool seen_recent = FALSE;
+	uint32_t seq;
+	uoff_t offset;
 
-	messages_count = mail_index_view_get_messages_count(ctx->view);
-	for (seq = 1; seq <= messages_count; seq++) {
-		if (mail_index_lookup(ctx->view, seq, &rec) < 0)
-			return -1;
+	mail_transaction_log_view_get_prev_pos(ctx->view->log_view,
+					       &seq, &offset);
 
-		if ((rec->flags & MAIL_RECENT) != 0) {
-			seen_recent = TRUE;
-			mail_index_update_flags(ctx->trans, rec->uid,
-						MODIFY_REMOVE, MAIL_RECENT);
-		}
-	}
-
-	if (!seen_recent) {
-		/* no recent messages, drop the sync_recent flag so we
-		   don't scan through the message again */
-		ctx->sync_recent = FALSE;
-	}
-
-	return 0;
+	ctx->last_tail_seq = seq;
+	ctx->last_tail_offset = offset + ctx->hdr->size + sizeof(*ctx->hdr);
 }
 
 static int
-mail_index_sync_read_and_sort(struct mail_index_sync_ctx *ctx,
-			      bool *seen_external_r)
+mail_index_sync_read_and_sort(struct mail_index_sync_ctx *ctx)
 {
+	struct mail_index_transaction *sync_trans = ctx->sync_trans;
 	struct mail_index_sync_list *synclist;
         const struct mail_index_transaction_keyword_update *keyword_updates;
 	unsigned int i, keyword_count;
 	int ret;
 
-	*seen_external_r = FALSE;
-
 	if ((ctx->view->map->hdr.flags & MAIL_INDEX_HDR_FLAG_HAVE_DIRTY) &&
-	    ctx->sync_dirty) {
+	    (ctx->flags & MAIL_INDEX_SYNC_FLAG_FLUSH_DIRTY) != 0) {
 		/* show dirty flags as flag updates */
-		if (mail_index_sync_add_dirty_updates(ctx) < 0)
-			return -1;
+		mail_index_sync_add_dirty_updates(ctx);
 	}
 
-	if (ctx->sync_recent) {
-		if (mail_index_sync_add_recent_updates(ctx) < 0)
-			return -1;
-	}
-
-	/* read all transactions from log into a transaction in memory */
+	/* read all transactions from log into a transaction in memory.
+	   skip the external ones, they're already synced to mailbox and
+	   included in our view */
 	while ((ret = mail_transaction_log_view_next(ctx->view->log_view,
 						     &ctx->hdr,
-						     &ctx->data, NULL)) > 0) {
+						     &ctx->data)) > 0) {
 		if ((ctx->hdr->type & MAIL_TRANSACTION_EXTERNAL) != 0)
-			*seen_external_r = TRUE;
-		 else
-			mail_index_sync_add_transaction(ctx);
+			continue;
+
+		T_BEGIN {
+			if (mail_index_sync_add_transaction(ctx))
+				mail_index_sync_update_mailbox_pos(ctx);
+		} T_END;
 	}
 
 	/* create an array containing all expunge, flag and keyword update
 	   arrays so we can easily go through all of the changes. */
-	keyword_count = !array_is_created(&ctx->trans->keyword_updates) ? 0 :
-		array_count(&ctx->trans->keyword_updates);
-	ARRAY_CREATE(&ctx->sync_list, default_pool,
-		     struct mail_index_sync_list, keyword_count + 2);
+	keyword_count = !array_is_created(&sync_trans->keyword_updates) ? 0 :
+		array_count(&sync_trans->keyword_updates);
+	i_array_init(&ctx->sync_list, keyword_count + 2);
 
-	if (array_is_created(&ctx->trans->expunges)) {
+	if (array_is_created(&sync_trans->expunges)) {
 		synclist = array_append_space(&ctx->sync_list);
-		synclist->array = &ctx->trans->expunges;
+		synclist->array = (void *)&sync_trans->expunges;
 	}
 
-	if (array_is_created(&ctx->trans->updates)) {
+	if (array_is_created(&sync_trans->updates)) {
 		synclist = array_append_space(&ctx->sync_list);
-		synclist->array = &ctx->trans->updates;
+		synclist->array = (void *)&sync_trans->updates;
 	}
 
 	/* we must return resets before keyword additions or they get lost */
-	if (array_is_created(&ctx->trans->keyword_resets)) {
+	if (array_is_created(&sync_trans->keyword_resets)) {
 		synclist = array_append_space(&ctx->sync_list);
-		synclist->array = &ctx->trans->keyword_resets;
+		synclist->array = (void *)&sync_trans->keyword_resets;
 	}
 
 	keyword_updates = keyword_count == 0 ? NULL :
-		array_get(&ctx->trans->keyword_updates, NULL);
+		array_idx(&sync_trans->keyword_updates, 0);
 	for (i = 0; i < keyword_count; i++) {
 		if (array_is_created(&keyword_updates[i].add_seq)) {
 			synclist = array_append_space(&ctx->sync_list);
-			synclist->array = &keyword_updates[i].add_seq;
+			synclist->array = (void *)&keyword_updates[i].add_seq;
 			synclist->keyword_idx = i;
 		}
 		if (array_is_created(&keyword_updates[i].remove_seq)) {
 			synclist = array_append_space(&ctx->sync_list);
-			synclist->array = &keyword_updates[i].remove_seq;
+			synclist->array =
+				(void *)&keyword_updates[i].remove_seq;
 			synclist->keyword_idx = i;
 			synclist->keyword_remove = TRUE;
 		}
@@ -257,21 +251,38 @@ mail_index_sync_read_and_sort(struct mail_index_sync_ctx *ctx,
 	return ret;
 }
 
-static int mail_index_need_lock(struct mail_index *index, bool sync_recent,
-				uint32_t log_file_seq, uoff_t log_file_offset)
+static bool
+mail_index_need_sync(struct mail_index *index, enum mail_index_sync_flags flags,
+		     uint32_t log_file_seq, uoff_t log_file_offset)
 {
-	if (sync_recent && index->hdr->recent_messages_count > 0)
-		return 1;
+	const struct mail_index_header *hdr = &index->map->hdr;
+	if ((flags & MAIL_INDEX_SYNC_FLAG_REQUIRE_CHANGES) == 0)
+		return TRUE;
 
-	if (index->hdr->log_file_seq > log_file_seq ||
-	     (index->hdr->log_file_seq == log_file_seq &&
-	      index->hdr->log_file_int_offset >= log_file_offset &&
-	      index->hdr->log_file_ext_offset >= log_file_offset)) {
-		/* already synced */
-		return mail_cache_need_compress(index->cache);
+	/* sync only if there's something to do */
+	if (hdr->first_recent_uid < hdr->next_uid &&
+	    (flags & MAIL_INDEX_SYNC_FLAG_DROP_RECENT) != 0)
+		return TRUE;
+
+	if ((hdr->flags & MAIL_INDEX_HDR_FLAG_HAVE_DIRTY) &&
+	    (flags & MAIL_INDEX_SYNC_FLAG_FLUSH_DIRTY) != 0)
+		return TRUE;
+
+	if (log_file_seq == (uint32_t)-1) {
+		/* we want to sync up to transaction log's head */
+		mail_transaction_log_get_head(index->log,
+					      &log_file_seq, &log_file_offset);
 	}
+	if ((hdr->log_file_tail_offset < log_file_offset &&
+	     hdr->log_file_seq == log_file_seq) ||
+	    hdr->log_file_seq < log_file_seq)
+		return TRUE;
 
-	return 1;
+	if (index->need_recreate)
+		return TRUE;
+
+	/* already synced */
+	return mail_cache_need_compress(index->cache);
 }
 
 static int
@@ -280,187 +291,248 @@ mail_index_sync_set_log_view(struct mail_index_view *view,
 {
 	uint32_t log_seq;
 	uoff_t log_offset;
+	bool reset;
 	int ret;
 
 	mail_transaction_log_get_head(view->index->log, &log_seq, &log_offset);
 
 	ret = mail_transaction_log_view_set(view->log_view,
                                             start_file_seq, start_file_offset,
-					    log_seq, log_offset,
-					    MAIL_TRANSACTION_TYPE_MASK);
+					    log_seq, log_offset, &reset);
 	if (ret <= 0) {
 		/* either corrupted or the file was deleted for
 		   some reason. either way, we can't go forward */
 		mail_index_set_error(view->index,
 			"Unexpected transaction log desync with index %s",
 			view->index->filepath);
-		mail_index_set_inconsistent(view->index);
 		return -1;
 	}
 	return 0;
 }
 
-static int mail_index_sync_commit_external(struct mail_index_sync_ctx *ctx)
+int mail_index_sync_begin(struct mail_index *index,
+			  struct mail_index_sync_ctx **ctx_r,
+			  struct mail_index_view **view_r,
+			  struct mail_index_transaction **trans_r,
+			  enum mail_index_sync_flags flags)
 {
 	int ret;
 
-	/* find the first external transaction, if there are any */
-	while ((ret = mail_transaction_log_view_next(ctx->view->log_view,
-						     &ctx->hdr, &ctx->data,
-						     NULL)) > 0) {
-		if ((ctx->hdr->type & MAIL_TRANSACTION_EXTERNAL) != 0)
-			break;
-	}
-	if (ret < 0)
-		return -1;
-
-	if (ret > 0) {
-		uint32_t seq;
-		uoff_t offset;
-
-		/* found it. update log view's range to begin from it and
-		   write all external transactions to index. */
-		mail_transaction_log_view_get_prev_pos(ctx->view->log_view,
-						       &seq, &offset);
-		if (mail_index_sync_set_log_view(ctx->view, seq, offset) < 0)
-			return -1;
-		if (mail_index_sync_update_index(ctx, TRUE) < 0)
-			return -1;
-	}
-	return 0;
+	ret = mail_index_sync_begin_to(index, ctx_r, view_r, trans_r,
+				       (uint32_t)-1, (uoff_t)-1, flags);
+	i_assert(ret != 0 ||
+		 (flags & MAIL_INDEX_SYNC_FLAG_REQUIRE_CHANGES) != 0);
+	return ret;
 }
 
-#define MAIL_INDEX_IS_SYNCS_SAME(index) \
-	((index)->hdr != NULL && \
-	 (index)->sync_log_file_seq == (index)->hdr->log_file_seq && \
-	 (index)->sync_log_file_offset == (index)->hdr->log_file_ext_offset)
-
-int mail_index_sync_begin(struct mail_index *index,
-                          struct mail_index_sync_ctx **ctx_r,
-			  struct mail_index_view **view_r,
-			  uint32_t log_file_seq, uoff_t log_file_offset,
-			  bool sync_recent, bool sync_dirty)
+static int
+mail_index_sync_begin_init(struct mail_index *index,
+			   enum mail_index_sync_flags flags,
+			   uint32_t log_file_seq, uoff_t log_file_offset)
 {
-	struct mail_index_sync_ctx *ctx;
-	struct mail_index_view *dummy_view;
+	const struct mail_index_header *hdr;
 	uint32_t seq;
 	uoff_t offset;
-	unsigned int lock_id = 0;
-	bool seen_external;
+	bool locked = FALSE;
+	int ret;
 
-	if (mail_transaction_log_sync_lock(index->log, &seq, &offset) < 0)
-		return -1;
+	/* if we require changes, don't lock transaction log yet. first check
+	   if there's anything to sync. */
+	if ((flags & MAIL_INDEX_SYNC_FLAG_REQUIRE_CHANGES) == 0) {
+		if (mail_transaction_log_sync_lock(index->log,
+						   &seq, &offset) < 0)
+			return -1;
+		locked = TRUE;
+	}
 
-	if (!index->mmap_disable || !MAIL_INDEX_IS_SYNCS_SAME(index) ||
-	    index->sync_log_file_seq != seq ||
-	    index->sync_log_file_offset != offset) {
-		/* make sure we have the latest file mapped */
-		if (mail_index_lock_shared(index, TRUE, &lock_id) < 0) {
-			mail_transaction_log_sync_unlock(index->log);
+	/* The view must contain what we expect the mailbox to look like
+	   currently. That allows the backend to update external flag
+	   changes (etc.) if the view doesn't match the mailbox.
+
+	   We'll update the view to contain everything that exist in the
+	   transaction log except for expunges. They're synced in
+	   mail_index_sync_commit(). */
+	if ((ret = mail_index_map(index, MAIL_INDEX_SYNC_HANDLER_HEAD)) <= 0) {
+		if (ret == 0) {
+			if (locked)
+				mail_transaction_log_sync_unlock(index->log);
 			return -1;
 		}
 
-		/* with mmap_disable the force parameter has somewhat special
-		   meaning, it syncs exactly to the log seq/offset in index
-		   file's header. */
-		if (mail_index_map(index, index->mmap_disable) <= 0) {
-			mail_transaction_log_sync_unlock(index->log);
-			mail_index_unlock(index, lock_id);
+		/* let's try again */
+		if (mail_index_map(index, MAIL_INDEX_SYNC_HANDLER_HEAD) <= 0) {
+			if (locked)
+				mail_transaction_log_sync_unlock(index->log);
 			return -1;
 		}
 	}
 
-	if ((index->hdr->flags & MAIL_INDEX_HDR_FLAG_FSCK) != 0) {
-		if (mail_index_fsck(index) <= 0) {
-			mail_index_unlock(index, lock_id);
-			mail_transaction_log_sync_unlock(index->log);
-			return -1;
-		}
-	}
-
-	if (!mail_index_need_lock(index, sync_recent,
+	if (!mail_index_need_sync(index, flags,
 				  log_file_seq, log_file_offset)) {
-		mail_index_unlock(index, lock_id);
-		mail_transaction_log_sync_unlock(index->log);
+		if (locked)
+			mail_transaction_log_sync_unlock(index->log);
 		return 0;
 	}
 
-	if (index->hdr->log_file_int_offset > index->hdr->log_file_ext_offset ||
-	    (index->hdr->log_file_seq == seq &&
-	     index->hdr->log_file_ext_offset > offset) ||
-	    (index->hdr->log_file_seq != seq &&
-	     !mail_transaction_log_is_head_prev(index->log,
-	     				index->hdr->log_file_seq,
-					index->hdr->log_file_ext_offset))) {
+	if (!locked) {
+		/* it looks like we have something to sync. lock the file and
+		   check again. */
+		flags &= ~MAIL_INDEX_SYNC_FLAG_REQUIRE_CHANGES;
+		return mail_index_sync_begin_init(index, flags, log_file_seq,
+						  log_file_offset);
+	}
+
+	hdr = &index->map->hdr;
+	if (hdr->log_file_tail_offset > hdr->log_file_head_offset ||
+	    hdr->log_file_seq > seq ||
+	    (hdr->log_file_seq == seq && hdr->log_file_tail_offset > offset)) {
 		/* broken sync positions. fix them. */
 		mail_index_set_error(index,
 			"broken sync positions in index file %s",
 			index->filepath);
-		if (mail_index_fsck(index) <= 0) {
-			mail_index_unlock(index, lock_id);
-			mail_transaction_log_sync_unlock(index->log);
-			return -1;
-		}
+		mail_index_fsck_locked(index);
 	}
+	return 1;
+}
+
+int mail_index_sync_begin_to(struct mail_index *index,
+			     struct mail_index_sync_ctx **ctx_r,
+			     struct mail_index_view **view_r,
+			     struct mail_index_transaction **trans_r,
+			     uint32_t log_file_seq, uoff_t log_file_offset,
+			     enum mail_index_sync_flags flags)
+{
+	const struct mail_index_header *hdr;
+	struct mail_index_sync_ctx *ctx;
+	struct mail_index_view *sync_view;
+	enum mail_index_transaction_flags trans_flags;
+	int ret;
+
+	i_assert(!index->syncing);
+
+	if (log_file_seq != (uint32_t)-1)
+		flags |= MAIL_INDEX_SYNC_FLAG_REQUIRE_CHANGES;
+
+	ret = mail_index_sync_begin_init(index, flags, log_file_seq,
+					 log_file_offset);
+	if (ret <= 0)
+		return ret;
+
+	hdr = &index->map->hdr;
 
 	ctx = i_new(struct mail_index_sync_ctx, 1);
 	ctx->index = index;
-	ctx->lock_id = lock_id;
-	ctx->sync_recent = sync_recent;
-	ctx->sync_dirty = sync_dirty;
+	ctx->last_tail_seq = hdr->log_file_seq;
+	ctx->last_tail_offset = hdr->log_file_tail_offset;
+	ctx->flags = flags;
 
 	ctx->view = mail_index_view_open(index);
 
-	dummy_view = mail_index_dummy_view_open(index);
-	ctx->trans = mail_index_transaction_begin(dummy_view, FALSE, TRUE);
-	mail_index_view_close(&dummy_view);
+	sync_view = mail_index_dummy_view_open(index);
+	ctx->sync_trans = mail_index_transaction_begin(sync_view,
+					MAIL_INDEX_TRANSACTION_FLAG_EXTERNAL);
+	mail_index_view_close(&sync_view);
 
-	if (mail_index_sync_set_log_view(ctx->view,
-					 index->hdr->log_file_seq,
-					 index->hdr->log_file_int_offset) < 0) {
-                mail_index_sync_rollback(&ctx);
-		return -1;
-	}
-
-	/* See if there are some external transactions which were
-	   written to transaction log, but weren't yet committed to
-	   index. commit them first to avoid conflicts with another
-	   external sync.
-
-	   This is mostly needed to make sure there won't be multiple
-	   appends with same UIDs, because those would cause
-	   transaction log to be marked corrupted.
-
-	   Note that any internal transactions must not be committed
-	   yet. They need to be synced with the real mailbox first. */
-	if (seq != index->hdr->log_file_seq ||
-	    offset != index->hdr->log_file_ext_offset) {
-		if (mail_index_sync_commit_external(ctx) < 0) {
-			mail_index_sync_rollback(&ctx);
-			return -1;
-		}
-
-		mail_index_view_close(&ctx->view);
-		ctx->view = mail_index_view_open(index);
-
-		if (mail_index_sync_set_log_view(ctx->view,
-					index->hdr->log_file_seq,
-					index->hdr->log_file_int_offset) < 0)
-			return -1;
+	/* we wish to see all the changes from last mailbox sync position to
+	   the end of the transaction log */
+	if (mail_index_sync_set_log_view(ctx->view, hdr->log_file_seq,
+					 hdr->log_file_tail_offset) < 0) {
+		/* if a log file is missing, there's nothing we can do except
+		   to skip over it. fix the problem with fsck and try again. */
+		mail_index_fsck_locked(index);
+		mail_index_sync_rollback(&ctx);
+		return mail_index_sync_begin_to(index, ctx_r, view_r, trans_r,
+						log_file_seq, log_file_offset,
+						flags);
 	}
 
 	/* we need to have all the transactions sorted to optimize
 	   caller's mailbox access patterns */
-	if (mail_index_sync_read_and_sort(ctx, &seen_external) < 0) {
+	if (mail_index_sync_read_and_sort(ctx) < 0) {
                 mail_index_sync_rollback(&ctx);
 		return -1;
 	}
 
+	ctx->view->index_sync_view = TRUE;
+
+	/* create the transaction after the view has been updated with
+	   external transactions and marked as sync view */
+	trans_flags = MAIL_INDEX_TRANSACTION_FLAG_EXTERNAL;
+	if ((ctx->flags & MAIL_INDEX_SYNC_FLAG_AVOID_FLAG_UPDATES) != 0)
+		trans_flags |= MAIL_INDEX_TRANSACTION_FLAG_AVOID_FLAG_UPDATES;
+	ctx->ext_trans = mail_index_transaction_begin(ctx->view, trans_flags);
+
+	index->syncing = TRUE;
+
 	*ctx_r = ctx;
 	*view_r = ctx->view;
+	*trans_r = ctx->ext_trans;
 	return 1;
 }
 
+static bool mail_index_sync_view_have_any(struct mail_index_view *view,
+					  enum mail_index_sync_flags flags)
+{
+	const struct mail_transaction_header *hdr;
+	const void *data;
+	uint32_t log_seq;
+	uoff_t log_offset;
+	bool reset;
+	int ret;
+
+	if (view->map->hdr.first_recent_uid < view->map->hdr.next_uid &&
+	    (flags & MAIL_INDEX_SYNC_FLAG_DROP_RECENT) != 0)
+		return TRUE;
+
+	if ((view->map->hdr.flags & MAIL_INDEX_HDR_FLAG_HAVE_DIRTY) &&
+	    (flags & MAIL_INDEX_SYNC_FLAG_FLUSH_DIRTY) != 0)
+		return TRUE;
+
+	mail_transaction_log_get_head(view->index->log, &log_seq, &log_offset);
+	if (mail_transaction_log_view_set(view->log_view,
+					  view->map->hdr.log_file_seq,
+					  view->map->hdr.log_file_tail_offset,
+					  log_seq, log_offset, &reset) <= 0) {
+		/* let the actual syncing handle the error */
+		return TRUE;
+	}
+
+	while ((ret = mail_transaction_log_view_next(view->log_view,
+						     &hdr, &data)) > 0) {
+		if ((hdr->type & MAIL_TRANSACTION_EXTERNAL) != 0)
+			continue;
+
+		switch (hdr->type & MAIL_TRANSACTION_TYPE_MASK) {
+		case MAIL_TRANSACTION_EXT_REC_UPDATE:
+			/* extension record updates aren't exactly needed
+			   to be synced, but cache syncing relies on tail
+			   offsets being updated. */
+		case MAIL_TRANSACTION_EXPUNGE:
+		case MAIL_TRANSACTION_FLAG_UPDATE:
+		case MAIL_TRANSACTION_KEYWORD_UPDATE:
+		case MAIL_TRANSACTION_KEYWORD_RESET:
+			return TRUE;
+		default:
+			break;
+		}
+	}
+	return ret < 0;
+}
+
+bool mail_index_sync_have_any(struct mail_index *index,
+			      enum mail_index_sync_flags flags)
+{
+	struct mail_index_view *view;
+	bool ret;
+
+	(void)mail_index_refresh(index);
+
+	view = mail_index_view_open(index);
+	ret = mail_index_sync_view_have_any(view, flags);
+	mail_index_view_close(&view);
+	return ret;
+}
+
 static void
 mail_index_sync_get_expunge(struct mail_index_sync_rec *rec,
 			    const struct mail_transaction_expunge *exp)
@@ -503,31 +575,10 @@ static void mail_index_sync_get_keyword_reset(struct mail_index_sync_rec *rec,
 	rec->uid2 = range->uid2;
 }
 
-static int mail_index_sync_rec_check(struct mail_index_view *view,
-				     struct mail_index_sync_rec *rec)
-{
-	switch (rec->type) {
-	case MAIL_INDEX_SYNC_TYPE_EXPUNGE:
-	case MAIL_INDEX_SYNC_TYPE_FLAGS:
-	case MAIL_INDEX_SYNC_TYPE_KEYWORD_ADD:
-	case MAIL_INDEX_SYNC_TYPE_KEYWORD_REMOVE:
-	case MAIL_INDEX_SYNC_TYPE_KEYWORD_RESET:
-		if (rec->uid1 > rec->uid2 || rec->uid1 == 0) {
-			mail_transaction_log_view_set_corrupted(view->log_view,
-				"Broken UID range: %u..%u (type 0x%x)",
-				rec->uid1, rec->uid2, rec->type);
-			return -1;
-		}
-		break;
-	case MAIL_INDEX_SYNC_TYPE_APPEND:
-		break;
-	}
-	return 0;
-}
-
-int mail_index_sync_next(struct mail_index_sync_ctx *ctx,
-			 struct mail_index_sync_rec *sync_rec)
+bool mail_index_sync_next(struct mail_index_sync_ctx *ctx,
+			  struct mail_index_sync_rec *sync_rec)
 {
+	struct mail_index_transaction *sync_trans = ctx->sync_trans;
 	struct mail_index_sync_list *sync_list;
 	const struct uid_range *uid_range = NULL;
 	unsigned int i, count, next_i;
@@ -539,7 +590,7 @@ int mail_index_sync_next(struct mail_index_sync_ctx *ctx,
 	/* FIXME: replace with a priority queue so we don't have to go
 	   through the whole list constantly. and remember to make sure that
 	   keyword resets are sent before adds! */
-	sync_list = array_get_modifyable(&ctx->sync_list, &count);
+	sync_list = array_get_modifiable(&ctx->sync_list, &count);
 	for (i = 0; i < count; i++) {
 		if (!array_is_created(sync_list[i].array) ||
 		    sync_list[i].idx == array_count(sync_list[i].array))
@@ -564,32 +615,29 @@ int mail_index_sync_next(struct mail_index_sync_ctx *ctx,
 				sync_rec->type = MAIL_INDEX_SYNC_TYPE_APPEND;
 				sync_rec->uid1 = ctx->append_uid_first;
 				sync_rec->uid2 = ctx->append_uid_last;
-				return 1;
+				return TRUE;
 			}
-			return 0;
+			return FALSE;
 		}
                 ctx->next_uid = next_found_uid;
 		i = next_i;
 		uid_range = array_idx(sync_list[i].array, sync_list[i].idx);
 	}
 
-	if (sync_list[i].array == &ctx->trans->expunges) {
+	if (sync_list[i].array == (void *)&sync_trans->expunges) {
 		mail_index_sync_get_expunge(sync_rec,
 			(const struct mail_transaction_expunge *)uid_range);
-	} else if (sync_list[i].array == &ctx->trans->updates) {
+	} else if (sync_list[i].array == (void *)&sync_trans->updates) {
 		mail_index_sync_get_update(sync_rec,
 			(const struct mail_transaction_flag_update *)uid_range);
-	} else if (sync_list[i].array == &ctx->trans->keyword_resets) {
+	} else if (sync_list[i].array == (void *)&sync_trans->keyword_resets) {
 		mail_index_sync_get_keyword_reset(sync_rec, uid_range);
 	} else {
 		mail_index_sync_get_keyword_update(sync_rec, uid_range,
 						   &sync_list[i]);
 	}
 	sync_list[i].idx++;
-
-	if (mail_index_sync_rec_check(ctx->view, sync_rec) < 0)
-		return -1;
-	return 1;
+	return TRUE;
 }
 
 bool mail_index_sync_have_more(struct mail_index_sync_ctx *ctx)
@@ -616,7 +664,7 @@ void mail_index_sync_reset(struct mail_index_sync_ctx *ctx)
 
 	ctx->next_uid = 0;
 
-	sync_list = array_get_modifyable(&ctx->sync_list, &count);
+	sync_list = array_get_modifiable(&ctx->sync_list, &count);
 	for (i = 0; i < count; i++)
 		sync_list[i].idx = 0;
 }
@@ -625,75 +673,102 @@ static void mail_index_sync_end(struct mail_index_sync_ctx **_ctx)
 {
         struct mail_index_sync_ctx *ctx = *_ctx;
 
-	*_ctx = NULL;
-	mail_index_unlock(ctx->index, ctx->lock_id);
+	i_assert(ctx->index->syncing);
 
-	i_assert(ctx->index->map == NULL ||
-		 !ctx->index->map->write_to_disk);
+	*_ctx = NULL;
+
+	ctx->index->syncing = FALSE;
 	mail_transaction_log_sync_unlock(ctx->index->log);
 
 	mail_index_view_close(&ctx->view);
-	mail_index_transaction_rollback(&ctx->trans);
+	mail_index_transaction_rollback(&ctx->sync_trans);
 	if (array_is_created(&ctx->sync_list))
 		array_free(&ctx->sync_list);
 	i_free(ctx);
 }
 
+static void
+mail_index_sync_update_mailbox_offset(struct mail_index_sync_ctx *ctx)
+{
+	uint32_t seq;
+	uoff_t offset;
+
+	mail_transaction_log_view_get_prev_pos(ctx->view->log_view,
+					       &seq, &offset);
+	mail_transaction_log_set_mailbox_sync_pos(ctx->index->log, seq, offset);
+
+	/* If tail offset has changed, make sure it gets written to
+	   transaction log. */
+	if (ctx->last_tail_offset != offset)
+		ctx->ext_trans->log_updates = TRUE;
+}
+
+static bool mail_index_sync_want_index_write(struct mail_index *index)
+{
+	uint32_t log_diff;
+
+	log_diff = index->map->hdr.log_file_tail_offset -
+		index->last_read_log_file_tail_offset;
+	if (log_diff > 1024)
+		return TRUE;
+
+	if (index->need_recreate)
+		return TRUE;
+	return FALSE;
+}
+
 int mail_index_sync_commit(struct mail_index_sync_ctx **_ctx)
 {
         struct mail_index_sync_ctx *ctx = *_ctx;
 	struct mail_index *index = ctx->index;
-	const struct mail_index_header *hdr;
-	uint32_t seq;
+	uint32_t seq, next_uid;
 	uoff_t offset;
+	bool want_rotate;
 	int ret = 0;
 
-	if (mail_transaction_log_view_is_corrupted(ctx->view->log_view))
-		ret = -1;
-
-	/* we have had the transaction log locked since the beginning of sync,
-	   so only external changes could have been committed. write them to
-	   the index here as well. */
-	mail_transaction_log_get_head(index->log, &seq, &offset);
-
-	hdr = index->hdr;
-	if (ret == 0 && (hdr->log_file_seq != seq ||
-			 hdr->log_file_int_offset != offset ||
-			 hdr->log_file_ext_offset != offset ||
-			 ctx->sync_recent)) {
-		/* write all pending changes to index. */
-		if (mail_index_sync_set_log_view(ctx->view,
-						 hdr->log_file_seq,
-						 hdr->log_file_int_offset) < 0)
-			ret = -1;
-		else if (mail_index_sync_update_index(ctx, FALSE) < 0)
-			ret = -1;
+	mail_index_sync_update_mailbox_offset(ctx);
+	if (mail_cache_need_compress(index->cache)) {
+		/* if cache compression fails, we don't really care.
+		   the cache offsets are updated only if the compression was
+		   successful. */
+		(void)mail_cache_compress(index->cache, ctx->ext_trans);
 	}
 
-	if (ret == 0 && mail_cache_need_compress(index->cache)) {
-		/* if cache compression fails, we don't really care */
-		if (mail_cache_compress(index->cache, ctx->view) == 0) {
-			/* cache_offsets have changed, sync them */
-			if (mail_index_sync_set_log_view(ctx->view,
-							 seq, offset) < 0)
-				ret = -1;
-			else if (mail_index_sync_update_index(ctx, FALSE) < 0)
-				ret = -1;
+	if ((ctx->flags & MAIL_INDEX_SYNC_FLAG_DROP_RECENT) != 0) {
+		next_uid = mail_index_transaction_get_next_uid(ctx->ext_trans);
+		if (index->map->hdr.first_recent_uid < next_uid) {
+			mail_index_update_header(ctx->ext_trans,
+				offsetof(struct mail_index_header,
+					 first_recent_uid),
+				&next_uid, sizeof(next_uid), FALSE);
 		}
 	}
 
-	if (ret == 0) {
-		index->sync_log_file_seq = index->map->hdr.log_file_seq;
-		index->sync_log_file_offset =
-			index->map->hdr.log_file_int_offset;
+	if (mail_index_transaction_commit(&ctx->ext_trans, &seq, &offset) < 0) {
+		mail_index_sync_end(&ctx);
+		return -1;
 	}
 
+	/* refresh the mapping with newly committed external transactions
+	   and the synced expunges. sync using file handler here so that the
+	   expunge handlers get called. */
+	if (mail_index_map(ctx->index, MAIL_INDEX_SYNC_HANDLER_FILE) <= 0)
+		ret = -1;
+
+	want_rotate = mail_transaction_log_want_rotate(index->log);
+	if (ret == 0 &&
+	    (want_rotate || mail_index_sync_want_index_write(index))) {
+		index->need_recreate = FALSE;
+		mail_index_write(index, want_rotate);
+	}
 	mail_index_sync_end(_ctx);
 	return ret;
 }
 
 void mail_index_sync_rollback(struct mail_index_sync_ctx **ctx)
 {
+	if ((*ctx)->ext_trans != NULL)
+		mail_index_transaction_rollback(&(*ctx)->ext_trans);
 	mail_index_sync_end(ctx);
 }
 
@@ -706,9 +781,8 @@ void mail_index_sync_flags_apply(const struct mail_index_sync_rec *sync_rec,
 }
 
 bool mail_index_sync_keywords_apply(const struct mail_index_sync_rec *sync_rec,
-				    array_t *keywords)
+				    ARRAY_TYPE(keyword_indexes) *keywords)
 {
-	ARRAY_SET_TYPE(keywords, unsigned int);
 	const unsigned int *keyword_indexes;
 	unsigned int idx = sync_rec->keyword_idx;
 	unsigned int i, count;
@@ -746,21 +820,29 @@ bool mail_index_sync_keywords_apply(const struct mail_index_sync_rec *sync_rec,
 void mail_index_sync_set_corrupted(struct mail_index_sync_map_ctx *ctx,
 				   const char *fmt, ...)
 {
-	const char *error;
 	va_list va;
+	uint32_t seq;
+	uoff_t offset;
+
+	ctx->errors = TRUE;
+
+	mail_transaction_log_view_get_prev_pos(ctx->view->log_view,
+					       &seq, &offset);
+
+	if (seq < ctx->view->index->fsck_log_head_file_seq ||
+	    (seq == ctx->view->index->fsck_log_head_file_seq &&
+	     offset < ctx->view->index->fsck_log_head_file_offset)) {
+		/* be silent */
+		return;
+	}
 
 	va_start(va, fmt);
-	t_push();
-	error = t_strdup_vprintf(fmt, va);
-	if (ctx->type == MAIL_INDEX_SYNC_HANDLER_FILE) {
-		mail_transaction_log_view_set_corrupted(ctx->view->log_view,
-							"%s", error);
-	} else {
+	T_BEGIN {
 		mail_index_set_error(ctx->view->index,
-			"View synchronization from transaction log "
-			"for index %s failed: %s", ctx->view->index->filepath,
-			error);
-	}
-	t_pop();
+				     "Log synchronization error at "
+				     "seq=%u,offset=%"PRIuUOFF_T" for %s: %s",
+				     seq, offset, ctx->view->index->filepath,
+				     t_strdup_vprintf(fmt, va));
+	} T_END;
 	va_end(va);
 }
