@@ -156,7 +156,7 @@ int mail_transaction_log_view_set(struct mail_transaction_log_view *view,
 			"file_seq=%u, min_file_offset (%"PRIuUOFF_T
 			") < hdr_size (%u)",
 			min_file_seq, min_file_offset,
-			view->tail->hdr.hdr_size);
+			view->log->files->hdr.hdr_size);
 		return -1;
 	}
 
