@@ -1,5 +1,5 @@
-#ifndef __MAIL_TRANSACTION_LOG_H
-#define __MAIL_TRANSACTION_LOG_H
+#ifndef MAIL_TRANSACTION_LOG_H
+#define MAIL_TRANSACTION_LOG_H
 
 #define MAIL_TRANSACTION_LOG_MAJOR_VERSION 1
 #define MAIL_TRANSACTION_LOG_MINOR_VERSION 0
@@ -31,6 +31,10 @@ enum mail_transaction_type {
 
 	MAIL_TRANSACTION_TYPE_MASK		= 0x0000ffff,
 
+#define MAIL_TRANSACTION_EXT_MASK \
+	(MAIL_TRANSACTION_EXT_INTRO | MAIL_TRANSACTION_EXT_RESET | \
+	MAIL_TRANSACTION_EXT_HDR_UPDATE | MAIL_TRANSACTION_EXT_REC_UPDATE)
+
 	/* since we'll expunge mails based on data read from transaction log,
 	   try to avoid the possibility of corrupted transaction log expunging
 	   messages. this value is ORed to the actual MAIL_TRANSACTION_EXPUNGE
@@ -91,6 +95,8 @@ struct mail_transaction_ext_intro {
 
 struct mail_transaction_ext_reset {
 	uint32_t new_reset_id;
+	uint8_t preserve_data;
+	uint8_t unused_padding[3];
 };
 
 /* these are set for the last ext_intro */
@@ -105,30 +111,60 @@ struct mail_transaction_ext_rec_update {
 	/* unsigned char data[]; */
 };
 
-struct mail_transaction_log *
-mail_transaction_log_open_or_create(struct mail_index *index);
-struct mail_transaction_log *
-mail_transaction_log_create(struct mail_index *index);
-void mail_transaction_log_close(struct mail_transaction_log **log);
+#define LOG_IS_BEFORE(seq1, offset1, seq2, offset2) \
+	(((offset1) < (offset2) && (seq1) == (seq2)) || (seq1) < (seq2))
 
-int mail_transaction_log_move_to_memory(struct mail_transaction_log *log);
+struct mail_transaction_log *
+mail_transaction_log_alloc(struct mail_index *index);
+void mail_transaction_log_free(struct mail_transaction_log **log);
+
+/* Open the transaction log. Returns 1 if ok, 0 if file doesn't exist or it's
+   is corrupted, -1 if there was some I/O error. */
+int mail_transaction_log_open(struct mail_transaction_log *log);
+/* Create, or recreate, the transaction log. Returns 0 if ok, -1 if error. */
+int mail_transaction_log_create(struct mail_transaction_log *log, bool reset);
+/* Close all the open transactions log files. */
+void mail_transaction_log_close(struct mail_transaction_log *log);
+
+/* Notify of indexid change */
+void mail_transaction_log_indexid_changed(struct mail_transaction_log *log);
+
+/* Returns the file seq/offset where the mailbox is currently synced at.
+   Since the log is rotated only when mailbox is fully synced, the sequence
+   points always to the latest file. This function doesn't actually find the
+   latest sync position, so you'll need to use eg. log_view_set() before
+   calling this. */
+void mail_transaction_log_get_mailbox_sync_pos(struct mail_transaction_log *log,
+					       uint32_t *file_seq_r,
+					       uoff_t *file_offset_r);
+/* Set the current mailbox sync position. file_seq must always be the latest
+   log file's sequence. The offset written automatically to the log when
+   other transactions are being written. */
+void mail_transaction_log_set_mailbox_sync_pos(struct mail_transaction_log *log,
+					       uint32_t file_seq,
+					       uoff_t file_offset);
 
 struct mail_transaction_log_view *
 mail_transaction_log_view_open(struct mail_transaction_log *log);
 void mail_transaction_log_view_close(struct mail_transaction_log_view **view);
 
-/* Set view boundaries. Returns -1 if error, 0 if files are lost, 1 if ok. */
-int
-mail_transaction_log_view_set(struct mail_transaction_log_view *view,
-			      uint32_t min_file_seq, uoff_t min_file_offset,
-			      uint32_t max_file_seq, uoff_t max_file_offset,
-			      enum mail_transaction_type type_mask);
+/* Set view boundaries. Returns -1 if error, 0 if files are lost, 1 if ok.
+   reset_r=TRUE if the whole index should be reset before applying any
+   changes. */
+int mail_transaction_log_view_set(struct mail_transaction_log_view *view,
+				  uint32_t min_file_seq, uoff_t min_file_offset,
+				  uint32_t max_file_seq, uoff_t max_file_offset,
+				  bool *reset_r);
+/* Clear the view. Keep oldest_file_seq log referenced so we don't get
+   desynced. */
+void mail_transaction_log_view_clear(struct mail_transaction_log_view *view,
+				     uint32_t oldest_file_seq);
 
 /* Read next transaction record from current position. The position is updated.
    Returns -1 if error, 0 if we're at end of the view, 1 if ok. */
 int mail_transaction_log_view_next(struct mail_transaction_log_view *view,
 				   const struct mail_transaction_header **hdr_r,
-				   const void **data_r, bool *skipped_r);
+				   const void **data_r);
 /* Seek to given position within view. Must be inside the view's range. */
 void mail_transaction_log_view_seek(struct mail_transaction_log_view *view,
 				    uint32_t seq, uoff_t offset);
@@ -139,12 +175,14 @@ void
 mail_transaction_log_view_get_prev_pos(struct mail_transaction_log_view *view,
 				       uint32_t *file_seq_r,
 				       uoff_t *file_offset_r);
+/* Returns TRUE if we're at the end of the view window. */
+bool mail_transaction_log_view_is_last(struct mail_transaction_log_view *view);
 
 /* Marks the log file in current position to be corrupted. */
 void
 mail_transaction_log_view_set_corrupted(struct mail_transaction_log_view *view,
 					const char *fmt, ...)
-	__attr_format__(2, 3);
+	ATTR_FORMAT(2, 3);
 bool
 mail_transaction_log_view_is_corrupted(struct mail_transaction_log_view *view);
 
@@ -170,4 +208,8 @@ void mail_transaction_log_get_head(struct mail_transaction_log *log,
 bool mail_transaction_log_is_head_prev(struct mail_transaction_log *log,
 				       uint32_t file_seq, uoff_t file_offset);
 
+/* Move currently opened log head file to memory (called by
+   mail_index_move_to_memory()) */
+void mail_transaction_log_move_to_memory(struct mail_transaction_log *log);
+
 #endif
