@@ -1,11 +1,20 @@
-#ifndef __SQL_API_PRIVATE_H
-#define __SQL_API_PRIVATE_H
+#ifndef SQL_API_PRIVATE_H
+#define SQL_API_PRIVATE_H
 
 #include "sql-api.h"
+#include "module-context.h"
 
-struct sql_db {
-	const char *name;
+struct sql_db_module_register {
+	unsigned int id;
+};
 
+union sql_db_module_context {
+	struct sql_db_module_register *reg;
+};
+
+extern struct sql_db_module_register sql_db_module_register;
+
+struct sql_db_vfuncs {
 	struct sql_db *(*init)(const char *connect_string);
 	void (*deinit)(struct sql_db *db);
 
@@ -29,9 +38,13 @@ struct sql_db {
 	void (*update)(struct sql_transaction_context *ctx, const char *query);
 };
 
-struct sql_result {
-	struct sql_db *db;
+struct sql_db {
+	const char *name;
+	struct sql_db_vfuncs v;
+	ARRAY_DEFINE(module_contexts, union sql_db_module_context *);
+};
 
+struct sql_result_vfuncs {
 	void (*free)(struct sql_result *result);
 	int (*next_row)(struct sql_result *result);
 
@@ -42,11 +55,32 @@ struct sql_result {
 
 	const char *(*get_field_value)(struct sql_result *result,
 				       unsigned int idx);
+	const unsigned char *
+		(*get_field_value_binary)(struct sql_result *result,
+					  unsigned int idx,
+					  size_t *size_r);
 	const char *(*find_field_value)(struct sql_result *result,
 					const char *field_name);
 	const char *const *(*get_values)(struct sql_result *result);
 
 	const char *(*get_error)(struct sql_result *result);
+};
+
+struct sql_field_map {
+	enum sql_field_type type;
+	size_t offset;
+};
+
+struct sql_result {
+	struct sql_result_vfuncs v;
+
+	struct sql_db *db;
+	const struct sql_field_def *fields;
+
+	unsigned int map_size;
+	struct sql_field_map *map;
+	void *fetch_dest;
+	size_t fetch_dest_size;
 
 	unsigned int callback:1;
 };
@@ -55,7 +89,9 @@ struct sql_transaction_context {
 	struct sql_db *db;
 };
 
-extern array_t ARRAY_DEFINE(sql_drivers, const struct sql_db *);
+ARRAY_DEFINE_TYPE(sql_drivers, const struct sql_db *);
+
+extern ARRAY_TYPE(sql_drivers) sql_drivers;
 extern struct sql_result sql_not_connected_result;
 
 #endif
