@@ -99,6 +99,10 @@ static int dbox_create(struct mail_storage *_storage, const char *data,
 	list_set.mail_storage_flags = &_storage->flags;
 	list_set.lock_method = &_storage->lock_method;
 
+	/* FIXME: ugly */
+	if (strcmp(layout, "maildir++") == 0 || strcmp(layout, "imapdir") == 0)
+		list_set.maildir_name = "";
+
 	if ((_storage->flags & MAIL_STORAGE_FLAG_NO_AUTOCREATE) != 0) {
 		if (stat(list_set.root_dir, &st) < 0) {
 			if (errno == ENOENT) {
@@ -451,6 +455,11 @@ dbox_list_delete_mailbox(struct mailbox_list *list, const char *name)
 			if (dbox_delete_nonrecursive(list, alt_path, name) < 0)
 				return -1;
 		}
+		if (*list->set.maildir_name == '\0') {
+			/* everything was in the one directory that was
+			   already deleted succesfully. */
+			return 0;
+		}
 		/* try to delete the directory also */
 		deleted = TRUE;
 		path = mailbox_list_get_path(list, name,
