@@ -1,17 +1,18 @@
-/* Copyright (C) 2002 Timo Sirainen */
+/* Copyright (c) 2002-2008 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
+#include "ioloop.h"
+#include "array.h"
 #include "istream.h"
 #include "utc-offset.h"
 #include "str.h"
 #include "message-address.h"
 #include "message-date.h"
-#include "message-body-search.h"
-#include "message-header-search.h"
+#include "message-search.h"
 #include "message-parser.h"
-#include "imap-date.h"
 #include "index-storage.h"
 #include "index-mail.h"
+#include "index-sort.h"
 #include "mail-search.h"
 
 #include <stdlib.h>
@@ -20,22 +21,27 @@
 #define TXT_UNKNOWN_CHARSET "[BADCHARSET] Unknown charset"
 #define TXT_INVALID_SEARCH_KEY "Invalid search key"
 
+#define SEARCH_NONBLOCK_COUNT 20
+#define SEARCH_NOTIFY_INTERVAL_SECS 10
+
 struct index_search_context {
         struct mail_search_context mail_ctx;
 	struct mail_index_view *view;
 	struct index_mailbox *ibox;
-	char *charset;
-	struct mail_search_arg *args;
 
 	uint32_t seq1, seq2;
 	struct mail *mail;
 	struct index_mail *imail;
 
-	pool_t hdr_pool;
+	pool_t search_pool;
 	const char *error;
 
+	struct timeval search_start_time, last_notify;
+
 	unsigned int failed:1;
+	unsigned int sorted:1;
 	unsigned int have_seqsets:1;
+	unsigned int have_flags_or_keywords:1;
 };
 
 struct search_header_context {
@@ -55,10 +61,32 @@ struct search_body_context {
 	const struct message_part *part;
 };
 
-static int search_parse_msgset_args(struct index_mailbox *ibox,
-				    const struct mail_index_header *hdr,
-				    struct mail_search_arg *args,
-				    uint32_t *seq1_r, uint32_t *seq2_r);
+static const enum message_header_parser_flags hdr_parser_flags =
+	MESSAGE_HEADER_PARSER_FLAG_CLEAN_ONELINE;
+
+static void search_parse_msgset_args(const struct mail_index_header *hdr,
+				     struct mail_search_arg *args,
+				     uint32_t *seq1_r, uint32_t *seq2_r);
+
+static void search_init_arg(struct mail_search_arg *arg,
+			    struct index_search_context *ctx)
+{
+	switch (arg->type) {
+	case SEARCH_SEQSET:
+		ctx->have_seqsets = TRUE;
+		break;
+	case SEARCH_FLAGS:
+	case SEARCH_KEYWORDS:
+		ctx->have_flags_or_keywords = TRUE;
+		break;
+	case SEARCH_ALL:
+		if (!arg->not)
+			arg->match_always = TRUE;
+		break;
+	default:
+		break;
+	}
+}
 
 static int seqset_contains(struct mail_search_seqset *set, uint32_t seq)
 {
@@ -71,81 +99,9 @@ static int seqset_contains(struct mail_search_seqset *set, uint32_t seq)
 	return FALSE;
 }
 
-static uoff_t str_to_uoff_t(const char *str)
+static void search_seqset_arg(struct mail_search_arg *arg,
+			      struct index_search_context *ctx)
 {
-	uoff_t num;
-
-	num = 0;
-	while (*str != '\0') {
-		if (*str < '0' || *str > '9')
-			return 0;
-
-		num = num*10 + (*str - '0');
-		str++;
-	}
-
-	return num;
-}
-
-/* Returns >0 = matched, 0 = not matched, -1 = unknown */
-static int search_arg_match_index(struct index_mail *imail,
-				  enum mail_search_arg_type type,
-				  const char *value)
-{
-	const struct mail_index_record *rec = imail->data.rec;
-	const char *const *keywords;
-
-	switch (type) {
-	/* flags */
-	case SEARCH_ANSWERED:
-		return rec->flags & MAIL_ANSWERED;
-	case SEARCH_DELETED:
-		return rec->flags & MAIL_DELETED;
-	case SEARCH_DRAFT:
-		return rec->flags & MAIL_DRAFT;
-	case SEARCH_FLAGGED:
-		return rec->flags & MAIL_FLAGGED;
-	case SEARCH_SEEN:
-		return rec->flags & MAIL_SEEN;
-	case SEARCH_RECENT:
-		return mail_get_flags(&imail->mail.mail) & MAIL_RECENT;
-	case SEARCH_KEYWORD:
-		keywords = mail_get_keywords(&imail->mail.mail);
-		if (keywords != NULL) {
-			while (*keywords != NULL) {
-				if (strcasecmp(*keywords, value) == 0)
-					return 1;
-				keywords++;
-			}
-		}
-		return 0;
-
-	default:
-		return -1;
-	}
-}
-
-static void search_init_seqset_arg(struct mail_search_arg *arg, void *context)
-{
-	struct index_search_context *ctx = context;
-
-	switch (arg->type) {
-	case SEARCH_SEQSET:
-		ctx->have_seqsets = TRUE;
-		break;
-	case SEARCH_ALL:
-		if (!arg->not)
-			arg->match_always = TRUE;
-		break;
-	default:
-		break;
-	}
-}
-
-static void search_seqset_arg(struct mail_search_arg *arg, void *context)
-{
-	struct index_search_context *ctx = context;
-
 	if (arg->type == SEARCH_SEQSET) {
 		if (seqset_contains(arg->value.seqset, ctx->mail_ctx.seq))
 			ARG_SET_RESULT(arg, 1);
@@ -154,18 +110,66 @@ static void search_seqset_arg(struct mail_search_arg *arg, void *context)
 	}
 }
 
-static void search_index_arg(struct mail_search_arg *arg, void *context)
+static int search_arg_match_keywords(struct index_search_context *ctx,
+				     struct mail_search_arg *arg)
 {
-	struct index_search_context *ctx = context;
+	ARRAY_TYPE(keyword_indexes) keyword_indexes_arr;
+	const struct mail_keywords *search_kws = arg->value.keywords;
+	const unsigned int *keyword_indexes;
+	unsigned int i, j, count;
 
-	if (ctx->imail->data.rec == NULL) {
-		/* expunged message */
-		ARG_SET_RESULT(arg, 0);
-		return;
+	t_array_init(&keyword_indexes_arr, 128);
+	mail_index_lookup_keywords(ctx->view, ctx->mail_ctx.seq,
+				   &keyword_indexes_arr);
+	keyword_indexes = array_get(&keyword_indexes_arr, &count);
+
+	/* there probably aren't many keywords, so O(n*m) for now */
+	for (i = 0; i < search_kws->count; i++) {
+		for (j = 0; j < count; j++) {
+			if (search_kws->idx[i] == keyword_indexes[j])
+				break;
+		}
+		if (j == count)
+			return 0;
 	}
+	return 1;
+}
 
-	switch (search_arg_match_index(ctx->imail, arg->type,
-				       arg->value.str)) {
+/* Returns >0 = matched, 0 = not matched, -1 = unknown */
+static int search_arg_match_index(struct index_search_context *ctx,
+				  struct mail_search_arg *arg,
+				  const struct mail_index_record *rec)
+{
+	enum mail_flags flags;
+	int ret;
+
+	switch (arg->type) {
+	case SEARCH_FLAGS:
+		/* recent flag shouldn't be set, but indexes from v1.0.x
+		   may contain it. */
+		flags = rec->flags & ~MAIL_RECENT;
+		if ((arg->value.flags & MAIL_RECENT) != 0 &&
+		    index_mailbox_is_recent(ctx->ibox, rec->uid))
+			flags |= MAIL_RECENT;
+		return (flags & arg->value.flags) == arg->value.flags;
+	case SEARCH_KEYWORDS:
+		T_BEGIN {
+			ret = search_arg_match_keywords(ctx, arg);
+		} T_END;
+		return ret;
+
+	default:
+		return -1;
+	}
+}
+
+static void search_index_arg(struct mail_search_arg *arg,
+			     struct index_search_context *ctx)
+{
+	const struct mail_index_record *rec;
+
+	rec = mail_index_lookup(ctx->view, ctx->mail_ctx.seq);
+	switch (search_arg_match_index(ctx, arg, rec)) {
 	case -1:
 		/* unknown */
 		break;
@@ -180,37 +184,32 @@ static void search_index_arg(struct mail_search_arg *arg, void *context)
 
 /* Returns >0 = matched, 0 = not matched, -1 = unknown */
 static int search_arg_match_cached(struct index_search_context *ctx,
-				   enum mail_search_arg_type type,
-				   const char *value)
+				   struct mail_search_arg *arg)
 {
 	struct tm *tm;
-	time_t date, search_time;
-	uoff_t virtual_size, search_size;
+	uoff_t virtual_size;
+	time_t date;
 	int timezone_offset;
 
-	switch (type) {
+	switch (arg->type) {
 	/* internal dates */
 	case SEARCH_BEFORE:
 	case SEARCH_ON:
 	case SEARCH_SINCE:
-		date = mail_get_received_date(ctx->mail);
-		if (date == (time_t)-1)
+		if (mail_get_received_date(ctx->mail, &date) < 0)
 			return -1;
 
 		tm = localtime(&date);
 		date += utc_offset(tm, date)*60;
 
-		if (!imap_parse_date(value, &search_time))
-			return 0;
-
-		switch (type) {
+		switch (arg->type) {
 		case SEARCH_BEFORE:
-			return date < search_time;
+			return date < arg->value.time;
 		case SEARCH_ON:
-			return date >= search_time &&
-				date < search_time + 3600*24;
+			return date >= arg->value.time &&
+				date < arg->value.time + 3600*24;
 		case SEARCH_SINCE:
-			return date >= search_time;
+			return date >= arg->value.time;
 		default:
 			/* unreachable */
 			break;
@@ -222,22 +221,18 @@ static int search_arg_match_cached(struct index_search_context *ctx,
 	case SEARCH_SENTSINCE:
 		/* NOTE: RFC-3501 specifies that timezone is ignored
 		   in searches. date is returned as UTC, so change it. */
-		date = mail_get_date(ctx->mail, &timezone_offset);
-		if (date == (time_t)-1)
+		if (mail_get_date(ctx->mail, &date, &timezone_offset) < 0)
 			return -1;
 		date += timezone_offset * 60;
 
-		if (!imap_parse_date(value, &search_time))
-			return 0;
-
-		switch (type) {
+		switch (arg->type) {
 		case SEARCH_SENTBEFORE:
-			return date < search_time;
+			return date < arg->value.time;
 		case SEARCH_SENTON:
-			return date >= search_time &&
-				date < search_time + 3600*24;
+			return date >= arg->value.time &&
+				date < arg->value.time + 3600*24;
 		case SEARCH_SENTSINCE:
-			return date >= search_time;
+			return date >= arg->value.time;
 		default:
 			/* unreachable */
 			break;
@@ -246,27 +241,23 @@ static int search_arg_match_cached(struct index_search_context *ctx,
 	/* sizes */
 	case SEARCH_SMALLER:
 	case SEARCH_LARGER:
-		virtual_size = mail_get_virtual_size(ctx->mail);
-		if (virtual_size == (uoff_t)-1)
+		if (mail_get_virtual_size(ctx->mail, &virtual_size) < 0)
 			return -1;
 
-		search_size = str_to_uoff_t(value);
-		if (type == SEARCH_SMALLER)
-			return virtual_size < search_size;
+		if (arg->type == SEARCH_SMALLER)
+			return virtual_size < arg->value.size;
 		else
-			return virtual_size > search_size;
+			return virtual_size > arg->value.size;
 
 	default:
 		return -1;
 	}
 }
 
-static void search_cached_arg(struct mail_search_arg *arg, void *context)
+static void search_cached_arg(struct mail_search_arg *arg,
+			      struct index_search_context *ctx)
 {
-	struct index_search_context *ctx = context;
-
-	switch (search_arg_match_cached(ctx, arg->type,
-					arg->value.str)) {
+	switch (search_arg_match_cached(ctx, arg)) {
 	case -1:
 		/* unknown */
 		break;
@@ -279,18 +270,15 @@ static void search_cached_arg(struct mail_search_arg *arg, void *context)
 	}
 }
 
-static int search_sent(enum mail_search_arg_type type, const char *search_value,
+static int search_sent(enum mail_search_arg_type type, time_t search_time,
 		       const unsigned char *sent_value, size_t sent_value_len)
 {
-	time_t search_time, sent_time;
+	time_t sent_time;
 	int timezone_offset;
 
 	if (sent_value == NULL)
 		return 0;
 
-	if (!imap_parse_date(search_value, &search_time))
-		return 0;
-
 	/* NOTE: RFC-3501 specifies that timezone is ignored
 	   in searches. sent_time is returned as UTC, so change it. */
 	if (!message_date_parse(sent_value, sent_value_len,
@@ -311,37 +299,62 @@ static int search_sent(enum mail_search_arg_type type, const char *search_value,
 	}
 }
 
-static struct header_search_context *
-search_header_context(struct index_search_context *ctx,
-		      struct mail_search_arg *arg)
+static struct message_search_context *
+msg_search_arg_context(struct index_search_context *ctx,
+		       struct mail_search_arg *arg)
 {
-	bool unknown_charset;
+	struct message_search_context *arg_ctx = arg->context;
+	enum message_search_flags flags;
+	int ret;
 
-	if (arg->context != NULL) {
-                message_header_search_reset(arg->context);
-		return arg->context;
+	if (arg_ctx != NULL)
+		return arg_ctx;
+
+	if (ctx->search_pool == NULL)
+		ctx->search_pool = pool_alloconly_create("search pool", 8192);
+
+	flags = (arg->type == SEARCH_BODY || arg->type == SEARCH_BODY_FAST) ?
+		MESSAGE_SEARCH_FLAG_SKIP_HEADERS : 0;
+
+	ret = message_search_init(ctx->search_pool, arg->value.str,
+				  ctx->mail_ctx.charset, flags,
+				  &arg_ctx);
+	if (ret > 0) {
+		arg->context = arg_ctx;
+		return arg_ctx;
 	}
-
-	if (ctx->hdr_pool == NULL) {
-		ctx->hdr_pool =
-			pool_alloconly_create("message_header_search", 8192);
-	}
-
-	arg->context = message_header_search_init(ctx->hdr_pool, arg->value.str,
-						  ctx->charset,
-						  &unknown_charset);
-	if (arg->context == NULL) {
-		ctx->error = unknown_charset ?
-			TXT_UNKNOWN_CHARSET : TXT_INVALID_SEARCH_KEY;
-	}
-
-	return arg->context;
+	if (ret == 0)
+		ctx->error = TXT_UNKNOWN_CHARSET;
+	else
+		ctx->error = TXT_INVALID_SEARCH_KEY;
+	return NULL;
 }
 
-static void search_header_arg(struct mail_search_arg *arg, void *context)
+static void compress_lwsp(string_t *dest, const unsigned char *src,
+			  unsigned int src_len)
 {
-	struct search_header_context *ctx = context;
-        struct header_search_context *hdr_search_ctx;
+	unsigned int i;
+	bool prev_lwsp = TRUE;
+
+	for (i = 0; i < src_len; i++) {
+		if (IS_LWSP(src[i])) {
+			if (!prev_lwsp) {
+				prev_lwsp = TRUE;
+				str_append_c(dest, ' ');
+			}
+		} else {
+			prev_lwsp = FALSE;
+			str_append_c(dest, src[i]);
+		}
+	}
+}
+
+static void search_header_arg(struct mail_search_arg *arg,
+			      struct search_header_context *ctx)
+{
+        struct message_search_context *msg_search_ctx;
+	struct message_block block;
+	struct message_header_line hdr;
 	int ret;
 
 	/* first check that the field name matches to argument. */
@@ -355,7 +368,7 @@ static void search_header_arg(struct mail_search_arg *arg, void *context)
 				ctx->hdr->use_full_value = TRUE;
 				return;
 			}
-			ret = search_sent(arg->type, arg->value.str,
+			ret = search_sent(arg->type, arg->value.time,
 					  ctx->hdr->full_value,
 					  ctx->hdr->full_value_len);
 			ARG_SET_RESULT(arg, ret);
@@ -364,13 +377,11 @@ static void search_header_arg(struct mail_search_arg *arg, void *context)
 
 	case SEARCH_HEADER:
 	case SEARCH_HEADER_ADDRESS:
+	case SEARCH_HEADER_COMPRESS_LWSP:
 		ctx->custom_header = TRUE;
 
 		if (strcasecmp(ctx->hdr->name, arg->hdr_field_name) != 0)
 			return;
-	case SEARCH_TEXT:
-		/* TEXT goes through all headers */
-		ctx->custom_header = TRUE;
 		break;
 	default:
 		return;
@@ -378,48 +389,65 @@ static void search_header_arg(struct mail_search_arg *arg, void *context)
 
 	if (arg->value.str[0] == '\0') {
 		/* we're just testing existence of the field. always matches. */
-		ret = 1;
-	} else {
-		if (ctx->hdr->continues) {
-			ctx->hdr->use_full_value = TRUE;
-			return;
-		}
+		ARG_SET_RESULT(arg, 1);
+		return;
+	}
 
-		t_push();
+	if (ctx->hdr->continues) {
+		ctx->hdr->use_full_value = TRUE;
+		return;
+	}
 
-		hdr_search_ctx = search_header_context(ctx->index_context, arg);
-		if (hdr_search_ctx == NULL)
-			ret = 0;
-		else if (arg->type == SEARCH_HEADER_ADDRESS) {
+	memset(&block, 0, sizeof(block));
+
+	/* We're searching only for values, so drop header name and middle
+	   parts. We use header searching so that MIME words will be decoded. */
+	hdr = *ctx->hdr;
+	hdr.name = ""; hdr.name_len = 0;
+	hdr.middle_len = 0;
+	block.hdr = &hdr;
+
+	msg_search_ctx = msg_search_arg_context(ctx->index_context, arg);
+	if (msg_search_ctx == NULL)
+		return;
+
+	T_BEGIN {
+		struct message_address *addr;
+		string_t *str;
+
+		switch (arg->type) {
+		case SEARCH_HEADER:
+			/* simple match */
+			break;
+		case SEARCH_HEADER_ADDRESS:
 			/* we have to match against normalized address */
-			struct message_address *addr;
-			string_t *str;
-
 			addr = message_address_parse(pool_datastack_create(),
 						     ctx->hdr->full_value,
 						     ctx->hdr->full_value_len,
 						     (unsigned int)-1, TRUE);
 			str = t_str_new(ctx->hdr->value_len);
 			message_address_write(str, addr);
-			ret = message_header_search(str_data(str), str_len(str),
-						    hdr_search_ctx) ? 1 : 0;
-		} else {
-			ret = message_header_search(ctx->hdr->full_value,
-						    ctx->hdr->full_value_len,
-						    hdr_search_ctx) ? 1 : 0;
+			hdr.value = hdr.full_value = str_data(str);
+			hdr.value_len = hdr.full_value_len = str_len(str);
+			break;
+		case SEARCH_HEADER_COMPRESS_LWSP:
+			/* convert LWSP to single spaces */
+			str = t_str_new(hdr.full_value_len);
+			compress_lwsp(str, hdr.full_value, hdr.full_value_len);
+			hdr.value = hdr.full_value = str_data(str);
+			hdr.value_len = hdr.full_value_len = str_len(str);
+			break;
+		default:
+			i_unreached();
 		}
-		t_pop();
-	}
+		ret = message_search_more(msg_search_ctx, &block) ? 1 : 0;
+	} T_END;
 
-	if (ret == 1 ||
-	    (arg->type != SEARCH_TEXT && arg->type != SEARCH_HEADER)) {
-		/* set only when we definitely know if it's a match */
-		ARG_SET_RESULT(arg, ret);
-	}
+	ARG_SET_RESULT(arg, ret);
 }
 
 static void search_header_unmatch(struct mail_search_arg *arg,
-				  void *context __attr_unused__)
+				  void *context ATTR_UNUSED)
 {
 	switch (arg->type) {
 	case SEARCH_SENTBEFORE:
@@ -433,6 +461,7 @@ static void search_header_unmatch(struct mail_search_arg *arg,
 		break;
 	case SEARCH_HEADER:
 	case SEARCH_HEADER_ADDRESS:
+	case SEARCH_HEADER_COMPRESS_LWSP:
 		ARG_SET_RESULT(arg, 0);
 		break;
 	default:
@@ -440,11 +469,9 @@ static void search_header_unmatch(struct mail_search_arg *arg,
 	}
 }
 
-static void search_header(struct message_part *part __attr_unused__,
-                          struct message_header_line *hdr, void *context)
+static void search_header(struct message_header_line *hdr,
+			  struct search_header_context *ctx)
 {
-	struct search_header_context *ctx = context;
-
 	if (hdr == NULL) {
 		/* end of headers, mark all unknown SEARCH_HEADERs unmatched */
 		mail_search_args_foreach(ctx->args, search_header_unmatch, ctx);
@@ -465,48 +492,41 @@ static void search_header(struct message_part *part __attr_unused__,
 	}
 }
 
-static void search_body(struct mail_search_arg *arg, void *context)
+static void search_body(struct mail_search_arg *arg,
+			struct search_body_context *ctx)
 {
-	struct search_body_context *ctx = context;
-        enum message_body_search_error error;
+	struct message_search_context *msg_search_ctx;
 	int ret;
-	bool retry = FALSE;
 
 	if (ctx->index_ctx->error != NULL)
 		return;
 
-	if (arg->type != SEARCH_TEXT && arg->type != SEARCH_BODY)
+	switch (arg->type) {
+	case SEARCH_BODY:
+	case SEARCH_BODY_FAST:
+	case SEARCH_TEXT:
+	case SEARCH_TEXT_FAST:
+		break;
+	default:
 		return;
+	}
+
+	msg_search_ctx = msg_search_arg_context(ctx->index_ctx, arg);
+	if (msg_search_ctx == NULL) {
+		ARG_SET_RESULT(arg, 0);
+		return;
+	}
 
-__retry:
 	i_stream_seek(ctx->input, 0);
-	ret = message_body_search(arg->value.str, ctx->index_ctx->charset,
-				  ctx->input, ctx->part,
-				  arg->type == SEARCH_TEXT, &error);
+	ret = message_search_msg(msg_search_ctx, ctx->input, ctx->part);
+	if (ret < 0 && ctx->input->stream_errno == 0) {
+		/* try again without cached parts */
+		mail_set_cache_corrupted(ctx->index_ctx->mail,
+					 MAIL_FETCH_MESSAGE_PARTS);
 
-	if (ret < 0) {
-		switch (error) {
-		case MESSAGE_BODY_SEARCH_ERROR_UNKNOWN_CHARSET:
-			ctx->index_ctx->error = TXT_UNKNOWN_CHARSET;
-			break;
-		case MESSAGE_BODY_SEARCH_ERROR_INVALID_KEY:
-			ctx->index_ctx->error = TXT_INVALID_SEARCH_KEY;
-			break;
-		case MESSAGE_BODY_SEARCH_ERROR_MESSAGE_PART_BROKEN:
-			if (retry)
-				i_panic("Couldn't fix broken body structure");
-
-			mail_cache_set_corrupted(ctx->index_ctx->ibox->cache,
-				"Broken message structure for mail UID %u",
-				ctx->index_ctx->mail->uid);
-
-			/* get the body parts, and try again */
-			ctx->index_ctx->imail->data.parts = NULL;
-			ctx->part = mail_get_parts(ctx->index_ctx->mail);
-
-			retry = TRUE;
-			goto __retry;
-		}
+		i_stream_seek(ctx->input, 0);
+		ret = message_search_msg(msg_search_ctx, ctx->input, NULL);
+		i_assert(ret >= 0 || ctx->input->stream_errno != 0);
 	}
 
 	ARG_SET_RESULT(arg, ret > 0);
@@ -533,8 +553,7 @@ static bool search_arg_match_text(struct mail_search_arg *args,
 
 		if (headers == NULL) {
 			headers_ctx = NULL;
-			input = mail_get_stream(ctx->mail, NULL, NULL);
-			if (input == NULL)
+			if (mail_get_stream(ctx->mail, NULL, NULL, &input) < 0)
 				return FALSE;
 		} else {
 			/* FIXME: do this once in init */
@@ -542,8 +561,8 @@ static bool search_arg_match_text(struct mail_search_arg *args,
 			headers_ctx =
 				mailbox_header_lookup_init(&ctx->ibox->box,
 							   headers);
-			input = mail_get_header_stream(ctx->mail, headers_ctx);
-			if (input == NULL) {
+			if (mail_get_header_stream(ctx->mail, headers_ctx,
+						   &input) < 0) {
 				mailbox_header_lookup_deinit(&headers_ctx);
 				return FALSE;
 			}
@@ -553,18 +572,19 @@ static bool search_arg_match_text(struct mail_search_arg *args,
 		hdr_ctx.index_context = ctx;
 		hdr_ctx.custom_header = TRUE;
 		hdr_ctx.args = args;
-		hdr_ctx.parse_headers = headers == NULL;
+		hdr_ctx.parse_headers = headers == NULL &&
+			index_mail_want_parse_headers(ctx->imail);
 
-		index_mail_parse_header_init(ctx->imail, headers_ctx);
-		message_parse_header(NULL, input, NULL,
+		if (hdr_ctx.parse_headers)
+			index_mail_parse_header_init(ctx->imail, headers_ctx);
+		message_parse_header(input, NULL, hdr_parser_flags,
 				     search_header, &hdr_ctx);
 		if (headers_ctx != NULL)
 			mailbox_header_lookup_deinit(&headers_ctx);
 	} else {
 		struct message_size hdr_size;
 
-		input = mail_get_stream(ctx->mail, &hdr_size, NULL);
-		if (input == NULL)
+		if (mail_get_stream(ctx->mail, &hdr_size, NULL, &input) < 0)
 			return FALSE;
 
 		i_stream_seek(input, hdr_size.physical_size);
@@ -576,16 +596,15 @@ static bool search_arg_match_text(struct mail_search_arg *args,
 		memset(&body_ctx, 0, sizeof(body_ctx));
 		body_ctx.index_ctx = ctx;
 		body_ctx.input = input;
-		body_ctx.part = mail_get_parts(ctx->mail);
+		(void)mail_get_parts(ctx->mail, &body_ctx.part);
 
 		mail_search_args_foreach(args, search_body, &body_ctx);
 	}
 	return TRUE;
 }
 
-static int search_msgset_fix_limits(struct index_mailbox *ibox,
-				    const struct mail_index_header *hdr,
-				    struct mail_search_seqset *set, bool not)
+static bool search_msgset_fix_limits(const struct mail_index_header *hdr,
+				     struct mail_search_seqset *set, bool not)
 {
 	if (set == NULL)
 		return FALSE;
@@ -598,7 +617,7 @@ static int search_msgset_fix_limits(struct index_mailbox *ibox,
 					continue;
 
 				/* completely outside our range */
-				return 0;
+				return FALSE;
 			}
 			/* either seq1 or seq2 is '*', so the last message is
 			   in range. */
@@ -608,12 +627,11 @@ static int search_msgset_fix_limits(struct index_mailbox *ibox,
 			set->seq2 = hdr->messages_count;
 
 		if (set->seq1 == 0 || set->seq2 == 0) {
-			mail_storage_set_syntax_error(ibox->box.storage,
-						      "Invalid messageset");
-			return -1;
+			/* this shouldn't happen. treat as nonexisting. */
+			return FALSE;
 		}
 	}
-	return 1;
+	return TRUE;
 }
 
 static int mail_search_seqset_cmp(const void *p1, const void *p2)
@@ -663,20 +681,18 @@ static void search_msgset_compress(struct mail_search_seqset *set,
 	*last_r = cur;
 }
 
-static int search_msgset_fix(struct index_mailbox *ibox,
-                             const struct mail_index_header *hdr,
-			     struct mail_search_seqset **set_p,
-			     uint32_t *seq1_r, uint32_t *seq2_r, bool not)
+static void search_msgset_fix(const struct mail_index_header *hdr,
+			      struct mail_search_seqset **set_p,
+			      uint32_t *seq1_r, uint32_t *seq2_r, bool not)
 {
 	struct mail_search_seqset *set = *set_p;
 	struct mail_search_seqset *last;
 	uint32_t min_seq, max_seq;
-	int ret;
 
-	if ((ret = search_msgset_fix_limits(ibox, hdr, set, not)) <= 0) {
+	if (!search_msgset_fix_limits(hdr, set, not)) {
 		*seq1_r = (uint32_t)-1;
 		*seq2_r = 0;
-		return ret;
+		return;
 	}
 	set = *set_p = search_msgset_sort(set);
 	search_msgset_compress(set, &last);
@@ -691,7 +707,7 @@ static int search_msgset_fix(struct index_mailbox *ibox,
 		if (min_seq > max_seq) {
 			*seq1_r = (uint32_t)-1;
 			*seq2_r = 0;
-			return 0;
+			return;
 		}
 	}
 
@@ -699,13 +715,11 @@ static int search_msgset_fix(struct index_mailbox *ibox,
 		*seq1_r = min_seq;
 	if (*seq2_r > max_seq)
 		*seq2_r = max_seq;
-	return 0;
 }
 
-static int search_or_parse_msgset_args(struct index_mailbox *ibox,
-				       const struct mail_index_header *hdr,
-				       struct mail_search_arg *args,
-				       uint32_t *seq1_r, uint32_t *seq2_r)
+static void search_or_parse_msgset_args(const struct mail_index_header *hdr,
+					struct mail_search_arg *args,
+					uint32_t *seq1_r, uint32_t *seq2_r)
 {
 	uint32_t seq1, seq2, min_seq1 = 0, max_seq2 = 0;
 
@@ -714,20 +728,15 @@ static int search_or_parse_msgset_args(struct index_mailbox *ibox,
 
 		if (args->type == SEARCH_SUB) {
 			i_assert(!args->not);
-			if (search_parse_msgset_args(ibox, hdr,
-						     args->value.subargs,
-						     &seq1, &seq2) < 0)
-				return -1;
+			search_parse_msgset_args(hdr, args->value.subargs,
+						 &seq1, &seq2);
 		} else if (args->type == SEARCH_OR) {
 			i_assert(!args->not);
-			if (search_or_parse_msgset_args(ibox, hdr,
-							args->value.subargs,
-							&seq1, &seq2) < 0)
-				return -1;
+			search_or_parse_msgset_args(hdr, args->value.subargs,
+						    &seq1, &seq2);
 		} else if (args->type == SEARCH_SEQSET) {
-			if (search_msgset_fix(ibox, hdr, &args->value.seqset,
-					      &seq1, &seq2, args->not) < 0)
-				return -1;
+			search_msgset_fix(hdr, &args->value.seqset,
+					  &seq1, &seq2, args->not);
 		}
 
 		if (min_seq1 == 0) {
@@ -746,158 +755,103 @@ static int search_or_parse_msgset_args(struct index_mailbox *ibox,
 		*seq1_r = min_seq1;
 	if (max_seq2 < *seq2_r)
 		*seq2_r = max_seq2;
-	return 0;
 }
 
-static int search_parse_msgset_args(struct index_mailbox *ibox,
-				    const struct mail_index_header *hdr,
-				    struct mail_search_arg *args,
-				    uint32_t *seq1_r, uint32_t *seq2_r)
+static void search_parse_msgset_args(const struct mail_index_header *hdr,
+				     struct mail_search_arg *args,
+				     uint32_t *seq1_r, uint32_t *seq2_r)
 {
 	for (; args != NULL; args = args->next) {
 		if (args->type == SEARCH_SUB) {
 			i_assert(!args->not);
-			if (search_parse_msgset_args(ibox, hdr,
-						     args->value.subargs,
-						     seq1_r, seq2_r) < 0)
-				return -1;
+			search_parse_msgset_args(hdr, args->value.subargs,
+						 seq1_r, seq2_r);
 		} else if (args->type == SEARCH_OR) {
 			/* go through our children and use the widest seqset
 			   range */
 			i_assert(!args->not);
-			if (search_or_parse_msgset_args(ibox, hdr,
-							args->value.subargs,
-							seq1_r, seq2_r) < 0)
-				return -1;
+			search_or_parse_msgset_args(hdr, args->value.subargs,
+						    seq1_r, seq2_r);
 		} else if (args->type == SEARCH_SEQSET) {
-			if (search_msgset_fix(ibox, hdr, &args->value.seqset,
-					      seq1_r, seq2_r, args->not) < 0)
-				return -1;
+			search_msgset_fix(hdr, &args->value.seqset,
+					  seq1_r, seq2_r, args->not);
 		}
 	}
-	return 0;
 }
 
-static int search_limit_lowwater(struct index_search_context *ctx,
-				 uint32_t uid_lowwater, uint32_t *first_seq)
+static void search_limit_lowwater(struct index_search_context *ctx,
+				  uint32_t uid_lowwater, uint32_t *first_seq)
 {
 	uint32_t seq1, seq2;
 
 	if (uid_lowwater == 0)
-		return 0;
-
-	if (mail_index_lookup_uid_range(ctx->view, uid_lowwater,
-					(uint32_t)-1, &seq1, &seq2) < 0) {
-		mail_storage_set_index_error(ctx->ibox);
-		return -1;
-	}
+		return;
 
+	mail_index_lookup_seq_range(ctx->view, uid_lowwater, (uint32_t)-1,
+				    &seq1, &seq2);
 	if (*first_seq < seq1)
 		*first_seq = seq1;
-	return 0;
 }
 
-static int search_limit_by_flags(struct index_search_context *ctx,
-                                 const struct mail_index_header *hdr,
-				 struct mail_search_arg *args,
-				 uint32_t *seq1, uint32_t *seq2)
+static bool search_limit_by_flags(struct index_search_context *ctx,
+				  const struct mail_index_header *hdr,
+				  struct mail_search_arg *args,
+				  uint32_t *seq1, uint32_t *seq2)
 {
 	for (; args != NULL; args = args->next) {
-		switch (args->type) {
-		case SEARCH_SEEN:
+		if (args->type != SEARCH_FLAGS) {
+			if (args->type == SEARCH_ALL) {
+				if (args->not)
+					return FALSE;
+			}
+			continue;
+		}
+		if ((args->value.flags & MAIL_SEEN) != 0) {
 			/* SEEN with 0 seen? */
 			if (!args->not && hdr->seen_messages_count == 0)
-				return 0;
+				return FALSE;
 
 			if (hdr->seen_messages_count == hdr->messages_count) {
 				/* UNSEEN with all seen? */
 				if (args->not)
-					return 0;
+					return FALSE;
 
 				/* SEEN with all seen */
 				args->match_always = TRUE;
 			} else if (args->not) {
 				/* UNSEEN with lowwater limiting */
-				if (search_limit_lowwater(ctx,
-                                		hdr->first_unseen_uid_lowwater,
-						seq1) < 0)
-					return -1;
+				search_limit_lowwater(ctx,
+                                	hdr->first_unseen_uid_lowwater, seq1);
 			}
-			break;
-		case SEARCH_DELETED:
+		}
+		if ((args->value.flags & MAIL_DELETED) != 0) {
 			/* DELETED with 0 deleted? */
 			if (!args->not && hdr->deleted_messages_count == 0)
-				return 0;
+				return FALSE;
 
 			if (hdr->deleted_messages_count ==
 			    hdr->messages_count) {
 				/* UNDELETED with all deleted? */
 				if (args->not)
-					return 0;
+					return FALSE;
 
 				/* DELETED with all deleted */
 				args->match_always = TRUE;
 			} else if (!args->not) {
 				/* DELETED with lowwater limiting */
-				if (search_limit_lowwater(ctx,
-                                		hdr->first_deleted_uid_lowwater,
-						seq1) < 0)
-					return -1;
+				search_limit_lowwater(ctx,
+                                	hdr->first_deleted_uid_lowwater, seq1);
 			}
-			break;
-		case SEARCH_ALL:
-			if (args->not)
-				return 0;
-			break;
-		default:
-			break;
 		}
 	}
 
 	return *seq1 <= *seq2;
 }
 
-static void search_args_fix_subs(struct mail_search_arg *args, bool parent_and)
-{
-	struct mail_search_arg *sub;
-
-	for (; args != NULL;) {
-		if (args->not && (args->type == SEARCH_SUB ||
-				  args->type == SEARCH_OR)) {
-			/* neg(p and q and ..) == neg(p) or neg(q) or ..
-			   neg(p or q or ..) == neg(p) and neg(q) and .. */
-			args->type = args->type == SEARCH_SUB ?
-				SEARCH_OR : SEARCH_SUB;
-			args->not = FALSE;
-			sub = args->value.subargs;
-			for (; sub != NULL; sub = sub->next)
-				sub->not = !sub->not;
-		}
-
-		if ((args->type == SEARCH_SUB && parent_and) ||
-		    (args->type == SEARCH_OR && !parent_and)) {
-			/* p and (q and ..) == p and q and ..
-			   p or (q or ..) == p or q or .. */
-			sub = args->value.subargs;
-			for (; sub->next != NULL; sub = sub->next) ;
-			sub->next = args->next;
-			*args = *args->value.subargs;
-			continue;
-		}
-
-		if (args->type == SEARCH_SUB || args->type == SEARCH_OR) {
-			search_args_fix_subs(args->value.subargs,
-					     args->type == SEARCH_SUB);
-		}
-		args = args->next;
-	}
-}
-
-static int search_get_seqset(struct index_search_context *ctx,
-			     struct mail_search_arg *args)
+static void search_get_seqset(struct index_search_context *ctx,
+			      struct mail_search_arg *args)
 {
         const struct mail_index_header *hdr;
-	int ret;
 
 	hdr = mail_index_get_header(ctx->view);
 	if (hdr->messages_count == 0) {
@@ -906,44 +860,28 @@ static int search_get_seqset(struct index_search_context *ctx,
 		   UID FETCH. */
 		ctx->seq1 = 1;
 		ctx->seq2 = 0;
-		return 0;
+		return;
 	}
 
 	ctx->seq1 = 1;
 	ctx->seq2 = hdr->messages_count;
 
-	search_args_fix_subs(args, TRUE);
-	if (search_parse_msgset_args(ctx->ibox, hdr, args,
-				     &ctx->seq1, &ctx->seq2) < 0)
-		return -1;
-
+	search_parse_msgset_args(hdr, args, &ctx->seq1, &ctx->seq2);
 	if (ctx->seq1 == 0) {
 		ctx->seq1 = 1;
 		ctx->seq2 = hdr->messages_count;
 	}
 	if (ctx->seq1 > ctx->seq2) {
 		/* no matches */
-		return 0;
+		return;
 	}
 
 	/* UNSEEN and DELETED in root search level may limit the range */
-	ret = search_limit_by_flags(ctx, hdr, args, &ctx->seq1, &ctx->seq2);
-	if (ret < 0)
-		return -1;
-	if (ret == 0) {
+	if (!search_limit_by_flags(ctx, hdr, args, &ctx->seq1, &ctx->seq2)) {
 		/* no matches */
 		ctx->seq1 = 1;
 		ctx->seq2 = 0;
 	}
-	return 0;
-}
-
-int index_storage_search_get_sorting(struct mailbox *box __attr_unused__,
-				     enum mail_sort_type *sort_program)
-{
-	/* currently we don't support sorting */
-	*sort_program = MAIL_SORT_END;
-	return 0;
 }
 
 struct mail_search_context *
@@ -955,33 +893,38 @@ index_storage_search_init(struct mailbox_transaction_context *_t,
 		(struct index_transaction_context *)_t;
 	struct index_search_context *ctx;
 
-	if (sort_program != NULL && *sort_program != MAIL_SORT_END) {
-		i_fatal("BUG: index_storage_search_init(): "
-			 "invalid sort_program");
-	}
-
 	ctx = i_new(struct index_search_context, 1);
 	ctx->mail_ctx.transaction = _t;
 	ctx->ibox = t->ibox;
 	ctx->view = t->trans_view;
-	ctx->charset = i_strdup(charset);
-	ctx->args = args;
+	ctx->mail_ctx.charset = i_strdup(charset);
+	ctx->mail_ctx.args = args;
+	ctx->mail_ctx.sort_program = index_sort_program_init(_t, sort_program);
 
-	mail_search_args_reset(ctx->args, TRUE);
+	array_create(&ctx->mail_ctx.module_contexts, default_pool,
+		     sizeof(void *), 5);
 
-	if (search_get_seqset(ctx, args) < 0) {
-		ctx->failed = TRUE;
-		ctx->seq1 = 1;
-		ctx->seq2 = 0;
-	} else {
-		(void)mail_search_args_foreach(args, search_init_seqset_arg,
-					       ctx);
-		/* Need to reset results for match_always cases */
-		mail_search_args_reset(ctx->args, FALSE);
-	}
+	mail_search_args_reset(ctx->mail_ctx.args, TRUE);
+
+	search_get_seqset(ctx, args);
+	(void)mail_search_args_foreach(args, search_init_arg, ctx);
+
+	/* Need to reset results for match_always cases */
+	mail_search_args_reset(ctx->mail_ctx.args, FALSE);
 	return &ctx->mail_ctx;
 }
 
+static void search_arg_deinit(struct mail_search_arg *arg,
+			      void *context ATTR_UNUSED)
+{
+	struct message_search_context *search_ctx = arg->context;
+
+	if (search_ctx != NULL) {
+		message_search_deinit(&search_ctx);
+		arg->context = NULL;
+	}
+}
+
 int index_storage_search_deinit(struct mail_search_context *_ctx)
 {
         struct index_search_context *ctx = (struct index_search_context *)_ctx;
@@ -991,13 +934,20 @@ int index_storage_search_deinit(struct mail_search_context *_ctx)
 
 	if (ctx->error != NULL) {
 		mail_storage_set_error(ctx->ibox->box.storage,
-				       "%s", ctx->error);
+				       MAIL_ERROR_PARAMS, ctx->error);
 	}
 
-	if (ctx->hdr_pool != NULL)
-		pool_unref(ctx->hdr_pool);
+	mail_search_args_reset(ctx->mail_ctx.args, FALSE);
+	(void)mail_search_args_foreach(ctx->mail_ctx.args,
+				       search_arg_deinit, NULL);
 
-	i_free(ctx->charset);
+	if (ctx->search_pool != NULL)
+		pool_unref(&ctx->search_pool);
+
+	if (ctx->mail_ctx.sort_program != NULL)
+		index_sort_program_deinit(&ctx->mail_ctx.sort_program);
+	array_free(&ctx->mail_ctx.module_contexts);
+	i_free(ctx->mail_ctx.charset);
 	i_free(ctx);
 	return ret;
 }
@@ -1007,27 +957,17 @@ static bool search_match_next(struct index_search_context *ctx)
         struct mail_search_arg *arg;
 	int ret;
 
-	/* check the index matches first */
-	ret = mail_search_args_foreach(ctx->args, search_index_arg, ctx);
-	if (ret >= 0)
-		return ret > 0;
-
-	if (ctx->imail->data.rec == NULL) {
-		/* expunged message, no way to check if the rest would have
-		   matched */
-		return FALSE;
-	}
-
 	/* next search only from cached arguments */
-	ret = mail_search_args_foreach(ctx->args, search_cached_arg, ctx);
+	ret = mail_search_args_foreach(ctx->mail_ctx.args,
+				       search_cached_arg, ctx);
 	if (ret >= 0)
 		return ret > 0;
 
 	/* open the mail file and check the rest */
-	if (!search_arg_match_text(ctx->args, ctx))
+	if (!search_arg_match_text(ctx->mail_ctx.args, ctx))
 		return FALSE;
 
-	for (arg = ctx->args; arg != NULL; arg = arg->next) {
+	for (arg = ctx->mail_ctx.args; arg != NULL; arg = arg->next) {
 		if (arg->result != 1)
 			return FALSE;
 	}
@@ -1035,38 +975,103 @@ static bool search_match_next(struct index_search_context *ctx)
 	return TRUE;
 }
 
-int index_storage_search_next(struct mail_search_context *_ctx,
-			      struct mail *mail)
+static void index_storage_search_notify(struct mailbox *box,
+					struct index_search_context *ctx)
+{
+	const struct mail_index_header *hdr;
+	float percentage;
+	unsigned int msecs, secs;
+
+	if (ctx->last_notify.tv_sec == 0) {
+		/* set the search time in here, in case a plugin
+		   already spent some time indexing the mailbox */
+		ctx->search_start_time = ioloop_timeval;
+	} else if (box->storage->callbacks->notify_ok != NULL) {
+		hdr = mail_index_get_header(ctx->ibox->view);
+
+		percentage = ctx->mail->seq * 100.0 / hdr->messages_count;
+		msecs = (ioloop_timeval.tv_sec -
+			 ctx->search_start_time.tv_sec) * 1000 +
+			(ioloop_timeval.tv_usec -
+			 ctx->search_start_time.tv_usec) / 1000;
+		secs = (msecs / (percentage / 100.0) - msecs) / 1000;
+
+		T_BEGIN {
+			const char *text;
+
+			text = t_strdup_printf("Searched %d%% of the mailbox, "
+					       "ETA %d:%02d", (int)percentage,
+					       secs/60, secs%60);
+			box->storage->callbacks->
+				notify_ok(box, text,
+					  box->storage->callback_context);
+		} T_END;
+	}
+	ctx->last_notify = ioloop_timeval;
+}
+
+int index_storage_search_next_nonblock(struct mail_search_context *_ctx,
+				       struct mail *mail, bool *tryagain_r)
 {
         struct index_search_context *ctx = (struct index_search_context *)_ctx;
 	struct mailbox *box = _ctx->transaction->box;
+	unsigned int count = 0;
 	int ret;
 
+	*tryagain_r = FALSE;
+
+	if (ctx->sorted) {
+		/* everything searched at this point already. just returning
+		   matches from sort list */
+		if (!index_sort_list_next(ctx->mail_ctx.sort_program, mail))
+			return 0;
+		return 1;
+	}
+
 	ctx->mail = mail;
-	ctx->imail = (struct index_mail *)mail;
+	ctx->imail = ((struct mail_private *)mail)->v.get_index_mail(mail);
+
+	if (ioloop_time - ctx->last_notify.tv_sec >=
+	    SEARCH_NOTIFY_INTERVAL_SECS)
+		index_storage_search_notify(box, ctx);
 
 	while ((ret = box->v.search_next_update_seq(_ctx)) > 0) {
-		if (mail_set_seq(mail, _ctx->seq) < 0) {
-			ret = -1;
-			break;
-		}
+		mail_set_seq(mail, _ctx->seq);
 
-		t_push();
-		ret = search_match_next(ctx) ? 1 : 0;
-		t_pop();
+		T_BEGIN {
+			ret = search_match_next(ctx) ? 1 : 0;
+		} T_END;
 
-		mail_search_args_reset(ctx->args, FALSE);
+		mail_search_args_reset(ctx->mail_ctx.args, FALSE);
 
 		if (ctx->error != NULL)
 			ret = -1;
-		if (ret != 0)
-			break;
+		if (ret != 0) {
+			if (ctx->mail_ctx.sort_program == NULL)
+				break;
+
+			index_sort_list_add(ctx->mail_ctx.sort_program, mail);
+		}
+
+		if (++count == SEARCH_NONBLOCK_COUNT) {
+			*tryagain_r = TRUE;
+			return 0;
+		}
 	}
 	if (ret < 0)
 		ctx->failed = TRUE;
 	ctx->mail = NULL;
 	ctx->imail = NULL;
 
+	if (ctx->mail_ctx.sort_program != NULL && ret == 0) {
+		/* finished searching the messages. now sort them and start
+		   returning the messages. */
+		ctx->sorted = TRUE;
+		index_sort_list_finish(ctx->mail_ctx.sort_program);
+		return index_storage_search_next_nonblock(_ctx, mail,
+							  tryagain_r);
+	}
+
 	return ret;
 }
 
@@ -1082,20 +1087,28 @@ int index_storage_search_next_update_seq(struct mail_search_context *_ctx)
 		_ctx->seq++;
 	}
 
-	if (!ctx->have_seqsets)
+	if (!ctx->have_seqsets && !ctx->have_flags_or_keywords)
 		return _ctx->seq <= ctx->seq2 ? 1 : 0;
 
 	ret = 0;
 	while (_ctx->seq <= ctx->seq2) {
 		/* check if the sequence matches */
-		ret = mail_search_args_foreach(ctx->args,
+		ret = mail_search_args_foreach(ctx->mail_ctx.args,
 					       search_seqset_arg, ctx);
-		if (ret != 0)
-			break;
+		if (ret != 0) {
+			/* check if flags/keywords match before anything else
+			   is done. mail_set_seq() can be a bit slow. */
+			if (!ctx->have_flags_or_keywords)
+				break;
+			ret = mail_search_args_foreach(ctx->mail_ctx.args,
+						       search_index_arg, ctx);
+			if (ret != 0)
+				break;
+		}
 
 		/* doesn't, try next one */
 		_ctx->seq++;
-		mail_search_args_reset(ctx->args, FALSE);
+		mail_search_args_reset(ctx->mail_ctx.args, FALSE);
 	}
 	return ret == 0 ? 0 : 1;
 }
