@@ -1,12 +1,15 @@
-/* Copyright (C) 2002-2004 Timo Sirainen */
+/* Copyright (c) 2002-2008 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
 #include "ioloop.h"
 #include "str.h"
+#include "nfs-workarounds.h"
 #include "maildir-storage.h"
 #include "maildir-uidlist.h"
+#include "maildir-filename.h"
 #include "maildir-keywords.h"
+#include "maildir-sync.h"
 #include "index-mail.h"
 #include "mail-copy.h"
 
@@ -20,6 +23,7 @@ struct hardlink_ctx {
 	unsigned int base_end_pos;
 
 	unsigned int size_set:1;
+	unsigned int vsize_set:1;
 	unsigned int success:1;
 	unsigned int preserve_filename:1;
 };
@@ -39,14 +43,14 @@ static int do_save_mail_size(struct maildir_mailbox *mbox, const char *path,
 		if (stat(path, &st) < 0) {
 			if (errno == ENOENT)
 				return 0;
-			mail_storage_set_critical(STORAGE(mbox->storage),
+			mail_storage_set_critical(&mbox->storage->storage,
 						  "stat(%s) failed: %m", path);
 			return -1;
 		}
 		size = st.st_size;
 	}
 
-	str = t_strdup_printf(",S=%"PRIuUOFF_T, size);
+	str = t_strdup_printf(",%c=%"PRIuUOFF_T, MAILDIR_EXTRA_FILE_SIZE, size);
 	str_insert(ctx->dest_path, ctx->base_end_pos, str);
 
 	ctx->dest_fname = strrchr(str_c(ctx->dest_path), '/') + 1;
@@ -54,25 +58,54 @@ static int do_save_mail_size(struct maildir_mailbox *mbox, const char *path,
 	return 1;
 }
 
+static void do_save_mail_vsize(const char *path, struct hardlink_ctx *ctx)
+{
+	const char *fname, *str;
+	uoff_t size;
+
+	fname = strrchr(path, '/');
+	fname = fname != NULL ? fname + 1 : path;
+
+	if (!maildir_filename_get_size(fname, MAILDIR_EXTRA_VIRTUAL_SIZE,
+				       &size))
+		return;
+
+	str = t_strdup_printf(",%c=%"PRIuUOFF_T,
+			      MAILDIR_EXTRA_VIRTUAL_SIZE, size);
+	str_insert(ctx->dest_path, ctx->base_end_pos, str);
+
+	ctx->dest_fname = strrchr(str_c(ctx->dest_path), '/') + 1;
+	ctx->vsize_set = TRUE;
+}
+
 static int do_hardlink(struct maildir_mailbox *mbox, const char *path,
-		       void *context)
+		       struct hardlink_ctx *ctx)
 {
-	struct hardlink_ctx *ctx = context;
 	int ret;
 
-	if (!ctx->preserve_filename && mbox->storage->save_size_in_filename &&
-	    !ctx->size_set) {
-		if ((ret = do_save_mail_size(mbox, path, ctx)) <= 0)
-			return ret;
+	if (!ctx->preserve_filename) {
+		if (mbox->storage->save_size_in_filename &&
+		    !ctx->size_set) {
+			if ((ret = do_save_mail_size(mbox, path, ctx)) <= 0)
+				return ret;
+		}
+		/* set virtual size if it's in the original file name */
+		if (!ctx->vsize_set)
+			do_save_mail_vsize(path, ctx);
 	}
 
-	if (link(path, str_c(ctx->dest_path)) < 0) {
+	if ((mbox->storage->storage.flags &
+	     MAIL_STORAGE_FLAG_NFS_FLUSH_STORAGE) != 0)
+		ret = nfs_safe_link(path, str_c(ctx->dest_path), FALSE);
+	else
+		ret = link(path, str_c(ctx->dest_path));
+	if (ret < 0) {
 		if (errno == ENOENT)
 			return 0;
 
 		if (ENOSPACE(errno)) {
-			mail_storage_set_error(STORAGE(mbox->storage),
-					       "Not enough disk space");
+			mail_storage_set_error(&mbox->storage->storage,
+				MAIL_ERROR_NOSPACE, MAIL_ERRSTR_NO_SPACE);
 			return -1;
 		}
 
@@ -82,7 +115,7 @@ static int do_hardlink(struct maildir_mailbox *mbox, const char *path,
 		if (errno == EACCES || ECANTLINK(errno) || errno == EEXIST)
 			return 1;
 
-		mail_storage_set_critical(STORAGE(mbox->storage),
+		mail_storage_set_critical(&mbox->storage->storage,
 					  "link(%s, %s) failed: %m",
 					  path, str_c(ctx->dest_path));
 		return -1;
@@ -99,15 +132,24 @@ maildir_copy_hardlink(struct maildir_transaction_context *t, struct mail *mail,
 {
 	struct maildir_mailbox *dest_mbox =
 		(struct maildir_mailbox *)t->ictx.ibox;
-	struct maildir_mailbox *src_mbox =
-		(struct maildir_mailbox *)mail->box;
+	struct maildir_mailbox *src_mbox;
 	struct maildir_save_context *ctx;
 	struct hardlink_ctx do_ctx;
-	const char *filename = NULL;
+	const char *path, *filename = NULL;
 	uint32_t seq;
 
 	i_assert((t->ictx.flags & MAILBOX_TRANSACTION_FLAG_EXTERNAL) != 0);
 
+	if (strcmp(mail->box->storage->name, MAILDIR_STORAGE_NAME) == 0)
+		src_mbox = (struct maildir_mailbox *)mail->box;
+	else if (strcmp(mail->box->storage->name, "raw") == 0) {
+		/* deliver uses raw format */
+		src_mbox = NULL;
+	} else {
+		/* Can't hard link files from the source storage */
+		return 0;
+	}
+
 	if (t->save_ctx == NULL)
 		t->save_ctx = maildir_save_transaction_init(t);
 	ctx = t->save_ctx;
@@ -120,7 +162,7 @@ maildir_copy_hardlink(struct maildir_transaction_context *t, struct mail *mail,
 	memset(&do_ctx, 0, sizeof(do_ctx));
 	do_ctx.dest_path = str_new(default_pool, 512);
 
-	if (dest_mbox->storage->copy_preserve_filename) {
+	if (dest_mbox->storage->copy_preserve_filename && src_mbox != NULL) {
 		enum maildir_uidlist_rec_flag src_flags;
 		const char *src_fname;
 
@@ -131,7 +173,7 @@ maildir_copy_hardlink(struct maildir_transaction_context *t, struct mail *mail,
 		src_fname = maildir_uidlist_lookup(src_mbox->uidlist,
 						   mail->uid, &src_flags);
 		if (src_fname != NULL &&
-		    maildir_uidlist_update(dest_mbox->uidlist) >= 0 &&
+		    maildir_uidlist_refresh(dest_mbox->uidlist) >= 0 &&
 		    maildir_uidlist_get_full_filename(dest_mbox->uidlist,
 						      src_fname) == NULL)
 			filename = t_strcut(src_fname, ':');
@@ -139,8 +181,7 @@ maildir_copy_hardlink(struct maildir_transaction_context *t, struct mail *mail,
 	if (filename == NULL) {
 		/* the generated filename is _always_ unique, so we don't
 		   bother trying to check if it already exists */
-		do_ctx.dest_fname =
-			maildir_generate_tmp_filename(&ioloop_timeval);
+		do_ctx.dest_fname = maildir_filename_generate();
 	} else {
 		do_ctx.dest_fname = filename;
 		do_ctx.preserve_filename = TRUE;
@@ -169,15 +210,26 @@ maildir_copy_hardlink(struct maildir_transaction_context *t, struct mail *mail,
 		}
 	} else
 #endif
-{
+	{
 		/* keywords, hardlink to tmp/ with basename and later when we
 		   have uidlist locked, move it to new/cur. */
 		str_printfa(do_ctx.dest_path, "%s/tmp/%s",
 			    dest_mbox->path, do_ctx.dest_fname);
 		do_ctx.base_end_pos = str_len(do_ctx.dest_path);
 	}
-	if (maildir_file_do(src_mbox, mail->uid, do_hardlink, &do_ctx) < 0)
-		return -1;
+	if (src_mbox != NULL) {
+		/* maildir */
+		if (maildir_file_do(src_mbox, mail->uid,
+				    do_hardlink, &do_ctx) < 0)
+			return -1;
+	} else {
+		/* raw / deliver */
+		if (mail_get_special(mail, MAIL_FETCH_UIDL_FILE_NAME,
+				     &path) < 0 || *path == '\0')
+			return 0;
+		if (do_hardlink(dest_mbox, path, &do_ctx) < 0)
+			return -1;
+	}
 
 	if (!do_ctx.success) {
 		/* couldn't copy with hardlinking, fallback to copying */
@@ -189,24 +241,24 @@ maildir_copy_hardlink(struct maildir_transaction_context *t, struct mail *mail,
 		/* hardlinked to destination, set hardlinked-flag */
 		seq = maildir_save_add(t, do_ctx.dest_fname,
 				       flags | MAILDIR_SAVE_FLAG_HARDLINK, NULL,
-				       dest_mail != NULL);
+				       dest_mail);
 	} else
 #endif
 {
 		/* hardlinked to tmp/, treat as normal copied mail */
 		seq = maildir_save_add(t, do_ctx.dest_fname, flags, keywords,
-				       dest_mail != NULL);
-	}
-
-	if (dest_mail != NULL) {
-		i_assert(seq != 0);
-
-		if (mail_set_seq(dest_mail, seq) < 0)
-			return -1;
+				       dest_mail);
 	}
 	return 1;
 }
 
+static bool
+maildir_compatible_file_modes(struct mailbox *box1, struct mailbox *box2)
+{
+	return box1->file_create_mode == box2->file_create_mode &&
+		box1->file_create_gid == box2->file_create_gid;
+}
+
 int maildir_copy(struct mailbox_transaction_context *_t, struct mail *mail,
 		 enum mail_flags flags, struct mail_keywords *keywords,
 		 struct mail *dest_mail)
@@ -217,11 +269,11 @@ int maildir_copy(struct mailbox_transaction_context *_t, struct mail *mail,
 	int ret;
 
 	if (mbox->storage->copy_with_hardlinks &&
-	    mail->box->storage == mbox->ibox.box.storage) {
-		t_push();
-		ret = maildir_copy_hardlink(t, mail, flags,
-					    keywords, dest_mail);
-		t_pop();
+	    maildir_compatible_file_modes(&mbox->ibox.box, mail->box)) {
+		T_BEGIN {
+			ret = maildir_copy_hardlink(t, mail, flags,
+						    keywords, dest_mail);
+		} T_END;
 
 		if (ret > 0)
 			return 0;
