@@ -219,7 +219,8 @@ static int maildir_get_pop3_state(struct index_mail *mail)
 	if (!not_pop3_only) {
 		/* either nothing is cached, or only vsize is cached. */
 		mail->pop3_state = 1;
-	} else if (vsize_dec != MAIL_CACHE_DECISION_YES) {
+	} else if (vsize_dec != MAIL_CACHE_DECISION_YES &&
+		   (mail->ibox->open_flags & MAILBOX_OPEN_POP3_SESSION) == 0) {
 		/* if virtual size isn't cached permanently,
 		   POP3 isn't being used */
 		mail->pop3_state = -1;
