@@ -883,13 +883,13 @@ maildir_list_iter_is_mailbox(struct mailbox_list_iterate_context *ctx
 			     const char *dir, const char *fname,
 			     const char *mailbox_name ATTR_UNUSED,
 			     enum mailbox_list_file_type type,
-			     enum mailbox_info_flags *flags_r)
+			     enum mailbox_info_flags *flags)
 {
-	struct stat st;
-	const char *path;
+	struct stat st, st2;
+	const char *path, *cur_path;
 
 	if (maildir_is_internal_name(fname)) {
-		*flags_r = MAILBOX_NONEXISTENT;
+		*flags |= MAILBOX_NONEXISTENT;
 		return 0;
 	}
 
@@ -897,7 +897,7 @@ maildir_list_iter_is_mailbox(struct mailbox_list_iterate_context *ctx
 	case MAILBOX_LIST_FILE_TYPE_FILE:
 	case MAILBOX_LIST_FILE_TYPE_OTHER:
 		/* non-directories are not */
-		*flags_r = MAILBOX_NOSELECT;
+		*flags |= MAILBOX_NOSELECT;
 		return 0;
 
 	case MAILBOX_LIST_FILE_TYPE_DIR:
@@ -908,24 +908,60 @@ maildir_list_iter_is_mailbox(struct mailbox_list_iterate_context *ctx
 
 	path = t_strdup_printf("%s/%s", dir, fname);
 	if (stat(path, &st) == 0) {
-		if (S_ISDIR(st.st_mode))
-			return 1;
-		else if (strncmp(fname, ".nfs", 4) == 0) {
-			/* temporary NFS file */
-			*flags_r = MAILBOX_NONEXISTENT;
-			return 0;
-		} else {
-			*flags_r = MAILBOX_NOSELECT;
+		if (!S_ISDIR(st.st_mode)) {
+			if (strncmp(fname, ".nfs", 4) == 0) {
+				/* temporary NFS file */
+				*flags |= MAILBOX_NONEXISTENT;
+			} else {
+				*flags |= MAILBOX_NOSELECT |
+					MAILBOX_NOINFERIORS;
+			}
 			return 0;
 		}
 	} else if (errno == ENOENT) {
-		/* this was a directory. maybe it has children. */
-		*flags_r = MAILBOX_NOSELECT;
-		return 1;
+		/* doesn't exist - probably a non-existing subscribed mailbox */
+		*flags |= MAILBOX_NONEXISTENT;
 	} else {
-		*flags_r = MAILBOX_NOSELECT;
-		return 0;
+		/* non-selectable. probably either access denied, or symlink
+		   destination not found. don't bother logging errors. */
+		*flags |= MAILBOX_NOSELECT;
 	}
+	if ((*flags & (MAILBOX_NOSELECT | MAILBOX_NONEXISTENT)) == 0) {
+		/* make sure it's a selectable mailbox */
+		cur_path = t_strconcat(path, "/cur", NULL);
+		if (stat(cur_path, &st2) < 0 || !S_ISDIR(st2.st_mode))
+			*flags |= MAILBOX_NOSELECT;
+
+		if (*ctx->list->set.maildir_name == '\0') {
+			/* now we can figure out based on the link count if we
+			   have child mailboxes or not. for a selectable
+			   mailbox we have 3 more links (cur/, new/ and tmp/)
+			   than non-selectable. */
+			if ((*flags & MAILBOX_NOSELECT) == 0) {
+				if (st.st_nlink > 5)
+					*flags |= MAILBOX_CHILDREN;
+				else
+					*flags |= MAILBOX_NOCHILDREN;
+			} else {
+				if (st.st_nlink > 2)
+					*flags |= MAILBOX_CHILDREN;
+				else
+					*flags |= MAILBOX_NOCHILDREN;
+			}
+		} else {
+			/* link count 3 may mean either a selectable mailbox
+			   or a non-selectable mailbox with 1 child. */
+			if (st.st_nlink > 3)
+				*flags |= MAILBOX_CHILDREN;
+			else if (st.st_nlink == 3) {
+				if ((*flags & MAILBOX_NOSELECT) != 0)
+					*flags |= MAILBOX_CHILDREN;
+				else
+					*flags |= MAILBOX_NOCHILDREN;
+			}
+		}
+	}
+	return 1;
 }
 
 static int
@@ -933,7 +969,7 @@ maildirplusplus_iter_is_mailbox(struct mailbox_list_iterate_context *ctx,
 				const char *dir, const char *fname,
 				const char *mailbox_name ATTR_UNUSED,
 				enum mailbox_list_file_type type,
-				enum mailbox_info_flags *flags_r)
+				enum mailbox_info_flags *flags)
 {
 	struct maildir_storage *storage = MAILDIR_LIST_CONTEXT(ctx->list);
 	struct mail_storage *_storage = &storage->storage;
@@ -953,7 +989,7 @@ maildirplusplus_iter_is_mailbox(struct mailbox_list_iterate_context *ctx,
 		    st.st_mtime < ioloop_time - 3600)
 			(void)unlink_directory(path, TRUE);
 
-		*flags_r = MAILBOX_NONEXISTENT;
+		*flags |= MAILBOX_NONEXISTENT;
 		return 0;
 	}
 
@@ -965,7 +1001,7 @@ maildirplusplus_iter_is_mailbox(struct mailbox_list_iterate_context *ctx,
 	case MAILBOX_LIST_FILE_TYPE_FILE:
 	case MAILBOX_LIST_FILE_TYPE_OTHER:
 		/* non-directories are not */
-		*flags_r = MAILBOX_NOSELECT;
+		*flags |= MAILBOX_NOSELECT;
 		return 0;
 
 	case MAILBOX_LIST_FILE_TYPE_UNKNOWN:
@@ -989,17 +1025,17 @@ maildirplusplus_iter_is_mailbox(struct mailbox_list_iterate_context *ctx,
 				ret = 1;
 			else {
 				if (strncmp(fname, ".nfs", 4) == 0)
-					*flags_r = MAILBOX_NONEXISTENT;
+					*flags |= MAILBOX_NONEXISTENT;
 				else
-					*flags_r = MAILBOX_NOSELECT;
+					*flags |= MAILBOX_NOSELECT;
 				ret = 0;
 			}
 		} else if (errno == ENOENT) {
 			/* just deleted? */
-			*flags_r = MAILBOX_NONEXISTENT;
+			*flags |= MAILBOX_NONEXISTENT;
 			ret = 0;
 		} else {
-			*flags_r = MAILBOX_NOSELECT;
+			*flags |= MAILBOX_NOSELECT;
 			ret = 0;
 		}
 	} else {
