@@ -90,6 +90,7 @@ struct maildir_uidlist {
 	unsigned int initial_read:1;
 	unsigned int initial_hdr_read:1;
 	unsigned int initial_sync:1;
+	unsigned int retry_rewind:1;
 };
 
 struct maildir_uidlist_sync_ctx {
@@ -305,9 +306,17 @@ maildir_uidlist_set_corrupted(struct maildir_uidlist *uidlist,
 	va_list args;
 
 	va_start(args, fmt);
-	mail_storage_set_critical(storage, "Broken file %s line %u: %s",
-				  uidlist->path, uidlist->read_line_count,
-				  t_strdup_vprintf(fmt, args));
+	if (uidlist->retry_rewind) {
+		mail_storage_set_critical(storage,
+			"Broken or unexpectedly changed file %s "
+			"line %u: %s - re-reading from beginning",
+			uidlist->path, uidlist->read_line_count,
+			t_strdup_vprintf(fmt, args));
+	} else {
+		mail_storage_set_critical(storage, "Broken file %s line %u: %s",
+			uidlist->path, uidlist->read_line_count,
+			t_strdup_vprintf(fmt, args));
+	}
 	va_end(args);
 }
 
@@ -337,18 +346,20 @@ static void maildir_uidlist_update_hdr(struct maildir_uidlist *uidlist,
 	mhdr->uidlist_size = st->st_size;
 }
 
-static void
+static unsigned int
 maildir_uidlist_records_array_delete(struct maildir_uidlist *uidlist,
 				     struct maildir_uidlist_rec *rec)
 {
 	struct maildir_uidlist_rec *const *recs, *const *pos;
-	unsigned int count;
+	unsigned int idx, count;
 
 	recs = array_get(&uidlist->records, &count);
 	pos = bsearch(&rec, recs, count, sizeof(*recs), maildir_uid_cmp);
 	i_assert(pos != NULL);
 
-	array_delete(&uidlist->records, pos - recs, 1);
+	idx = pos - recs;
+	array_delete(&uidlist->records, idx, 1);
+	return idx;
 }
 
 static bool
@@ -405,7 +416,7 @@ static bool maildir_uidlist_next(struct maildir_uidlist *uidlist,
 	}
 	if (uid <= uidlist->prev_read_uid) {
 		maildir_uidlist_set_corrupted(uidlist, 
-					      "UIDs not ordered (%u > %u)",
+					      "UIDs not ordered (%u >= %u)",
 					      uid, uidlist->prev_read_uid);
 		return FALSE;
 	}
@@ -627,16 +638,23 @@ maildir_uidlist_update_read(struct maildir_uidlist *uidlist,
 		uidlist->prev_read_uid = 0;
 		uidlist->change_counter++;
 		uidlist->read_records_count = 0;
+		uidlist->retry_rewind = last_read_offset != 0 && try_retry;
 
 		ret = 1;
 		while ((line = i_stream_read_next_line(input)) != NULL) {
 			uidlist->read_records_count++;
 			uidlist->read_line_count++;
 			if (!maildir_uidlist_next(uidlist, line)) {
-				ret = 0;
+				if (!uidlist->retry_rewind)
+					ret = 0;
+				else {
+					ret = -1;
+					*retry_r = TRUE;
+				}
 				break;
 			}
                 }
+		uidlist->retry_rewind = FALSE;
 		if (input->stream_errno != 0)
                         ret = -1;
 
@@ -663,7 +681,7 @@ maildir_uidlist_update_read(struct maildir_uidlist *uidlist,
 		uidlist->fd_size = st.st_size;
 		uidlist->last_read_offset = input->v_offset;
 		maildir_uidlist_update_hdr(uidlist, &st);
-        } else {
+        } else if (!*retry_r) {
                 /* I/O error */
                 if (input->stream_errno == ESTALE && try_retry)
 			*retry_r = TRUE;
@@ -1189,6 +1207,7 @@ static bool maildir_uidlist_want_recreate(struct maildir_uidlist_sync_ctx *ctx)
 static int maildir_uidlist_sync_update(struct maildir_uidlist_sync_ctx *ctx)
 {
 	struct maildir_uidlist *uidlist = ctx->uidlist;
+	struct mail_storage *storage = uidlist->ibox->box.storage;
 	struct stat st;
 	uoff_t file_size;
 
@@ -1198,7 +1217,8 @@ static int maildir_uidlist_sync_update(struct maildir_uidlist_sync_ctx *ctx)
 
 		hdr = mail_index_get_header(uidlist->ibox->view);
 		uidlist->uid_validity = hdr->uid_validity != 0 ?
-			hdr->uid_validity : (uint32_t)ioloop_time;
+			hdr->uid_validity :
+			maildir_get_uidvalidity_next(storage);
 	}
 
 
@@ -1211,7 +1231,7 @@ static int maildir_uidlist_sync_update(struct maildir_uidlist_sync_ctx *ctx)
 
 		uidlist->fd = nfs_safe_open(uidlist->path, O_RDWR);
 		if (uidlist->fd == -1) {
-			mail_storage_set_critical(uidlist->ibox->box.storage,
+			mail_storage_set_critical(storage,
 				"open(%s) failed: %m", uidlist->path);
 			return -1;
 		}
@@ -1220,7 +1240,7 @@ static int maildir_uidlist_sync_update(struct maildir_uidlist_sync_ctx *ctx)
 	i_assert(ctx->first_unwritten_pos != (unsigned int)-1);
 
 	if (lseek(uidlist->fd, 0, SEEK_END) < 0) {
-		mail_storage_set_critical(uidlist->ibox->box.storage,
+		mail_storage_set_critical(storage,
 			"lseek(%s) failed: %m", uidlist->path);
 		return -1;
 	}
@@ -1230,7 +1250,7 @@ static int maildir_uidlist_sync_update(struct maildir_uidlist_sync_ctx *ctx)
 		return -1;
 
 	if (fstat(uidlist->fd, &st) < 0) {
-		mail_storage_set_critical(uidlist->ibox->box.storage,
+		mail_storage_set_critical(storage,
 			"fstat(%s) failed: %m", uidlist->path);
 		return -1;
 	}
@@ -1444,24 +1464,25 @@ void maildir_uidlist_sync_remove(struct maildir_uidlist_sync_ctx *ctx,
 				 const char *filename)
 {
 	struct maildir_uidlist_rec *rec;
+	unsigned int idx;
 
 	i_assert(ctx->partial);
 
-	if (ctx->first_unwritten_pos != (unsigned int)-1) {
-		i_assert(ctx->first_unwritten_pos > 0);
-		ctx->first_unwritten_pos--;
-	}
-	if (ctx->first_nouid_pos != (unsigned int)-1) {
-		i_assert(ctx->first_nouid_pos > 0);
-		ctx->first_nouid_pos--;
-	}
-
 	rec = hash_lookup(ctx->uidlist->files, filename);
 	i_assert(rec != NULL);
 	i_assert(rec->uid != (uint32_t)-1);
 
 	hash_remove(ctx->uidlist->files, filename);
-	maildir_uidlist_records_array_delete(ctx->uidlist, rec);
+	idx = maildir_uidlist_records_array_delete(ctx->uidlist, rec);
+
+	if (ctx->first_unwritten_pos != (unsigned int)-1) {
+		i_assert(ctx->first_unwritten_pos > idx);
+		ctx->first_unwritten_pos--;
+	}
+	if (ctx->first_nouid_pos != (unsigned int)-1) {
+		i_assert(ctx->first_nouid_pos > idx);
+		ctx->first_nouid_pos--;
+	}
 
 	ctx->changed = TRUE;
 	ctx->uidlist->recreate = TRUE;
