@@ -1,24 +1,23 @@
-#ifndef __MBOX_STORAGE_H
-#define __MBOX_STORAGE_H
+#ifndef MBOX_STORAGE_H
+#define MBOX_STORAGE_H
 
 /* Padding to leave in X-Keywords header when rewriting mbox */
 #define MBOX_HEADER_PADDING 50
 /* Don't write Content-Length header unless it's value is larger than this. */
 #define MBOX_MIN_CONTENT_LENGTH_SIZE 1024
 
-#define SUBSCRIPTION_FILE_NAME ".subscriptions"
+#define MBOX_STORAGE_NAME "mbox"
+#define MBOX_SUBSCRIPTION_FILE_NAME ".subscriptions"
 #define MBOX_INDEX_PREFIX "dovecot.index"
 #define MBOX_INDEX_DIR_NAME ".imap"
 
 #include "index-storage.h"
-
-#define STORAGE(mbox_storage) \
-	(&(mbox_storage)->storage.storage)
-#define INDEX_STORAGE(mbox_storage) \
-	(&(mbox_storage)->storage)
+#include "mailbox-list-private.h"
 
 struct mbox_storage {
-	struct index_storage storage;
+	struct mail_storage storage;
+
+	union mailbox_list_module_context list_module_ctx;
 };
 
 struct mbox_mailbox {
@@ -50,14 +49,16 @@ struct mbox_mailbox {
 	unsigned int mbox_dotlocked:1;
 	unsigned int mbox_used_privileges:1;
 	unsigned int mbox_privileged_locking:1;
+	unsigned int syncing:1;
 };
 
 struct mbox_transaction_context {
 	struct index_transaction_context ictx;
+	union mail_index_transaction_module_context module_ctx;
 
 	struct mbox_save_context *save_ctx;
 	unsigned int mbox_lock_id;
-	unsigned int mbox_modified:1;
+	unsigned int mails_saved:1;
 };
 
 extern struct mail_vfuncs mbox_mail_vfuncs;
@@ -66,19 +67,8 @@ extern unsigned int mbox_hide_headers_count, mbox_save_drop_headers_count;
 
 int mbox_set_syscall_error(struct mbox_mailbox *mbox, const char *function);
 
-struct mailbox_list_context *
-mbox_mailbox_list_init(struct mail_storage *storage,
-		       const char *ref, const char *mask,
-		       enum mailbox_list_flags flags);
-int mbox_mailbox_list_deinit(struct mailbox_list_context *ctx);
-struct mailbox_list *mbox_mailbox_list_next(struct mailbox_list_context *ctx);
-
-struct mailbox_transaction_context *
-mbox_transaction_begin(struct mailbox *box,
-		       enum mailbox_transaction_flags flags);
-int mbox_transaction_commit(struct mailbox_transaction_context *t,
-			    enum mailbox_sync_flags flags);
-void mbox_transaction_rollback(struct mailbox_transaction_context *t);
+void mbox_transaction_class_init(void);
+void mbox_transaction_class_deinit(void);
 
 struct mailbox_sync_context *
 mbox_storage_sync_init(struct mailbox *box, enum mailbox_sync_flags flags);
@@ -87,14 +77,12 @@ int mbox_save_init(struct mailbox_transaction_context *_t,
 		   enum mail_flags flags, struct mail_keywords *keywords,
 		   time_t received_date, int timezone_offset,
 		   const char *from_envelope, struct istream *input,
-		   bool want_mail, struct mail_save_context **ctx_r);
+		   struct mail *dest_mail, struct mail_save_context **ctx_r);
 int mbox_save_continue(struct mail_save_context *ctx);
-int mbox_save_finish(struct mail_save_context *ctx, struct mail *dest_mail);
+int mbox_save_finish(struct mail_save_context *ctx);
 void mbox_save_cancel(struct mail_save_context *ctx);
 
 int mbox_transaction_save_commit(struct mbox_save_context *ctx);
 void mbox_transaction_save_rollback(struct mbox_save_context *ctx);
 
-bool mbox_is_valid_mask(struct mail_storage *storage, const char *mask);
-
 #endif
