@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 Timo Sirainen */
+/* Copyright (c) 2004-2008 Dovecot authors, see the included COPYING file */
 
 /* MD5 header summing logic was pretty much copy&pasted from popa3d by
    Solar Designer */
@@ -109,24 +109,11 @@ static bool parse_x_status(struct mbox_sync_mail_context *ctx,
 	return TRUE;
 }
 
-static bool keyword_is_valid(const char *keyword)
-{
-	/* try to only prevent the most malicious looking keywords. */
-	for (; *keyword != '\0'; keyword++) {
-		if (*keyword == '(' || *keyword == ')' ||
-		    *keyword == '{' || *keyword == '}' ||
-		    *keyword == '\\' || *keyword == '"' ||
-		    (unsigned char)*keyword <= 32)
-			return FALSE;
-	}
-	return TRUE;
-}
-
 static void
 parse_imap_keywords_list(struct mbox_sync_mail_context *ctx,
                          struct message_header_line *hdr, size_t pos)
 {
-	const char *keyword;
+	const char *keyword, *error;
 	size_t keyword_start;
 	unsigned int idx, count;
 
@@ -145,16 +132,13 @@ parse_imap_keywords_list(struct mbox_sync_mail_context *ctx,
 		}
 
 		/* add it to index's keyword list if it's not there already */
-		t_push();
 		keyword = t_strndup(hdr->full_value + keyword_start,
 				    pos - keyword_start);
-		if (keyword_is_valid(keyword)) {
-			(void)mail_index_keyword_lookup(
-				ctx->sync_ctx->mbox->ibox.index,
-				keyword, TRUE, &idx);
+		if (index_mailbox_keyword_is_valid(&ctx->sync_ctx->mbox->ibox,
+						   keyword, &error)) {
+			mail_index_keyword_lookup_or_create(
+				ctx->sync_ctx->mbox->ibox.index, keyword, &idx);
 		}
-		t_pop();
-
 		count++;
 	}
 
@@ -240,7 +224,9 @@ static bool parse_x_imap_base(struct mbox_sync_mail_context *ctx,
 	ctx->hdr_pos[MBOX_HDR_X_IMAPBASE] = str_len(ctx->header);
 	ctx->seen_imapbase = TRUE;
 
-	parse_imap_keywords_list(ctx, hdr, i);
+	T_BEGIN {
+		parse_imap_keywords_list(ctx, hdr, i);
+	} T_END;
 	parse_trailing_whitespace(ctx, hdr);
 	return TRUE;
 }
@@ -257,10 +243,10 @@ static bool parse_x_imap(struct mbox_sync_mail_context *ctx,
 	return TRUE;
 }
 
-static bool parse_x_keywords(struct mbox_sync_mail_context *ctx,
-			     struct message_header_line *hdr)
+static bool parse_x_keywords_real(struct mbox_sync_mail_context *ctx,
+				  struct message_header_line *hdr)
 {
-	array_t ARRAY_DEFINE(keyword_list, unsigned int);
+	ARRAY_TYPE(keyword_indexes) keyword_list;
 	const unsigned int *list;
 	string_t *keyword;
 	size_t keyword_start;
@@ -271,9 +257,8 @@ static bool parse_x_keywords(struct mbox_sync_mail_context *ctx,
 		return FALSE; /* duplicate header, delete */
 
 	/* read keyword indexes to temporary array first */
-	t_push();
 	keyword = t_str_new(128);
-	ARRAY_CREATE(&keyword_list, pool_datastack_create(), unsigned int, 16);
+	t_array_init(&keyword_list, 16);
 
 	for (pos = 0; pos < hdr->full_value_len; ) {
 		if (IS_LWSP_LF(hdr->full_value[pos])) {
@@ -292,10 +277,9 @@ static bool parse_x_keywords(struct mbox_sync_mail_context *ctx,
 		str_append_n(keyword, hdr->full_value + keyword_start,
 			     pos - keyword_start);
 		if (!mail_index_keyword_lookup(ctx->sync_ctx->mbox->ibox.index,
-					       str_c(keyword), FALSE, &idx)) {
+					       str_c(keyword), &idx)) {
 			/* keyword wasn't found. that means the sent mail
 			   originally contained X-Keywords header. Delete it. */
-			t_pop();
 			return FALSE;
 		}
 
@@ -314,19 +298,28 @@ static bool parse_x_keywords(struct mbox_sync_mail_context *ctx,
 	/* once we know how many keywords there are, we can allocate the array
 	   from mail_keyword_pool without wasting memory. */
 	if (array_count(&keyword_list) > 0) {
-		ARRAY_CREATE(&ctx->mail.keywords,
+		p_array_init(&ctx->mail.keywords,
 			     ctx->sync_ctx->mail_keyword_pool,
-			     unsigned int, array_count(&keyword_list));
+			     array_count(&keyword_list));
 		array_append_array(&ctx->mail.keywords, &keyword_list);
 	}
 
 	ctx->hdr_pos[MBOX_HDR_X_KEYWORDS] = str_len(ctx->header);
 	parse_trailing_whitespace(ctx, hdr);
-
-	t_pop();
 	return TRUE;
 }
 
+static bool parse_x_keywords(struct mbox_sync_mail_context *ctx,
+			     struct message_header_line *hdr)
+{
+	bool ret;
+
+	T_BEGIN {
+		ret = parse_x_keywords_real(ctx, hdr);
+	} T_END;
+	return ret;
+}
+
 static bool parse_x_uid(struct mbox_sync_mail_context *ctx,
 			struct message_header_line *hdr)
 {
@@ -440,7 +433,6 @@ static struct mbox_sync_header_func header_funcs[] = {
 	{ "X-Status", parse_x_status },
 	{ "X-UID", parse_x_uid }
 };
-#define HEADER_FUNCS_COUNT (sizeof(header_funcs) / sizeof(*header_funcs))
 
 static int mbox_sync_bsearch_header_func_cmp(const void *p1, const void *p2)
 {
@@ -476,7 +468,7 @@ void mbox_sync_parse_next_mail(struct istream *input,
         mbox_md5_ctx = mbox_md5_init();
 
         line_start_pos = 0;
-	hdr_ctx = message_parse_header_init(input, NULL, FALSE);
+	hdr_ctx = message_parse_header_init(input, NULL, 0);
 	while ((ret = message_parse_header_next(hdr_ctx, &hdr)) > 0) {
 		if (hdr->eoh) {
 			ctx->have_eoh = TRUE;
@@ -490,7 +482,7 @@ void mbox_sync_parse_next_mail(struct istream *input,
 		}
 
 		func = bsearch(hdr->name, header_funcs,
-			       HEADER_FUNCS_COUNT, sizeof(*header_funcs),
+			       N_ELEMENTS(header_funcs), sizeof(*header_funcs),
 			       mbox_sync_bsearch_header_func_cmp);
 
 		if (func != NULL) {
@@ -544,8 +536,8 @@ void mbox_sync_parse_next_mail(struct istream *input,
 	ctx->body_offset = input->v_offset;
 }
 
-int mbox_sync_parse_match_mail(struct mbox_mailbox *mbox,
-			       struct mail_index_view *view, uint32_t seq)
+bool mbox_sync_parse_match_mail(struct mbox_mailbox *mbox,
+				struct mail_index_view *view, uint32_t seq)
 {
         struct mbox_sync_mail_context ctx;
 	struct message_header_parser_ctx *hdr_ctx;
@@ -562,21 +554,17 @@ int mbox_sync_parse_match_mail(struct mbox_mailbox *mbox,
 	   just not yet been rewritten. In that case use MD5 sum, if it
 	   exists. */
 
-	if (mail_index_lookup_uid(view, seq, &uid) < 0) {
-		mail_storage_set_index_error(&mbox->ibox);
-		return -1;
-	}
-
+	mail_index_lookup_uid(view, seq, &uid);
 	memset(&ctx, 0, sizeof(ctx));
         mbox_md5_ctx = mbox_md5_init();
 
-	hdr_ctx = message_parse_header_init(mbox->mbox_stream, NULL, FALSE);
+	hdr_ctx = message_parse_header_init(mbox->mbox_stream, NULL, 0);
 	while ((ret = message_parse_header_next(hdr_ctx, &hdr)) > 0) {
 		if (hdr->eoh)
 			break;
 
 		func = bsearch(hdr->name, header_funcs,
-			       HEADER_FUNCS_COUNT, sizeof(*header_funcs),
+			       N_ELEMENTS(header_funcs), sizeof(*header_funcs),
 			       mbox_sync_bsearch_header_func_cmp);
 		if (func != NULL) {
 			if (strcasecmp(hdr->name, "X-UID") == 0) {
@@ -604,11 +592,8 @@ int mbox_sync_parse_match_mail(struct mbox_mailbox *mbox,
 	/* match by MD5 sum */
 	mbox->mbox_save_md5 = TRUE;
 
-	if (mail_index_lookup_ext(view, seq, mbox->ibox.md5hdr_ext_idx,
-				  &data) < 0) {
-		mail_storage_set_index_error(&mbox->ibox);
-		return -1;
-	}
+	mail_index_lookup_ext(view, seq, mbox->ibox.md5hdr_ext_idx,
+			      &data, NULL);
 	return data == NULL ? 0 :
 		memcmp(data, ctx.hdr_md5_sum, 16) == 0;
 }
