@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 Timo Sirainen */
+/* Copyright (c) 2004-2008 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "istream.h"
@@ -12,23 +12,27 @@ int mail_storage_copy(struct mailbox_transaction_context *t, struct mail *mail,
 	struct mail_save_context *ctx;
 	struct istream *input;
 	const char *from_envelope;
+	time_t received_date;
 
-	input = mail_get_stream(mail, NULL, NULL);
-	if (input == NULL)
+	if (mail_get_stream(mail, NULL, NULL, &input) < 0)
+		return -1;
+	if (mail_get_received_date(mail, &received_date) < 0)
+		return -1;
+	if (mail_get_special(mail, MAIL_FETCH_FROM_ENVELOPE,
+			     &from_envelope) < 0)
 		return -1;
 
-	from_envelope = mail_get_special(mail, MAIL_FETCH_FROM_ENVELOPE);
+	if (*from_envelope == '\0')
+		from_envelope = NULL;
 
-	if (mailbox_save_init(t, flags, keywords,
-			      mail_get_received_date(mail),
-			      0, from_envelope, input, dest_mail != NULL,
-			      &ctx) < 0)
+	if (mailbox_save_init(t, flags, keywords, received_date,
+			      0, from_envelope, input, dest_mail, &ctx) < 0)
 		return -1;
 
-	while (i_stream_read(input) != -1) {
+	do {
 		if (mailbox_save_continue(ctx) < 0)
 			break;
-	}
+	} while (i_stream_read(input) != -1);
 
 	if (input->stream_errno != 0) {
 		mail_storage_set_critical(t->box->storage,
@@ -37,5 +41,5 @@ int mail_storage_copy(struct mailbox_transaction_context *t, struct mail *mail,
 		return -1;
 	}
 
-	return mailbox_save_finish(&ctx, dest_mail);
+	return mailbox_save_finish(&ctx);
 }
