@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2003 Timo Sirainen */
+/* Copyright (c) 2002-2008 Dovecot authors, see the included COPYING file */
 
 /* @UNSAFE: whole file */
 
@@ -41,14 +41,20 @@ buffer_check_limits(struct real_buffer *buf, size_t pos, size_t data_size)
 {
 	size_t new_size;
 
-	if ((size_t)-1 - pos < data_size) {
+	if (unlikely((size_t)-1 - pos < data_size)) {
 		i_panic("Buffer write out of range (%"PRIuSIZE_T
 			" + %"PRIuSIZE_T")", pos, data_size);
 	}
 	new_size = pos + data_size;
 
+	if (new_size > buf->used && buf->used < buf->dirty) {
+		/* clear used..dirty area */
+		size_t max = I_MIN(I_MIN(buf->alloc, buf->dirty), new_size);
+
+		memset(buf->w_buffer + buf->used, 0, max - buf->used);
+	}
 	if (new_size > buf->alloc) {
-		if (!buf->dynamic) {
+		if (unlikely(!buf->dynamic)) {
 			i_panic("Buffer full (%"PRIuSIZE_T" > %"PRIuSIZE_T", "
 				"pool %s)", pos + data_size, buf->alloc,
 				pool_get_name(buf->pool));
@@ -113,7 +119,7 @@ buffer_t *buffer_create_dynamic(pool_t pool, size_t init_size)
 	return (buffer_t *)buf;
 }
 
-void _buffer_free(buffer_t **_buf)
+void buffer_free(buffer_t **_buf)
 {
 	struct real_buffer *buf = (struct real_buffer *)*_buf;
 
@@ -123,7 +129,7 @@ void _buffer_free(buffer_t **_buf)
 	p_free(buf->pool, buf);
 }
 
-void *_buffer_free_without_data(buffer_t **_buf)
+void *buffer_free_without_data(buffer_t **_buf)
 {
 	struct real_buffer *buf = (struct real_buffer *)*_buf;
 	void *data;
@@ -135,13 +141,16 @@ void *_buffer_free_without_data(buffer_t **_buf)
 	return data;
 }
 
-void buffer_reset(buffer_t *_buf)
+pool_t buffer_get_pool(buffer_t *_buf)
 {
 	struct real_buffer *buf = (struct real_buffer *)_buf;
 
-	memset(buf->w_buffer, 0, I_MAX(buf->used, buf->dirty));
-	buf->dirty = 0;
-	buf->used = 0;
+	return buf->pool;
+}
+
+void buffer_reset(buffer_t *buf)
+{
+	buffer_set_used_size(buf, 0);
 }
 
 void buffer_write(buffer_t *_buf, size_t pos,
@@ -265,7 +274,7 @@ void *buffer_append_space_unsafe(buffer_t *buf, size_t size)
 	return buffer_get_space_unsafe(buf, buf->used, size);
 }
 
-void *buffer_get_modifyable_data(const buffer_t *_buf, size_t *used_size_r)
+void *buffer_get_modifiable_data(const buffer_t *_buf, size_t *used_size_r)
 {
 	const struct real_buffer *buf = (const struct real_buffer *)_buf;
 
@@ -280,7 +289,7 @@ void buffer_set_used_size(buffer_t *_buf, size_t used_size)
 
 	i_assert(used_size <= buf->alloc);
 
-	if (used_size < buf->used && buf->used > buf->dirty)
+	if (buf->used > buf->dirty)
 		buf->dirty = buf->used;
 
 	buf->used = used_size;
