@@ -1,4 +1,4 @@
-/* Copyright (c) 2004 Timo Sirainen */
+/* Copyright (c) 2004-2008 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "buffer.h"
@@ -36,7 +36,7 @@ void file_cache_free(struct file_cache **_cache)
 		if (munmap_anon(cache->mmap_base, cache->mmap_length) < 0)
 			i_error("munmap_anon() failed: %m");
 	}
-	buffer_free(cache->page_bitmask);
+	buffer_free(&cache->page_bitmask);
 	i_free(cache);
 }
 
@@ -269,8 +269,15 @@ void file_cache_invalidate(struct file_cache *cache, uoff_t offset, uoff_t size)
 	if (offset >= cache->read_highwater || size == 0)
 		return;
 
-	if (size > cache->read_highwater - offset)
+	if (size > cache->read_highwater - offset) {
+		/* ignore anything after read highwater */
 		size = cache->read_highwater - offset;
+	}
+	if (size >= cache->read_highwater) {
+		/* we're invalidating everything up to read highwater.
+		   drop the highwater position. */
+		cache->read_highwater = offset & ~(page_size-1);
+	}
 
 	size = (offset + size + page_size-1) / page_size;
 	offset /= page_size;
