@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Timo Sirainen */
+/* Copyright (c) 2003-2008 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -56,7 +56,7 @@ const char *module_file_get_name(const char *fname)
 	if (*p == '_')
 		fname = p + 1;
 
-	p = strstr(fname, ".so");
+	p = strstr(fname, MODULE_SUFFIX);
 	if (p == NULL)
 		return fname;
 
@@ -166,13 +166,12 @@ static bool module_want_load(const char **names, const char *name)
 	return FALSE;
 }
 
-static void check_duplicates(array_t *names, const char *name, const char *dir)
+static void check_duplicates(ARRAY_TYPE(const_string) *names,
+			     const char *name, const char *dir)
 {
-	ARRAY_SET_TYPE(names, const char *);
 	const char *const *names_p, *base_name, *tmp;
 	unsigned int i, count;
 
-	t_push();
 	base_name = module_file_get_name(name);
 	names_p = array_get(names, &count);
 	for (i = 0; i < count; i++) {
@@ -182,19 +181,19 @@ static void check_duplicates(array_t *names, const char *name, const char *dir)
 			i_fatal("Multiple files for module %s: %s/%s, %s/%s",
 				base_name, dir, name, dir, names_p[i]);
 	}
-	t_pop();
 }
 
-struct module *module_dir_load(const char *dir, const char *module_names,
-			       bool require_init_funcs, const char *version)
+static struct module *
+module_dir_load_real(const char *dir, const char *module_names,
+		     bool require_init_funcs, const char *version)
 {
 	DIR *dirp;
 	struct dirent *d;
-	const char *name, *path, *p, *stripped_name, **names_p;
+	const char *name, *p, **names_p;
 	const char **module_names_arr;
 	struct module *modules, *module, **module_pos;
 	unsigned int i, count;
-	array_t ARRAY_DEFINE(names, const char *);
+	ARRAY_TYPE(const_string) names;
 	pool_t pool;
 
 	if (getenv("DEBUG") != NULL)
@@ -213,7 +212,7 @@ struct module *module_dir_load(const char *dir, const char *module_names,
 	}
 
 	pool = pool_alloconly_create("module loader", 1024);
-	ARRAY_CREATE(&names, pool, const char *, 32);
+	p_array_init(&names, pool, 32);
 
 	modules = NULL;
 	while ((d = readdir(dirp)) != NULL) {
@@ -222,21 +221,21 @@ struct module *module_dir_load(const char *dir, const char *module_names,
 		if (name[0] == '.')
 			continue;
 
-		p = strstr(name, ".so");
+		p = strstr(name, MODULE_SUFFIX);
 		if (p == NULL || strlen(p) != 3)
 			continue;
 
-		check_duplicates(&names, name, dir);
+		T_BEGIN {
+			check_duplicates(&names, name, dir);
+		} T_END;
 
 		name = p_strdup(pool, d->d_name);
 		array_append(&names, &name, 1);
 	}
 
-	names_p = array_get_modifyable(&names, NULL);
-	count = array_count(&names);
+	names_p = array_get_modifiable(&names, &count);
 	qsort(names_p, count, sizeof(const char *), module_name_cmp);
 
-	t_push();
 	if (module_names == NULL)
 		module_names_arr = NULL;
 	else {
@@ -250,10 +249,10 @@ struct module *module_dir_load(const char *dir, const char *module_names,
 	}
 
 	module_pos = &modules;
-	for (i = 0; i < count; i++) {
-		const char *name = names_p[i];
+	for (i = 0; i < count; i++) T_BEGIN {
+		const char *path, *stripped_name;
 
-		t_push();
+		name = names_p[i];
 		stripped_name = module_file_get_name(name);
 		if (!module_want_load(module_names_arr, stripped_name))
 			module = NULL;
@@ -264,13 +263,13 @@ struct module *module_dir_load(const char *dir, const char *module_names,
 			if (module == NULL && module_names_arr != NULL)
 				i_fatal("Couldn't load required plugins");
 		}
-		t_pop();
 
 		if (module != NULL) {
 			*module_pos = module;
 			module_pos = &module->next;
 		}
-	}
+	} T_END;
+
 	if (module_names_arr != NULL) {
 		/* make sure all modules were found */
 		for (; *module_names_arr != NULL; module_names_arr++) {
@@ -280,8 +279,7 @@ struct module *module_dir_load(const char *dir, const char *module_names,
 			}
 		}
 	}
-	t_pop();
-	pool_unref(pool);
+	pool_unref(&pool);
 
 	if (closedir(dirp) < 0)
 		i_error("closedir(%s) failed: %m", dir);
@@ -289,13 +287,28 @@ struct module *module_dir_load(const char *dir, const char *module_names,
 	return modules;
 }
 
+struct module *module_dir_load(const char *dir, const char *module_names,
+			       bool require_init_funcs, const char *version)
+{
+	struct module *modules;
+
+	T_BEGIN {
+		modules = module_dir_load_real(dir, module_names,
+					       require_init_funcs, version);
+	} T_END;
+	return modules;
+}
+
 void module_dir_init(struct module *modules)
 {
 	struct module *module;
 
 	for (module = modules; module != NULL; module = module->next) {
-		if (module->init != NULL)
-			module->init();
+		if (module->init != NULL) {
+			T_BEGIN {
+				module->init();
+			} T_END;
+		}
 	}
 }
 
@@ -311,20 +324,22 @@ void module_dir_deinit(struct module *modules)
 		return;
 
 	/* @UNSAFE: deinitialize in reverse order */
-	t_push();
-	rev = t_new(struct module *, count);
-	for (i = 0, module = modules; i < count; i++, module = module->next)
-		rev[count-i-1] = module;
-
-	for (i = 0; i < count; i++) {
-		module = rev[i];
-
-		if (module->deinit != NULL) {
-			module->deinit();
-			module->deinit = NULL;
+	T_BEGIN {
+		rev = t_new(struct module *, count);
+		for (i = 0, module = modules; i < count; i++) {
+			rev[count-i-1] = module;
+			module = module->next;
 		}
-	}
-	t_pop();
+
+		for (i = 0; i < count; i++) {
+			module = rev[i];
+
+			if (module->deinit != NULL) {
+				module->deinit();
+				module->deinit = NULL;
+			}
+		}
+	} T_END;
 }
 
 void module_dir_unload(struct module **modules)
@@ -345,19 +360,19 @@ void module_dir_unload(struct module **modules)
 
 #else
 
-struct module *module_dir_load(const char *dir __attr_unused__,
-			       const char *module_names __attr_unused__,
-			       bool require_init_funcs __attr_unused__)
+struct module *module_dir_load(const char *dir ATTR_UNUSED,
+			       const char *module_names ATTR_UNUSED,
+			       bool require_init_funcs ATTR_UNUSED)
 {
 	i_error("Dynamically loadable module support not built in");
 	return NULL;
 }
 
-void module_dir_deinit(struct module *modules __attr_unused__)
+void module_dir_deinit(struct module *modules ATTR_UNUSED)
 {
 }
 
-void module_dir_unload(struct module **modules __attr_unused__)
+void module_dir_unload(struct module **modules ATTR_UNUSED)
 {
 }
 
