@@ -1,11 +1,12 @@
-#ifndef __MASTER_H
-#define __MASTER_H
+#ifndef MASTER_H
+#define MASTER_H
 
 struct client;
 
 #include "../master/master-login-interface.h"
 
-typedef void master_callback_t(struct client *client, bool success);
+typedef void master_callback_t(struct client *client,
+			       enum master_login_status status);
 
 void master_request_login(struct client *client, master_callback_t *callback,
 			  unsigned int auth_pid, unsigned int auth_id);
