@@ -19,6 +19,7 @@
 
 #include <stdlib.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <pwd.h>
 #include <syslog.h>
 
@@ -125,6 +126,17 @@ auth_process_input_user(struct auth_process *process, const char *args)
 		return FALSE;
 	}
 
+	if (!auth_success_written) {
+		int fd;
+
+		fd = creat(AUTH_SUCCESS_PATH, 0666);
+		if (fd == -1)
+			i_error("creat(%s) failed: %m", AUTH_SUCCESS_PATH);
+		else
+			(void)close(fd);
+		auth_success_written = TRUE;
+	}
+
 	auth_master_callback(list[1], list + 2, request);
 	hash_remove(process->requests, POINTER_CAST(id));
 	return TRUE;
