@@ -476,6 +476,11 @@ static bool auth_settings_verify(struct auth_settings *auth)
 	}
 
 	for (s = auth->sockets; s != NULL; s = s->next) {
+		if (auth->count > 1 && strcmp(s->type, "listen") == 0) {
+			i_error("Currently auth process count must be 1 if "
+				"you're using auth socket listeners.");
+			return FALSE;
+		}
 		fix_base_path(auth->parent->defaults, &s->master.path);
 		fix_base_path(auth->parent->defaults, &s->client.path);
 	}
@@ -609,6 +614,7 @@ static bool get_imap_capability(struct settings *set)
 	static const char *args[] = {
 		"uid=65534",
 		"gid=65534",
+		"home=/tmp",
 		NULL
 	};
 	enum master_login_status login_status;
