@@ -1,4 +1,4 @@
-/* Copyright (C) 2006 Timo Sirainen */
+/* Copyright (c) 2006-2008 Dovecot authors, see the included COPYING file */
 
 /* FIXME: If we don't have permission to change flags/keywords, the changes
    should still be stored temporarily for this session. However most clients
@@ -7,34 +7,28 @@
 #include "lib.h"
 #include "array.h"
 #include "istream.h"
-#include "mail-storage-private.h"
+#include "mailbox-list-private.h"
 #include "acl-api-private.h"
 #include "acl-plugin.h"
 
 #include <sys/stat.h>
 
-#define ACL_CONTEXT(obj) \
-	*((void **)array_idx_modifyable(&(obj)->module_contexts, \
-					acl_storage_module_id))
+#define ACL_MAIL_CONTEXT(obj) \
+	MODULE_CONTEXT(obj, acl_mail_module)
 
 struct acl_mailbox {
-	struct mailbox_vfuncs super;
+	union mailbox_module_context module_ctx;
 	struct acl_object *aclobj;
 
 	unsigned int save_hack:1;
 };
 
-struct acl_mail {
-	struct mail_vfuncs super;
+struct acl_transaction_context {
+	union mailbox_transaction_module_context module_ctx;
 };
 
-static int acl_mailbox_close(struct mailbox *box)
-{
-	struct acl_mailbox *abox = ACL_CONTEXT(box);
-
-	acl_object_deinit(&abox->aclobj);
-	return abox->super.close(box);
-}
+static MODULE_CONTEXT_DEFINE_INIT(acl_mail_module, &mail_module_register);
+static struct acl_transaction_context acl_transaction_failure;
 
 static int mailbox_acl_right_lookup(struct mailbox *box, unsigned int right_idx)
 {
@@ -43,16 +37,62 @@ static int mailbox_acl_right_lookup(struct mailbox *box, unsigned int right_idx)
 	int ret;
 
 	ret = acl_object_have_right(abox->aclobj,
-				    astorage->acl_storage_right_idx[right_idx]);
+			astorage->rights.acl_storage_right_idx[right_idx]);
 	if (ret > 0)
 		return 1;
-	if (ret < 0)
+	if (ret < 0) {
+		mail_storage_set_internal_error(box->storage);
 		return -1;
+	}
 
-	mail_storage_set_error(box->storage, MAIL_STORAGE_ERR_NO_PERMISSION);
+	mail_storage_set_error(box->storage, MAIL_ERROR_PERM,
+			       MAIL_ERRSTR_NO_PERMISSION);
 	return 0;
 }
 
+static bool acl_is_readonly(struct mailbox *box)
+{
+	struct acl_mailbox *abox = ACL_CONTEXT(box);
+
+	if (abox->module_ctx.super.is_readonly(box))
+		return TRUE;
+
+	if (mailbox_acl_right_lookup(box, ACL_STORAGE_RIGHT_INSERT) > 0)
+		return FALSE;
+	if (mailbox_acl_right_lookup(box, ACL_STORAGE_RIGHT_EXPUNGE) > 0)
+		return FALSE;
+
+	/* Next up is the "shared flag rights" */
+	if (mailbox_acl_right_lookup(box, ACL_STORAGE_RIGHT_WRITE) > 0)
+		return FALSE;
+	if ((box->private_flags_mask & MAIL_DELETED) == 0 &&
+	    mailbox_acl_right_lookup(box, ACL_STORAGE_RIGHT_WRITE_DELETED) > 0)
+		return FALSE;
+	if ((box->private_flags_mask & MAIL_SEEN) == 0 &&
+	    mailbox_acl_right_lookup(box, ACL_STORAGE_RIGHT_WRITE_SEEN) > 0)
+		return FALSE;
+
+	return TRUE;
+}
+
+static bool acl_allow_new_keywords(struct mailbox *box)
+{
+	struct acl_mailbox *abox = ACL_CONTEXT(box);
+
+	if (!abox->module_ctx.super.allow_new_keywords(box))
+		return FALSE;
+
+	return mailbox_acl_right_lookup(box, ACL_STORAGE_RIGHT_WRITE) > 0;
+}
+
+static int acl_mailbox_close(struct mailbox *box)
+{
+	struct acl_mailbox *abox = ACL_CONTEXT(box);
+
+	acl_object_deinit(&abox->aclobj);
+	return abox->module_ctx.super.close(box);
+}
+
 static int
 acl_get_write_rights(struct mailbox *box,
 		     bool *flags_r, bool *flag_seen_r, bool *flag_del_r)
@@ -76,17 +116,25 @@ acl_get_write_rights(struct mailbox *box,
 	return 0;
 }
 
-static int
+static void acl_transaction_set_failure(struct mailbox_transaction_context *t)
+{
+	MODULE_CONTEXT_SET(t, acl_storage_module,
+			   &acl_transaction_failure);
+}
+
+static void
 acl_mail_update_flags(struct mail *_mail, enum modify_type modify_type,
 		      enum mail_flags flags)
 {
 	struct mail_private *mail = (struct mail_private *)_mail;
-	struct acl_mail *amail = ACL_CONTEXT(mail);
+	union mail_module_context *amail = ACL_MAIL_CONTEXT(mail);
 	bool acl_flags, acl_flag_seen, acl_flag_del;
 
 	if (acl_get_write_rights(_mail->box, &acl_flags, &acl_flag_seen,
-				 &acl_flag_del) < 0)
-		return -1;
+				 &acl_flag_del) < 0) {
+		acl_transaction_set_failure(_mail->transaction);
+		return;
+	}
 
 	if (modify_type != MODIFY_REPLACE) {
 		/* adding/removing flags. just remove the disallowed
@@ -101,41 +149,43 @@ acl_mail_update_flags(struct mail *_mail, enum modify_type modify_type,
 		/* we don't have permission to replace all the flags. */
 		if (!acl_flags && !acl_flag_seen && !acl_flag_del) {
 			/* no flag changes allowed. ignore silently. */
-			return 0;
+			return;
 		}
 
 		/* handle this by first removing the allowed flags and
 		   then adding the allowed flags */
-		if (acl_mail_update_flags(_mail, MODIFY_REMOVE,
-					  ~flags) < 0)
-			return -1;
-		return acl_mail_update_flags(_mail, MODIFY_ADD, flags);
+		acl_mail_update_flags(_mail, MODIFY_REMOVE, ~flags);
+		if (flags != 0)
+			acl_mail_update_flags(_mail, MODIFY_ADD, flags);
+		return;
 	}
 
-	return amail->super.update_flags(_mail, modify_type, flags);
+	amail->super.update_flags(_mail, modify_type, flags);
 }
 
-static int
+static void
 acl_mail_update_keywords(struct mail *_mail, enum modify_type modify_type,
 			 struct mail_keywords *keywords)
 {
 	struct mail_private *mail = (struct mail_private *)_mail;
-	struct acl_mail *amail = ACL_CONTEXT(mail);
+	union mail_module_context *amail = ACL_MAIL_CONTEXT(mail);
 	int ret;
 
 	ret = mailbox_acl_right_lookup(_mail->box, ACL_STORAGE_RIGHT_WRITE);
 	if (ret <= 0) {
 		/* if we don't have permission, just silently return success. */
-		return ret;
+		if (ret < 0)
+			acl_transaction_set_failure(_mail->transaction);
+		return;
 	}
 
-	return amail->super.update_keywords(_mail, modify_type, keywords);
+	amail->super.update_keywords(_mail, modify_type, keywords);
 }
 
-static int acl_mail_expunge(struct mail *_mail)
+static void acl_mail_expunge(struct mail *_mail)
 {
 	struct mail_private *mail = (struct mail_private *)_mail;
-	struct acl_mail *amail = ACL_CONTEXT(mail);
+	union mail_module_context *amail = ACL_MAIL_CONTEXT(mail);
 	int ret;
 
 	ret = mailbox_acl_right_lookup(_mail->box, ACL_STORAGE_RIGHT_EXPUNGE);
@@ -143,10 +193,12 @@ static int acl_mail_expunge(struct mail *_mail)
 		/* if we don't have permission, silently return success so
 		   users won't see annoying error messages in case their
 		   clients try automatic expunging. */
-		return ret;
+		if (ret < 0)
+			acl_transaction_set_failure(_mail->transaction);
+		return;
 	}
 
-	return amail->super.expunge(_mail);
+	amail->super.expunge(_mail);
 }
 
 static struct mail *
@@ -155,20 +207,21 @@ acl_mail_alloc(struct mailbox_transaction_context *t,
 	       struct mailbox_header_lookup_ctx *wanted_headers)
 {
 	struct acl_mailbox *abox = ACL_CONTEXT(t->box);
-	struct acl_mail *amail;
+	union mail_module_context *amail;
 	struct mail *_mail;
 	struct mail_private *mail;
 
-	_mail = abox->super.mail_alloc(t, wanted_fields, wanted_headers);
+	_mail = abox->module_ctx.super.
+		mail_alloc(t, wanted_fields, wanted_headers);
 	mail = (struct mail_private *)_mail;
 
-	amail = p_new(mail->pool, struct acl_mail, 1);
+	amail = p_new(mail->pool, union mail_module_context, 1);
 	amail->super = mail->v;
 
 	mail->v.update_flags = acl_mail_update_flags;
 	mail->v.update_keywords = acl_mail_update_keywords;
 	mail->v.expunge = acl_mail_expunge;
-	array_idx_set(&mail->module_contexts, acl_storage_module_id, &amail);
+	MODULE_CONTEXT_SET_SELF(mail, acl_mail_module, amail);
 	return _mail;
 }
 
@@ -197,7 +250,7 @@ acl_save_init(struct mailbox_transaction_context *t,
 	      enum mail_flags flags, struct mail_keywords *keywords,
 	      time_t received_date, int timezone_offset,
 	      const char *from_envelope, struct istream *input,
-	      bool want_mail, struct mail_save_context **ctx_r)
+	      struct mail *dest_mail, struct mail_save_context **ctx_r)
 {
 	struct acl_mailbox *abox = ACL_CONTEXT(t->box);
 
@@ -206,9 +259,10 @@ acl_save_init(struct mailbox_transaction_context *t,
 	if (acl_save_get_flags(t->box, &flags, &keywords) < 0)
 		return -1;
 
-	return abox->super.save_init(t, flags, keywords, received_date,
-				     timezone_offset, from_envelope,
-				     input, want_mail, ctx_r);
+	return abox->module_ctx.super.
+		save_init(t, flags, keywords, received_date,
+			  timezone_offset, from_envelope,
+			  input, dest_mail, ctx_r);
 }
 
 static int
@@ -223,7 +277,54 @@ acl_copy(struct mailbox_transaction_context *t, struct mail *mail,
 	if (acl_save_get_flags(t->box, &flags, &keywords) < 0)
 		return -1;
 
-	return abox->super.copy(t, mail, flags, keywords, dest_mail);
+	return abox->module_ctx.super.copy(t, mail, flags, keywords, dest_mail);
+}
+
+static int
+acl_transaction_commit(struct mailbox_transaction_context *ctx,
+		       uint32_t *uid_validity_r,
+		       uint32_t *first_saved_uid_r, uint32_t *last_saved_uid_r)
+{
+	struct acl_mailbox *abox = ACL_CONTEXT(ctx->box);
+	void *at = ACL_CONTEXT(ctx);
+
+	if (at != NULL) {
+		abox->module_ctx.super.transaction_rollback(ctx);
+		return -1;
+	}
+
+	return abox->module_ctx.super.
+		transaction_commit(ctx, uid_validity_r,
+				   first_saved_uid_r, last_saved_uid_r);
+}
+
+static int
+acl_keywords_create(struct mailbox *box, const char *const keywords[],
+		    struct mail_keywords **keywords_r, bool skip_invalid)
+{
+	struct acl_mailbox *abox = ACL_CONTEXT(box);
+	int ret;
+
+	ret = mailbox_acl_right_lookup(box, ACL_STORAGE_RIGHT_WRITE);
+	if (ret < 0) {
+		if (!skip_invalid)
+			return -1;
+		/* we can't return failure. assume we don't have permissions. */
+		ret = 0;
+	}
+
+	if (ret == 0) {
+		/* no permission to update any flags. just return empty
+		   keywords list. */
+		const char *null = NULL;
+
+		return abox->module_ctx.super.keywords_create(box, &null,
+							      keywords_r,
+							      skip_invalid);
+	}
+
+	return abox->module_ctx.super.keywords_create(box, keywords,
+						      keywords_r, skip_invalid);
 }
 
 struct mailbox *acl_mailbox_open_box(struct mailbox *box)
@@ -232,14 +333,19 @@ struct mailbox *acl_mailbox_open_box(struct mailbox *box)
 	struct acl_mailbox *abox;
 
 	abox = p_new(box->pool, struct acl_mailbox, 1);
-	abox->super = box->v;
-	abox->aclobj = acl_object_init_from_name(astorage->backend,
+	abox->module_ctx.super = box->v;
+	abox->aclobj = acl_object_init_from_name(astorage->rights.backend,
+						 box->storage,
 						 mailbox_get_name(box));
 	
+	box->v.is_readonly = acl_is_readonly;
+	box->v.allow_new_keywords = acl_allow_new_keywords;
 	box->v.close = acl_mailbox_close;
 	box->v.mail_alloc = acl_mail_alloc;
 	box->v.save_init = acl_save_init;
+	box->v.keywords_create = acl_keywords_create;
 	box->v.copy = acl_copy;
-	array_idx_set(&box->module_contexts, acl_storage_module_id, &abox);
+	box->v.transaction_commit = acl_transaction_commit;
+	MODULE_CONTEXT_SET(box, acl_storage_module, abox);
 	return box;
 }
