@@ -530,9 +530,6 @@ lazy_expunge_hook_mail_namespaces_created(struct mail_namespace *namespaces)
 	const char *const *p;
 	int i;
 
-	if (lazy_expunge_next_hook_mail_namespaces_created != NULL)
-		lazy_expunge_next_hook_mail_namespaces_created(namespaces);
-
 	p = t_strsplit_spaces(getenv("LAZY_EXPUNGE"), " ");
 	for (i = 0; i < LAZY_NAMESPACE_COUNT; i++, p++) {
 		const char *name = *p;
@@ -554,12 +551,20 @@ lazy_expunge_hook_mail_namespaces_created(struct mail_namespace *namespaces)
 		lstorage = LAZY_EXPUNGE_CONTEXT(lazy_namespaces[i]->storage);
 		lstorage->internal_namespace = TRUE;
 	}
+
+	if (lazy_expunge_next_hook_mail_namespaces_created != NULL)
+		lazy_expunge_next_hook_mail_namespaces_created(namespaces);
 }
 
 void lazy_expunge_plugin_init(void)
 {
-	if (getenv("LAZY_EXPUNGE") == NULL)
+	if (getenv("LAZY_EXPUNGE") == NULL) {
+		if (getenv("DEBUG") != NULL) {
+			i_info("lazy_expunge: No lazy_expunge setting - "
+			       "plugin disabled");
+		}
 		return;
+	}
 
 	lazy_expunge_next_hook_mail_namespaces_created =
 		hook_mail_namespaces_created;
@@ -575,7 +580,7 @@ void lazy_expunge_plugin_init(void)
 
 void lazy_expunge_plugin_deinit(void)
 {
-	if (getenv("LAZY_EXPUNGE") == NULL)
+	if (getenv("LAZY_EXPUNGE") != NULL)
 		return;
 
 	hook_mail_namespaces_created =
