@@ -253,20 +253,24 @@ static int quota_save_finish(struct mail_save_context *ctx)
 	return quota_check(ctx->transaction, ctx->dest_mail);
 }
 
-static void quota_mailbox_sync_finish(struct quota_mailbox *qbox)
+static void quota_mailbox_sync_cleanup(struct quota_mailbox *qbox)
 {
 	if (array_is_created(&qbox->expunge_uids)) {
 		array_clear(&qbox->expunge_uids);
 		array_clear(&qbox->expunge_sizes);
 	}
 
-	if (qbox->expunge_qt != NULL) {
-		if (qbox->expunge_qt->tmp_mail != NULL) {
-			mail_free(&qbox->expunge_qt->tmp_mail);
-			mailbox_transaction_rollback(&qbox->expunge_trans);
-		}
-		(void)quota_transaction_commit(&qbox->expunge_qt);
+	if (qbox->expunge_qt != NULL && qbox->expunge_qt->tmp_mail != NULL) {
+		mail_free(&qbox->expunge_qt->tmp_mail);
+		mailbox_transaction_rollback(&qbox->expunge_trans);
 	}
+}
+
+static void quota_mailbox_sync_commit(struct quota_mailbox *qbox)
+{
+	quota_mailbox_sync_cleanup(qbox);
+	if (qbox->expunge_qt != NULL)
+		(void)quota_transaction_commit(&qbox->expunge_qt);
 	qbox->recalculate = FALSE;
 }
 
@@ -283,8 +287,11 @@ static void quota_mailbox_sync_notify(struct mailbox *box, uint32_t uid,
 		qbox->module_ctx.super.sync_notify(box, uid, sync_type);
 
 	if (sync_type != MAILBOX_SYNC_TYPE_EXPUNGE || qbox->recalculate) {
-		if (uid == 0)
-			quota_mailbox_sync_finish(qbox);
+		if (uid == 0) {
+			/* free the transaction before view syncing begins,
+			   otherwise it'll crash. */
+			quota_mailbox_sync_cleanup(qbox);
+		}
 		return;
 	}
 
@@ -334,11 +341,14 @@ static int quota_mailbox_sync_deinit(struct mailbox_sync_context *ctx,
 				     struct mailbox_status *status_r)
 {
 	struct quota_mailbox *qbox = QUOTA_CONTEXT(ctx->box);
+	int ret;
 
-	/* just in case sync_notify() wasn't called with uid=0 */
-	quota_mailbox_sync_finish(qbox);
-
-	return qbox->module_ctx.super.sync_deinit(ctx, status_items, status_r);
+	ret = qbox->module_ctx.super.sync_deinit(ctx, status_items, status_r);
+	/* update quota only after syncing is finished. the quota commit may
+	   recalculate the quota and cause all mailboxes to be synced,
+	   including the one we're already syncing. */
+	quota_mailbox_sync_commit(qbox);
+	return ret;
 }
 
 static int quota_mailbox_close(struct mailbox *box)
