@@ -253,6 +253,7 @@ static int read_configuration(const char *path)
 	p_array_init(&trash_boxes, config_pool, INIT_TRASH_MAILBOX_COUNT);
 
 	input = i_stream_create_fd(fd, (size_t)-1, FALSE);
+	i_stream_set_return_partial_line(input, TRUE);
 	while ((line = i_stream_read_next_line(input)) != NULL) {
 		/* <priority> <mailbox name> */
 		name = strchr(line, ' ');
@@ -288,30 +289,26 @@ trash_hook_mail_namespaces_created(struct mail_namespace *namespaces)
 {
 	const char *env;
 
-	if (trash_next_hook_mail_namespaces_created != NULL)
-		trash_next_hook_mail_namespaces_created(namespaces);
-
 	env = getenv("TRASH");
 	if (env == NULL) {
 		if (getenv("DEBUG") != NULL)
-			i_info("trash plugin: No trash setting, disabled");
-		return;
-	}
-
-	if (quota_set == NULL) {
+			i_info("trash: No trash setting - plugin disabled");
+	} else if (quota_set == NULL) {
 		i_error("trash plugin: quota plugin not initialized");
-		return;
-	}
-
-	config_pool = pool_alloconly_create("trash config",
+	} else {
+		config_pool = pool_alloconly_create("trash config",
 					sizeof(trash_boxes) +
 					BUFFER_APPROX_SIZE +
 					INIT_TRASH_MAILBOX_COUNT *
 					(sizeof(struct trash_mailbox) + 32));
-	if (read_configuration(env) == 0) {
-		trash_next_quota_test_alloc = quota_set->test_alloc;
-		quota_set->test_alloc = trash_quota_test_alloc;
+		if (read_configuration(env) == 0) {
+			trash_next_quota_test_alloc = quota_set->test_alloc;
+			quota_set->test_alloc = trash_quota_test_alloc;
+		}
 	}
+
+	if (trash_next_hook_mail_namespaces_created != NULL)
+		trash_next_hook_mail_namespaces_created(namespaces);
 }
 
 void trash_plugin_init(void)
