@@ -1,5 +1,5 @@
-#ifndef __TRASH_PLUGIN_H
-#define __TRASH_PLUGIN_H
+#ifndef TRASH_PLUGIN_H
+#define TRASH_PLUGIN_H
 
 void trash_plugin_init(void);
 void trash_plugin_deinit(void);
