@@ -195,15 +195,18 @@ void client_input(struct pop3_client *client)
 {
 	char *line, *args;
 
-	timeout_reset(client->to_idle_disconnect);
+	i_assert(!client->common.authenticating);
 
+	timeout_reset(client->to_idle_disconnect);
 	if (!client_read(client))
 		return;
 
 	client_ref(client);
 
 	o_stream_cork(client->output);
-	while (!client->output->closed &&
+	/* if a command starts an authentication, stop processing further
+	   commands until the authentication is finished. */
+	while (!client->output->closed && !client->common.authenticating &&
 	       (line = i_stream_next_line(client->input)) != NULL) {
 		args = strchr(line, ' ');
 		if (args != NULL)
